@@ -15,23 +15,23 @@ my class Any { # declared in BOOTSTRAP
     # my class Any is Mu
 
     multi method ACCEPTS(Any:D: Mu:D \a) { self === a }
-    multi method ACCEPTS(Any:D: Mu:U \a) { False }
+    multi method ACCEPTS(Any:D: Mu:U $ --> False) { }
     multi method ACCEPTS(Any:U: Any \topic) { # use of Any on topic to force autothreading
         nqp::p6bool(nqp::istype(topic, self)) # so that all(@foo) ~~ Type works as expected
     }
 
     proto method EXISTS-KEY(|) is nodal { * }
-    multi method EXISTS-KEY(Any:U: $) { False }
-    multi method EXISTS-KEY(Any:D: $) { False }
+    multi method EXISTS-KEY(Any:U: $ --> False) { }
+    multi method EXISTS-KEY(Any:D: $ --> False) { }
 
     proto method DELETE-KEY(|) is nodal { * }
-    multi method DELETE-KEY(Any:U: $) { Nil }
+    multi method DELETE-KEY(Any:U: $ --> Nil) { }
     multi method DELETE-KEY(Any:D: $) {
         Failure.new("Can not remove values from a {self.^name}")
     }
 
     proto method DELETE-POS(|) is nodal { * }
-    multi method DELETE-POS(Any:U: $pos) { Nil }
+    multi method DELETE-POS(Any:U: $pos --> Nil) { }
     multi method DELETE-POS(Any:D: $pos) {
         Failure.new("Can not remove elements from a {self.^name}")
     }
@@ -65,6 +65,8 @@ my class Any { # declared in BOOTSTRAP
     multi method Slip() { self.list.Slip }
     proto method Array(|) is nodal { * }
     multi method Array() { self.list.Array }
+    proto method Seq(|) is nodal { * }
+    multi method Seq() { self.list.Seq }
 
     proto method hash(|) is nodal { * }
     multi method hash(Any:U:) { my % = () }
@@ -78,11 +80,11 @@ my class Any { # declared in BOOTSTRAP
     multi method Map() { self.hash.Map }
 
     proto method elems(|) is nodal { * }
-    multi method elems(Any:U:) { 1 }
+    multi method elems(Any:U: --> 1) { }
     multi method elems(Any:D:) { self.list.elems }
 
     proto method end(|) is nodal { * }
-    multi method end(Any:U:) { 0 }
+    multi method end(Any:U: --> 0) { }
     multi method end(Any:D:) { self.list.end }
 
     proto method keys(|) is nodal { * }
@@ -106,6 +108,8 @@ my class Any { # declared in BOOTSTRAP
     multi method antipairs(Any:D:) { self.list.antipairs }
 
     proto method invert(|) is nodal { * }
+    multi method invert(Any:U:) { () }
+    multi method invert(Any:D:) { self.list.invert }
 
     proto method pick(|) is nodal { * }
     multi method pick()   { self.list.pick     }
@@ -115,9 +119,10 @@ my class Any { # declared in BOOTSTRAP
     multi method roll()   { self.list.roll     }
     multi method roll($n) { self.list.roll($n) }
 
-    proto method iterator(|) { * }
     multi method iterator(Any:) { self.list.iterator }
 
+    method match(Any:U: |) { self.Str; nqp::getlexcaller('$/') = Nil }
+
     proto method classify(|) is nodal { * }
     multi method classify() {
         die "Must specify something to classify with, a Callable, Hash or List";
@@ -146,7 +151,6 @@ my class Any { # declared in BOOTSTRAP
         Hash.^parameterize(Any,Any).new.categorize-list($test, self.list, :&as);
     }
 
-    method rotor(|c) is nodal { self.list.rotor(|c) }
     method reverse() is nodal { self.list.reverse }
     method combinations(|c) is nodal { self.list.combinations(|c) }
     method permutations(|c) is nodal { self.list.permutations(|c) }
@@ -205,7 +209,7 @@ my class Any { # declared in BOOTSTRAP
     }
 
     proto method EXISTS-POS(|) is nodal { * }
-    multi method EXISTS-POS(Any:U: Any:D $) { False }
+    multi method EXISTS-POS(Any:U: Any:D $ --> False) { }
     multi method EXISTS-POS(Any:U: Any:U $pos) {
         die "Cannot use '{$pos.^name}' as an index";
     }
@@ -241,16 +245,30 @@ my class Any { # declared in BOOTSTRAP
 
     proto method AT-POS(|) is nodal {*}
     multi method AT-POS(Any:U \SELF: int \pos) is raw {
-        nqp::bindattr(my $v, Scalar, '$!whence',
-            -> { SELF.defined || (SELF = Array.new);
-                 SELF.BIND-POS(pos, $v) });
-        $v
+        nqp::p6bindattrinvres(
+          my $scalar,
+          Scalar,
+          '$!whence',
+          -> { nqp::if(
+                 nqp::isconcrete(SELF),
+                 SELF,
+                 (SELF = Array.new)
+               ).BIND-POS(pos, $scalar)
+             }
+        )
     }
     multi method AT-POS(Any:U \SELF: Int:D \pos) is raw {
-        nqp::bindattr(my $v, Scalar, '$!whence',
-            -> { SELF.defined || (SELF = Array.new);
-                 SELF.BIND-POS(nqp::unbox_i(pos), $v) });
-        $v
+        nqp::p6bindattrinvres(
+          my $scalar,
+          Scalar,
+          '$!whence',
+          -> { nqp::if(
+                 nqp::isconcrete(SELF),
+                 SELF,
+                 (SELF = Array.new)
+               ).BIND-POS(pos, $scalar)
+             }
+        )
     }
     multi method AT-POS(Any:U: Num:D \pos) is raw {
         nqp::isnanorinf(pos)
@@ -380,11 +398,18 @@ my class Any { # declared in BOOTSTRAP
           !! "Type {self.WHAT.perl} does not support associative indexing."
         )
     }
-    multi method AT-KEY(Any:U \SELF: $key) is raw {
-        nqp::bindattr(my $v, Scalar, '$!whence',
-            -> { SELF.defined || (SELF = Hash.new);
-                 SELF.BIND-KEY($key, $v) });
-        $v
+    multi method AT-KEY(Any:U \SELF: \key) is raw {
+        nqp::p6bindattrinvres(
+          my $scalar,
+          Scalar,
+          '$!whence',
+          -> { nqp::if(
+                 nqp::isconcrete(SELF),
+                 SELF,
+                 (SELF = Hash.new)
+               ).BIND-KEY(key, $scalar)
+             }
+        )
     }
 
     proto method BIND-KEY(|) is nodal { * }
@@ -409,14 +434,27 @@ my class Any { # declared in BOOTSTRAP
     }
     method FLATTENABLE_HASH() is nodal { nqp::hash() }
 
-    # XXX GLR do these really need to force a list?
-    method Set()     is nodal {     Set.new-from-pairs(self.list) }
-    method SetHash() is nodal { SetHash.new-from-pairs(self.list) }
-    method Bag()     is nodal {     Bag.new-from-pairs(self.list) }
-    method BagHash() is nodal { BagHash.new-from-pairs(self.list) }
-    method Mix()     is nodal {     Mix.new-from-pairs(self.list) }
-    method MixHash() is nodal { MixHash.new-from-pairs(self.list) }
-    method Supply() is nodal { self.list.Supply }
+    proto method Set(|) is nodal { * }
+    multi method Set(Any:) { Set.new-from-pairs(self.list) }
+
+    proto method SetHash(|) is nodal { * }
+    multi method SetHash(Any:) { SetHash.new-from-pairs(self.list) }
+
+    proto method Bag(|) is nodal { * }
+    multi method Bag(Any:) { Bag.new-from-pairs(self.list) }
+
+    proto method BagHash(|) is nodal { * }
+    multi method BagHash(Any:) { BagHash.new-from-pairs(self.list) }
+
+    proto method Mix(|) is nodal { * }
+    multi method Mix(Any:) { Mix.new-from-pairs(self.list) }
+
+    proto method MixHash(|) is nodal { * }
+    multi method MixHash() { MixHash.new-from-pairs(self.list) }
+
+    # XXX GLR does this really need to force a list?
+    proto method Supply(|) is nodal { * }
+    multi method Supply() { self.list.Supply }
 
     method nl-out() { "\n" }
     method print-nl() { self.print(self.nl-out) }
@@ -560,7 +598,7 @@ sub dd(|) {
             my $name := try $var.VAR.?name;
             my $type := $var.WHAT.^name;
             my $what := $var.?is-lazy
-              ?? $var[^10].perl.chop ~ "... (lazy list)"
+              ?? $var[^10].perl.chop ~ "... lazy list)"
               !! $var.perl;
             note $name ?? "$type $name = $what" !! $what;
         }
