@@ -1,7 +1,11 @@
+# for our tantrums
 my class X::TypeCheck { ... };
+my class X::TypeCheck::Splice { ... }
 my class X::Subscript::Negative { ... };
 my class X::NotEnoughDimensions { ... };
 my class X::Assignment::ArrayShapeMismatch { ... };
+
+# stub what we need now
 my class array is repr('VMArray') { ... };
 
 # An Array is a List that ensures every item added to it is in a Scalar
@@ -16,10 +20,12 @@ my class Array { # declared in BOOTSTRAP
         has $!descriptor;
 
         method new(\target, Mu \descriptor) {
-            my \rt = nqp::create(self);
-            nqp::bindattr(rt, self, '$!target', target);
-            nqp::bindattr(rt, self, '$!descriptor', descriptor);
-            rt
+            nqp::stmts(
+              nqp::bindattr((my \rt = nqp::create(self)),
+                self,'$!target',target),
+              nqp::p6bindattrinvres(rt,
+                self,'$!descriptor',descriptor)
+            )
         }
 
         method push(Mu \value) {
@@ -41,6 +47,8 @@ my class Array { # declared in BOOTSTRAP
         }
     }
 
+    multi method clone(Array:D:) { [self] }
+
     method iterator(Array:D:) {
 
         # something to iterate over in the future
@@ -134,68 +142,38 @@ my class Array { # declared in BOOTSTRAP
 
         # everything we need is already there
         elsif nqp::getattr(self,List,'$!reified').DEFINITE {
-            class :: does Iterator {
-                has int $!i;
-                has $!reified;
-                has $!descriptor;
-
-                method !SET-SELF(\array) {
-                    $!i           = -1;
-                    $!reified    := nqp::getattr(array, List,  '$!reified');
-                    $!descriptor := nqp::getattr(array, Array, '$!descriptor');
-                    self
-                }
-                method new(\array) { nqp::create(self)!SET-SELF(array) }
-
-                method pull-one() is raw {
-                    nqp::ifnull(
-                      nqp::atpos($!reified,$!i = nqp::add_i($!i,1)),
-                      nqp::islt_i($!i,nqp::elems($!reified)) # found a hole
-                        ?? nqp::p6bindattrinvres(
-                             (my \v := nqp::p6scalarfromdesc($!descriptor)),
-                             Scalar,
-                             '$!whence',
-                             -> { nqp::bindpos($!reified,$!i,v) }
-                           )
-                        !! IterationEnd
-                    )
-                }
-
-                method push-all($target --> IterationEnd) {
-                    my int $elems = nqp::elems($!reified);
-                    nqp::while(   # doesn't sink
-                      nqp::islt_i($!i = nqp::add_i($!i,1),$elems),
-                      $target.push(nqp::ifnull(
-                        nqp::atpos($!reified,$!i),
-                        nqp::p6bindattrinvres(
-                          (my \v := nqp::p6scalarfromdesc($!descriptor)),
-                          Scalar,
-                          '$!whence',
-                          -> { nqp::bindpos($!reified,$!i,v) }
-                        )
-                      ))
-                    )
-                }
-            }.new(self)
+            Rakudo::Iterator.ReifiedArray(self)
         }
 
         # nothing now or in the future to iterate over
         else {
-            Rakudo::Internals.EmptyIterator
+            Rakudo::Iterator.Empty
         }
     }
     method from-iterator(Array:U: Iterator $iter) {
-        my \result := nqp::create(self);
-        my \buffer := nqp::create(IterationBuffer);
-        my \todo := nqp::create(List::Reifier);
-        nqp::bindattr(result, List, '$!reified', buffer);
-        nqp::bindattr(result, List, '$!todo', todo);
-        nqp::bindattr(todo, List::Reifier, '$!reified', buffer);
-        nqp::bindattr(todo, List::Reifier, '$!current-iter', $iter);
-        nqp::bindattr(todo, List::Reifier, '$!reification-target',
-            result.reification-target());
-        todo.reify-until-lazy();
-        result
+        nqp::if(
+          nqp::eqaddr(
+            $iter.push-until-lazy(
+              my \target := ArrayReificationTarget.new(
+                (my \buffer := nqp::create(IterationBuffer)),
+                nqp::null
+              )
+            ),
+            IterationEnd
+          ),
+          nqp::p6bindattrinvres(nqp::create(self),List,'$!reified',buffer),
+          nqp::stmts(
+            nqp::bindattr((my \result := nqp::create(self)),
+              List,'$!reified',buffer),
+            nqp::bindattr((my \todo := nqp::create(List::Reifier)),
+              List::Reifier,'$!current-iter',$iter),
+            nqp::bindattr(todo,
+              List::Reifier,'$!reified',buffer),
+            nqp::bindattr(todo,
+              List::Reifier,'$!reification-target',target),
+            nqp::p6bindattrinvres(result,List,'$!todo',todo)
+          )
+        )
     }
 
     proto method new(|) { * }
@@ -290,7 +268,7 @@ my class Array { # declared in BOOTSTRAP
     multi method List(Array:D: :$view) {
         nqp::if(
           self.is-lazy,                           # can't make a List
-          Failure.new(X::Cannot::Lazy.new(:action<List>)),
+          X::Cannot::Lazy.new(:action<List>).throw,
 
           nqp::if(                                # all reified
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
@@ -320,7 +298,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::ifnull(
@@ -355,7 +333,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::ifnull(
@@ -398,7 +376,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::if(
@@ -437,7 +415,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::if(
@@ -476,7 +454,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::stmts(
             nqp::if(
               nqp::getattr(self,List,'$!reified').DEFINITE,
@@ -498,7 +476,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::stmts(
             nqp::if(
               nqp::getattr(self,List,'$!reified').DEFINITE,
@@ -520,7 +498,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           nqp::islt_i($pos,0),
           Failure.new(X::OutOfRange.new(
-            :what($*INDEX // 'Index'),:got($pos),:range<0..Inf>)),
+            :what($*INDEX // 'Index'),:got($pos),:range<0..^Inf>)),
           nqp::if(
             (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE,
             nqp::if(
@@ -559,13 +537,13 @@ my class Array { # declared in BOOTSTRAP
     # MUST have a separate Slip variant to have it slip
     multi method push(Array:D: Slip \value) {
         self.is-lazy
-          ?? Failure.new(X::Cannot::Lazy.new(action => 'push to'))
+          ?? X::Cannot::Lazy.new(action => 'push to').throw
           !! self!append-list(value)
     }
     multi method push(Array:D: \value) {
         nqp::if(
           self.is-lazy,
-          Failure.new(X::Cannot::Lazy.new(action => 'push to')),
+          X::Cannot::Lazy.new(action => 'push to').throw,
           nqp::stmts(
             nqp::push(
               nqp::if(
@@ -582,14 +560,14 @@ my class Array { # declared in BOOTSTRAP
     }
     multi method push(Array:D: **@values is raw) {
         self.is-lazy
-          ?? Failure.new(X::Cannot::Lazy.new(action => 'push to'))
+          ?? X::Cannot::Lazy.new(action => 'push to').throw
           !! self!append-list(@values)
     }
 
     multi method append(Array:D: \value) {
         nqp::if(
           self.is-lazy,
-          Failure.new(X::Cannot::Lazy.new(action => 'append to')),
+          X::Cannot::Lazy.new(action => 'append to').throw,
           nqp::if(
             (nqp::iscont(value) || nqp::not_i(nqp::istype(value, Iterable))),
             nqp::stmts(
@@ -610,7 +588,7 @@ my class Array { # declared in BOOTSTRAP
     }
     multi method append(Array:D: **@values is raw) {
         self.is-lazy
-          ?? Failure.new(X::Cannot::Lazy.new(action => 'append to'))
+          ?? X::Cannot::Lazy.new(action => 'append to').throw
           !! self!append-list(@values)
     }
     method !append-list(@values) {
@@ -630,7 +608,7 @@ my class Array { # declared in BOOTSTRAP
             IterationEnd
           ),
           self,
-          Failure.new(X::Cannot::Lazy.new(:action<push>,:what(self.^name)))
+          X::Cannot::Lazy.new(:action<push>,:what(self.^name)).throw
         )
     }
 
@@ -734,8 +712,7 @@ my class Array { # declared in BOOTSTRAP
         )
     }
 
-    my $nqplist := nqp::list;  # for splicing in without values
-    proto method splice(|) is nodal { * }
+    my $empty := nqp::create(IterationBuffer); # splicing in without values
     #------ splice() candidates
     multi method splice(Array:D \SELF:) {
         nqp::if(
@@ -809,14 +786,14 @@ my class Array { # declared in BOOTSTRAP
               nqp::atpos(nqp::getattr(self,List,'$!reified'),$i))
           ),
           nqp::splice(
-            nqp::getattr(self,List,'$!reified'),$nqplist,$offset,$size),
+            nqp::getattr(self,List,'$!reified'),$empty,$offset,$size),
           $result
         )
     }
     method !splice-offset-fail($got) {
-        Failure.new(X::OutOfRange.new(
+        X::OutOfRange.new(
           :what('Offset argument to splice'), :$got, :range("0..{self.elems}")
-        ))
+        ).throw
     }
 
     #------ splice(offset,size) candidates
@@ -893,7 +870,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::stmts(
           (my $result := self!splice-save($offset,$size,my int $removed)),
           nqp::splice(
-            nqp::getattr(self,List,'$!reified'),$nqplist,$offset,$removed),
+            nqp::getattr(self,List,'$!reified'),$empty,$offset,$removed),
           $result
         )
     }
@@ -931,11 +908,11 @@ my class Array { # declared in BOOTSTRAP
         nqp::if(
           $offset > self.elems,
           self!splice-offset-fail($offset),
-          Failure.new(X::OutOfRange.new(
+          X::OutOfRange.new(
             :what('Size argument to splice'),
             :$got,
             :range("0..^{self.elems - $offset}")
-          ))
+          ).throw
         )
     }
     #------ splice(offset,size,array) candidates
@@ -1009,7 +986,8 @@ my class Array { # declared in BOOTSTRAP
                   nqp::elems(nqp::if(
                     nqp::getattr(self,List,'$!reified').DEFINITE,
                     nqp::getattr(self,List,'$!reified'),
-                    nqp::bindattr(self,List,'$!reified',nqp::list)
+                    nqp::bindattr(self,List,'$!reified',
+                      nqp::create(IterationBuffer))
                   )),
                   nqp::unbox_i($offset),
                 ),
@@ -1044,11 +1022,11 @@ my class Array { # declared in BOOTSTRAP
               ),
               nqp::if(
                 nqp::islt_i($i,$elems),   # exited loop because of wrong type
-                Failure.new(X::TypeCheck::Splice.new(
+                X::TypeCheck::Splice.new(
                   :action<splice>,
                   :got(nqp::atpos($new,$i).WHAT),
                   :$expected
-                )),
+                ).throw,
                 nqp::stmts(
                   ($result := self!splice-save($offset,$size,$removed)),
                   nqp::splice(
@@ -1058,7 +1036,34 @@ my class Array { # declared in BOOTSTRAP
               )
             )
           ),
-          Failure.new(X::Cannot::Lazy.new(:action('splice in')))
+          X::Cannot::Lazy.new(:action('splice in')).throw
+        )
+    }
+
+    multi method tail(Array:D: $n) {
+        nqp::if(
+          nqp::getattr(self,List,'$!todo').DEFINITE,
+          self.Any::tail($n),
+          Seq.new(
+            nqp::if(
+              (my $reified := nqp::getattr(self,List,'$!reified')).DEFINITE
+                && nqp::elems($reified),
+              nqp::stmts(
+                (my $iterator := Rakudo::Iterator.ReifiedArray(self)),
+                nqp::if(
+                  nqp::istype($n,Callable)
+                    && nqp::isgt_i((my $skip := -($n(0).Int)),0),
+                  $iterator.skip-at-least($skip),
+                  nqp::unless(
+                    nqp::istype($n,Whatever) || $n == Inf,
+                    $iterator.skip-at-least(nqp::elems($reified) - $n)
+                  )
+                ),
+                $iterator
+              ),
+              Rakudo::Iterator.Empty
+            )
+          )
         )
     }
 
@@ -1073,7 +1078,7 @@ my class Array { # declared in BOOTSTRAP
         nqp::isnull($!descriptor) ?? Any !! $!descriptor.default
     }
     method dynamic() {
-        nqp::isnull($!descriptor) ?? Nil !! so $!descriptor.dynamic
+        nqp::isnull($!descriptor) ?? False !! so $!descriptor.dynamic
     }
     multi method perl(Array:D \SELF:) {
         SELF.perlseen('Array', {
