@@ -60,6 +60,13 @@ my class Backtrace::Frame {
     }
     method is-setting(Backtrace::Frame:D:) {
         $!file.starts-with("SETTING::")
+#?if jvm
+          || $!file.ends-with("CORE.setting")
+#?endif
+#?if !jvm
+          || $!file.ends-with("CORE.setting." ~ Rakudo::Internals.PRECOMP-EXT)
+#?endif
+          || $!file.ends-with(".nqp")
     }
 }
 
@@ -131,7 +138,7 @@ my class Backtrace {
             next unless $file;
 
             if CompUnit::RepositoryRegistry.file-for-spec($file) -> $path {
-                $file := $path.abspath;
+                $file := $path.absolute;
             }
 
             # now *that's* an evil hack
@@ -315,15 +322,14 @@ my class Backtrace {
         my $bt = $!bt;
         for $bt.keys {
             my $p6sub := $bt[$_]<sub>;
-            if nqp::istype($p6sub, Sub) {
-                return True if $p6sub.name eq 'THREAD-ENTRY';
-            }
-            elsif nqp::istype($p6sub, ForeignCode) {
+            if nqp::istype($p6sub, ForeignCode) {
                 try {
                     my Mu $sub := nqp::getattr(nqp::decont($p6sub), ForeignCode, '$!do');
-                    return True if nqp::iseq_s(nqp::getcodename($sub), 'eval');
-                    return True if nqp::iseq_s(nqp::getcodename($sub), 'print_control');
-                    return False if nqp::iseq_s(nqp::getcodename($sub), 'compile');
+                    my str $name = nqp::getcodename($sub);
+                    return True if nqp::iseq_s($name, 'THREAD-ENTRY');
+                    return True if nqp::iseq_s($name, 'eval');
+                    return True if nqp::iseq_s($name, 'print_control');
+                    return False if nqp::iseq_s($name, 'compile');
                 }
             }
         }
