@@ -10,34 +10,51 @@ my class Bag does Baggy {
           $!WHICH := self!WHICH
         )
     }
-    method total(Bag:D: --> Int) {
+    method total(Bag:D: --> Int:D) {
         nqp::if(
           nqp::attrinited(self,Bag,'$!total'),
           $!total,
-          $!total := self!TOTAL
+          $!total := Rakudo::QuantHash.BAG-TOTAL(self.raw_hash)
+        )
+    }
+
+#--- object creation methods
+    multi method new(Bag:_:) {
+        nqp::if(
+          nqp::eqaddr(self.WHAT,Bag),
+          bag(),
+          nqp::create(self)
         )
     }
 
 #--- interface methods
+    method SET-SELF(Bag:D: \elems) {
+        nqp::if(
+          nqp::elems(elems),
+          nqp::stmts(                 # need to have allocated %!elems
+            nqp::bindattr(%!elems,Map,'$!storage',elems),
+            self
+          ),
+          bag()
+        )
+    }
+
     multi method DELETE-KEY(Bag:D: \k) {
         X::Immutable.new(method => 'DELETE-KEY', typename => self.^name).throw;
     }
 
 #--- selection methods
-    multi method grab(Bag:D: $count?) {
-        X::Immutable.new( method => 'grab', typename => self.^name ).throw;
-    }
     multi method grabpairs(Bag:D: $count?) {
         X::Immutable.new( method => 'grabpairs', typename => self.^name ).throw;
     }
 
 #--- coercion methods
-    method Bag     { self }
-    method BagHash { BagHash.new-from-pairs(%!elems.values) }
-    method Mix {
+    multi method Bag(Bag:D:) { self }
+    multi method Mix(Bag:D:) {
         nqp::p6bindattrinvres(nqp::create(Mix),Mix,'%!elems',%!elems)
     }
-    method MixHash { MixHash.new-from-pairs(%!elems.values) }
+
+    method clone() { nqp::clone(self) }
 
     proto method classify-list(|) {
         X::Immutable.new(:method<classify-list>, :typename(self.^name)).throw;
