@@ -4,16 +4,33 @@ my class X::Buf::Pack::NonASCII { ... }
 my class X::Cannot::Empty       { ... }
 my class X::Cannot::Lazy        { ... }
 my class X::Experimental        { ... }
-my class X::TypeCheck           { ... }
 
 my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is array_type(T) {
-    my int $bpe = (T.^nativesize / 8).Int;  # other then *8 not supported yet
+    X::NYI.new(
+      feature => "{$?CLASS.^name.comb(/^ \w+ /)}s with native {T.^name}"
+    ).throw unless nqp::istype(T,Int);
 
-    X::NYI.new(feature => "{$?CLASS.^name.comb(/^ \w+ /)}s with native {T.^name}").throw
-      unless nqp::istype(T,Int);
+    # other then *8 not supported yet
+    my int $bpe = try {
+#?if jvm
+        # https://irclog.perlgeek.de/perl6-dev/2017-01-20#i_13961377
+        CATCH { default { Nil } }
+#?endif
+        (T.^nativesize / 8).Int
+    } // 1;
 
     multi method WHICH(Blob:D:) {
-        self.^name ~ '|' ~ nqp::sha1(self.decode("latin-1"))
+        nqp::box_s(
+          nqp::concat(
+            nqp::if(
+              nqp::eqaddr(self.WHAT,Blob),
+              'Blob|',
+            nqp::concat(nqp::unbox_s(self.^name), '|')
+            ),
+            nqp::sha1(self.decode("latin-1"))
+          ),
+          ObjAt
+        )
     }
 
     multi method new(Blob:) { nqp::create(self) }
@@ -87,7 +104,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     multi method Bool(Blob:D:) { nqp::p6bool(nqp::elems(self)) }
 
     multi method elems(Blob:D:)   { nqp::p6box_i(nqp::elems(self)) }
-    multi method elems(Blob:U:)   { 1 }
+    multi method elems(Blob:U: --> 1)   { }
     method Numeric(Blob:D:) { nqp::p6box_i(nqp::elems(self)) }
     method Int(Blob:D:)     { nqp::p6box_i(nqp::elems(self)) }
 
@@ -97,17 +114,23 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     multi method Str(Blob:D:)   { X::Buf::AsStr.new(method => 'Str'  ).throw }
     multi method Stringy(Blob:D:) { X::Buf::AsStr.new(method => 'Stringy' ).throw }
 
-    method decode(Blob:D: $encoding = 'utf-8') {
+    proto method decode(|) { * }
+    multi method decode(Blob:D:) {
+        nqp::p6box_s(nqp::decode(self, 'utf8'))
+    }
+    multi method decode(Blob:D: $encoding) {
         nqp::p6box_s(
           nqp::decode(self, Rakudo::Internals.NORMALIZE_ENCODING($encoding)))
     }
 
     multi method list(Blob:D:) {
-        Seq.new(class :: does Rakudo::Internals::BlobbyIterator {
+        Seq.new(class :: does Rakudo::Iterator::Blobby {
             method pull-one() is raw {
-                nqp::islt_i(++$!i,$!elems)
-                  ?? nqp::atpos_i($!blob,$!i)
-                  !! IterationEnd
+                nqp::if(
+                  nqp::islt_i(($!i = nqp::add_i($!i,1)),nqp::elems($!blob)),
+                  nqp::atpos_i($!blob,$!i),
+                  IterationEnd
+                )
             }
         }.new(self))
     }
@@ -131,7 +154,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
         my int $pos;
         my int $todo;
         if nqp::istype($from,Range) {
-            ($pos,my int $max) = $from.int-bounds;
+            $from.int-bounds($pos, my int $max);
             $todo = $max - $pos + 1;
         }
         else {
@@ -211,7 +234,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
         my int $i     = -1;
 
         nqp::bindpos_s($list,$i,
-          nqp::tostr_I(nqp::p6box_i(nqp::atpos_i(self,$i)))) 
+          nqp::tostr_I(nqp::p6box_i(nqp::atpos_i(self,$i))))
           while nqp::islt_i(++$i,$elems);
 
         nqp::join($delim.Str,$list)
@@ -388,7 +411,7 @@ constant blob32 = Blob[uint32];
 constant blob64 = Blob[uint64];
 
 my class utf8 does Blob[uint8] is repr('VMArray') {
-    method decode(utf8:D: $encoding = 'utf-8') {
+    multi method decode(utf8:D: $encoding) {
         my $enc = Rakudo::Internals.NORMALIZE_ENCODING($encoding);
         die "Can not decode a utf-8 buffer as if it were $encoding"
             unless $enc eq 'utf8';
@@ -400,7 +423,7 @@ my class utf8 does Blob[uint8] is repr('VMArray') {
 }
 
 my class utf16 does Blob[uint16] is repr('VMArray') {
-    method decode(utf16:D: $encoding = 'utf-16') {
+    multi method decode(utf16:D: $encoding = 'utf-16') {
         my $enc = Rakudo::Internals.NORMALIZE_ENCODING($encoding);
         die "Can not decode a utf-16 buffer as if it were $encoding"
             unless $enc eq 'utf16';
@@ -412,7 +435,7 @@ my class utf16 does Blob[uint16] is repr('VMArray') {
 }
 
 my class utf32 does Blob[uint32] is repr('VMArray') {
-    method decode(utf32:D: $encoding = 'utf-32') {
+    multi method decode(utf32:D: $encoding = 'utf-32') {
         my $enc = Rakudo::Internals.NORMALIZE_ENCODING($encoding);
         die "Can not decode a utf-32 buffer as if it were $encoding"
             unless $enc eq 'utf32';
@@ -430,37 +453,39 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
     multi method AT-POS(Buf:D: int \pos) is raw {
         nqp::islt_i(pos,0)
           ?? Failure.new(X::OutOfRange.new(
-               :what($*INDEX // 'Index'),:got(pos),:range<0..Inf>))
+               :what($*INDEX // 'Index'),:got(pos),:range<0..^Inf>))
           !! nqp::atposref_i(self, pos)
     }
     multi method AT-POS(Buf:D: Int:D \pos) is raw {
         my int $pos = nqp::unbox_i(pos);
         nqp::islt_i($pos,0)
           ?? Failure.new(X::OutOfRange.new(
-               :what($*INDEX // 'Index'),:got(pos),:range<0..Inf>))
+               :what($*INDEX // 'Index'),:got(pos),:range<0..^Inf>))
           !! nqp::atposref_i(self,$pos)
     }
 
     multi method ASSIGN-POS(Buf:D: int \pos, Mu \assignee) {
         nqp::islt_i(pos,0)
           ?? Failure.new(X::OutOfRange.new(
-               :what($*INDEX // 'Index'),:got(pos),:range<0..Inf>))
+               :what($*INDEX // 'Index'),:got(pos),:range<0..^Inf>))
           !! nqp::bindpos_i(self,\pos,assignee)
     }
     multi method ASSIGN-POS(Buf:D: Int:D \pos, Mu \assignee) {
         my int $pos = nqp::unbox_i(pos);
         nqp::islt_i($pos,0)
           ?? Failure.new(X::OutOfRange.new(
-               :what($*INDEX // 'Index'),:got(pos),:range<0..Inf>))
+               :what($*INDEX // 'Index'),:got(pos),:range<0..^Inf>))
           !! nqp::bindpos_i(self,$pos,assignee)
     }
 
     multi method list(Buf:D:) {
-        Seq.new(class :: does Rakudo::Internals::BlobbyIterator {
+        Seq.new(class :: does Rakudo::Iterator::Blobby {
             method pull-one() is raw {
-                nqp::islt_i(++$!i,$!elems)
-                  ?? nqp::atposref_i($!blob,$!i)
-                  !! IterationEnd
+                nqp::if(
+                  nqp::islt_i(($!i = nqp::add_i($!i,1)),nqp::elems($!blob)),
+                  nqp::atposref_i($!blob,$!i),
+                  IterationEnd
+                )
             }
         }.new(self))
     }
@@ -612,7 +637,7 @@ multi sub pack(@template, *@items) {
                 my $data = shift @items // Buf.new;
                 $data.=encode if nqp::istype($data,Str);
                 if $amount eq '*' {
-                    $amount = $data.cache.elems;
+                    $amount = $data.elems;
                 }
                 if $amount eq '' {
                     $amount = 1;
@@ -686,8 +711,8 @@ multi sub prefix:<~^>(Blob:D \a) {
     my $r := nqp::create($a);
     nqp::setelems($a,$elems);
 
-    my int $i    = -1;
-    my int $mask = 0xFFFFFFFFFFFFFFFF;
+    my int    $i    = -1;
+    my uint64 $mask = 0xFFFFFFFFFFFFFFFF;
     nqp::bindpos_i($r,$i,nqp::bitxor_i(nqp::atpos_i($a,$i),$mask))
       while nqp::islt_i(++$i,$elems);
 
