@@ -4,7 +4,7 @@ my class Code does Callable { # declared in BOOTSTRAP
     #     has Signature $!signature;  # Signature object
     #     has @!compstuff;            # Place for the compiler to hang stuff
 
-    multi method ACCEPTS(Code:D $self: Mu $topic) {
+    multi method ACCEPTS(Code:D $self: Mu $topic is raw) {
         $self.count ?? $self($topic) !! $self()
     }
 
@@ -14,6 +14,10 @@ my class Code does Callable { # declared in BOOTSTRAP
 
     method signature(Code:D:) { $!signature }
 
+    proto method prec(|) { * }
+    multi method prec() { my % }
+    multi method prec(Str:D $) { '' }
+
     multi method Str(Code:D:) {
         warn( self.WHAT.perl ~ " object coerced to string (please use .gist or .perl to do that)"); self.name
     }
