@@ -1,10 +1,16 @@
 {
     role CompUnit::PrecompilationRepository {
-        method load(CompUnit::PrecompilationId $id) {
+        method try-load(
+            CompUnit::PrecompilationDependency::File $dependency,
+            IO::Path :$source,
+            CompUnit::PrecompilationStore :@precomp-stores,
+            --> CompUnit::Handle:D) {
             Nil
         }
 
-        method may-precomp() {
+        method load(CompUnit::PrecompilationId $id --> Nil) { }
+
+        method may-precomp(--> Bool:D) {
             True # would be a good place to check an environment variable
         }
     }
@@ -16,6 +22,7 @@ class CompUnit { ... }
 class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationRepository {
     has CompUnit::PrecompilationStore $.store;
     my %loaded;
+    my $loaded-lock = Lock.new;
     my $first-repo-id;
 
     my $lle;
@@ -26,17 +33,19 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
         CompUnit::PrecompilationDependency::File $dependency,
         IO::Path :$source = $dependency.src.IO,
         CompUnit::PrecompilationStore :@precomp-stores = Array[CompUnit::PrecompilationStore].new($.store),
-    ) returns CompUnit::Handle {
+     --> CompUnit::Handle:D) {
         my $RMD = $*RAKUDO_MODULE_DEBUG;
         my $id = $dependency.id;
         $RMD("try-load $id: $source") if $RMD;
 
         # Even if we may no longer precompile, we should use already loaded files
-        return %loaded{$id} if %loaded{$id}:exists;
+        $loaded-lock.protect: {
+            return %loaded{$id} if %loaded{$id}:exists;
+        }
 
         my ($handle, $checksum) = (
             self.may-precomp and (
-                my $loaded = self.load($id, :since($source.modified), :@precomp-stores) # already precompiled?
+                my $loaded = self.load($id, :source($source), :checksum($dependency.checksum), :@precomp-stores) # already precompiled?
                 or self.precompile($source, $id, :source-name($dependency.source-name), :force($loaded ~~ Failure))
                     and self.load($id, :@precomp-stores) # if not do it now
             )
@@ -60,6 +69,7 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
         if $*RAKUDO_MODULE_DEBUG -> $RMD { $RMD("Loading precompiled\n$unit") }
 #?if moar
         my $handle := CompUnit::Loader.load-precompilation-file($unit.bytecode-handle);
+        $unit.close;
 #?endif
 #?if !moar
         my $handle := CompUnit::Loader.load-precompilation($unit.bytecode);
@@ -91,7 +101,7 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
         Nil
     }
 
-    method !load-dependencies(CompUnit::PrecompilationUnit:D $precomp-unit, Instant $since, @precomp-stores) {
+    method !load-dependencies(CompUnit::PrecompilationUnit:D $precomp-unit, @precomp-stores) {
         my $compiler-id = CompUnit::PrecompilationId.new($*PERL.compiler.id);
         my $RMD = $*RAKUDO_MODULE_DEBUG;
         my $resolve = False;
@@ -106,6 +116,7 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
             $RMD("Repo chain changed: $repo-id ne {$first-repo-id}. Need to re-check dependencies.") if $RMD;
             $resolve = True;
         }
+        $resolve = False unless %*ENV<RAKUDO_RERESOLVE_DEPENDENCIES> // 1;
         my @dependencies;
         for $precomp-unit.dependencies -> $dependency {
             $RMD("dependency: $dependency") if $RMD;
@@ -116,31 +127,23 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
                 return False unless $comp-unit and $comp-unit.repo-id eq $dependency.id;
             }
 
-            my $file;
-            my $store = @precomp-stores.first({ $file = $_.path($compiler-id, $dependency.id); $file.e });
-            $RMD("Could not find $dependency.spec()") if $RMD and not $store;
-            return False unless $store;
-
-            if $resolve { # a repo changed, so maybe it's a change in our source file
-                my $modified = $file.modified;
-                $RMD("$file\nspec: $dependency.spec()\nmtime: $modified\nsince: $since")
-                  if $RMD;
-
-                my $srcIO = CompUnit::RepositoryRegistry.file-for-spec($dependency.src) // $dependency.src.IO;
-                $RMD("source: $srcIO mtime: " ~ $srcIO.modified) if $RMD and $srcIO.e;
-                return False if not $srcIO.e or $modified < $srcIO.modified;
-            }
-
-            my $dependency-precomp = $store.load-unit($compiler-id, $dependency.id);
-            $RMD("dependency checksum $dependency.checksum() unit: $dependency-precomp.checksum()") if $RMD;
-            return False if $dependency-precomp.checksum ne $dependency.checksum;
+            my $dependency-precomp = @precomp-stores
+                .map({ $_.load-unit($compiler-id, $dependency.id) })
+                .first(*.defined)
+                or do {
+                    $RMD("Could not find $dependency.spec()") if $RMD;
+                    return False;
+                }
+            return False unless $dependency-precomp.is-up-to-date($dependency, :check-source($resolve));
 
             @dependencies.push: $dependency-precomp;
         }
 
-        for @dependencies -> $dependency-precomp {
-            unless %loaded{$dependency-precomp.id}:exists {
-                %loaded{$dependency-precomp.id} = self!load-handle-for-path($dependency-precomp);
+        $loaded-lock.protect: {
+            for @dependencies -> $dependency-precomp {
+                unless %loaded{$dependency-precomp.id}:exists {
+                    %loaded{$dependency-precomp.id} = self!load-handle-for-path($dependency-precomp);
+                }
             }
         }
 
@@ -162,6 +165,7 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
     multi method load(
         Str $id,
         Instant :$since,
+        IO::Path :$source,
         CompUnit::PrecompilationStore :@precomp-stores = Array[CompUnit::PrecompilationStore].new($.store),
     ) {
         self.load(CompUnit::PrecompilationId.new($id), :$since, :@precomp-stores)
@@ -169,24 +173,30 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
 
     multi method load(
         CompUnit::PrecompilationId $id,
+        IO::Path :$source,
+        Str :$checksum is copy,
         Instant :$since,
         CompUnit::PrecompilationStore :@precomp-stores = Array[CompUnit::PrecompilationStore].new($.store),
     ) {
-        return %loaded{$id} if %loaded{$id}:exists;
+        $loaded-lock.protect: {
+            return %loaded{$id} if %loaded{$id}:exists;
+        }
         my $RMD = $*RAKUDO_MODULE_DEBUG;
         my $compiler-id = CompUnit::PrecompilationId.new($*PERL.compiler.id);
         my $unit = self!load-file(@precomp-stores, $id);
         if $unit {
-            my $modified = $unit.modified;
-            if (not $since or $modified > $since)
-                and self!load-dependencies($unit, $modified, @precomp-stores)
+            if (not $since or $unit.modified > $since)
+                and (not $source or ($checksum //= nqp::sha1($source.slurp(:enc<iso-8859-1>))) eq $unit.source-checksum)
+                and self!load-dependencies($unit, @precomp-stores)
             {
-                return (%loaded{$id} = self!load-handle-for-path($unit)), $unit.checksum;
+                my \loaded = self!load-handle-for-path($unit);
+                $loaded-lock.protect: { %loaded{$id} = loaded };
+                return (loaded, $unit.checksum);
             }
             else {
-                if $*RAKUDO_MODULE_DEBUG -> $RMD {
-                    $RMD("Outdated precompiled $unit\nmtime: $modified\nsince: $since")
-                }
+                $RMD("Outdated precompiled {$unit}{$source ?? " for $source" !! ''}\n"
+                     ~ "    mtime: {$unit.modified}{$since ?? ", since: $since" !! ''}\n"
+                     ~ "    checksum: {$unit.source-checksum}, expected: $checksum") if $RMD;
                 $unit.close;
                 fail "Outdated precompiled $unit";
             }
@@ -220,62 +230,67 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
             self.store.unlock;
             return True;
         }
+        my $source-checksum = nqp::sha1($path.slurp(:enc<iso-8859-1>));
         my $bc = "$io.bc".IO;
 
         $lle     //= Rakudo::Internals.LL-EXCEPTION;
         $profile //= Rakudo::Internals.PROFILE;
         $optimize //= Rakudo::Internals.OPTIMIZE;
-        my %ENV := %*ENV;
-        %ENV<RAKUDO_PRECOMP_WITH> = $*REPO.repo-chain.map(*.path-spec).join(',');
+        my %env = %*ENV; # Local copy for us to tweak
+        %env<RAKUDO_PRECOMP_WITH> = $*REPO.repo-chain.map(*.path-spec).join(',');
 
-        my $rakudo_precomp_loading = %ENV<RAKUDO_PRECOMP_LOADING>;
+        my $rakudo_precomp_loading = %env<RAKUDO_PRECOMP_LOADING>;
         my $modules = $rakudo_precomp_loading ?? Rakudo::Internals::JSON.from-json: $rakudo_precomp_loading !! [];
         die "Circular module loading detected trying to precompile $path" if $modules.Set{$path.Str}:exists;
-        %ENV<RAKUDO_PRECOMP_LOADING> = Rakudo::Internals::JSON.to-json: [|$modules, $path.Str];
-
-        my $current_dist = %ENV<RAKUDO_PRECOMP_DIST>;
-        %ENV<RAKUDO_PRECOMP_DIST> = $*RESOURCES ?? $*RESOURCES.Str !! '{}';
+        %env<RAKUDO_PRECOMP_LOADING> = Rakudo::Internals::JSON.to-json: [|$modules, $path.Str];
+        %env<RAKUDO_PRECOMP_DIST> = $*RESOURCES ?? $*RESOURCES.Str !! '{}';
 
         $RMD("Precompiling $path into $bc ($lle $profile $optimize)") if $RMD;
         my $perl6 = $*EXECUTABLE
             .subst('perl6-debug', 'perl6') # debugger would try to precompile it's UI
             .subst('perl6-gdb', 'perl6')
             .subst('perl6-jdb-server', 'perl6-j') ;
-        if %*ENV<RAKUDO_PRECOMP_NESTED_JDB> {
+        if %env<RAKUDO_PRECOMP_NESTED_JDB> {
             $perl6.subst-mutate('perl6-j', 'perl6-jdb-server');
-            note "starting jdb on port " ~ ++%*ENV<RAKUDO_JDB_PORT>;
+            note "starting jdb on port " ~ ++%env<RAKUDO_JDB_PORT>;
         }
-        my $proc = run(
-          $perl6,
-          $lle,
-          $profile,
-          $optimize,
-          "--target=" ~ Rakudo::Internals.PRECOMP-TARGET,
-          "--output=$bc",
-          "--source-name=$source-name",
-          $path,
-          :out,
-          :err,
-        );
-        %ENV.DELETE-KEY(<RAKUDO_PRECOMP_WITH>);
-        if $rakudo_precomp_loading {
-            %ENV<RAKUDO_PRECOMP_LOADING> = $rakudo_precomp_loading;
-        }
-        else {
-            %ENV.DELETE-KEY(<RAKUDO_PRECOMP_LOADING>);
-        }
-        %ENV.DELETE-KEY(<RAKUDO_PRECOMP_LOADING>);
-        %ENV<RAKUDO_PRECOMP_DIST> = $current_dist;
+        my $out = '';
+        my $err = '';
+        my $status;
+        react {
+            my $proc = Proc::Async.new(
+                $perl6,
+                $lle,
+                $profile,
+                $optimize,
+                "--target=" ~ Rakudo::Internals.PRECOMP-TARGET,
+                "--output=$bc",
+                "--source-name=$source-name",
+                $path
+            );
 
-        my @result = $proc.out.lines.unique;
-        if not $proc.out.close or $proc.status {  # something wrong
+            whenever $proc.stdout {
+                $out ~= $_
+            }
+            unless $RMD {
+                whenever $proc.stderr {
+                    $err ~= $_
+                }
+            }
+            whenever $proc.start(ENV => %env) {
+                $status = .exitcode
+            }
+        }
+
+        my @result = $out.lines.unique;
+        if $status {  # something wrong
             self.store.unlock;
-            $RMD("Precomping $path failed: $proc.status()") if $RMD;
+            $RMD("Precomping $path failed: $status") if $RMD;
             Rakudo::Internals.VERBATIM-EXCEPTION(1);
-            die $proc.err.slurp-rest(:close);
+            die $RMD ?? @result !! $err;
         }
 
-        if $proc.err.slurp-rest(:close) -> $warnings {
+        if not $RMD and $err -> $warnings {
             $*ERR.print($warnings);
         }
         unless $bc.e {
@@ -294,14 +309,14 @@ class CompUnit::PrecompilationRepository::Default does CompUnit::PrecompilationR
             }
             my $dependency = CompUnit::PrecompilationDependency::File.deserialize($dependency-str);
             next if %dependencies{$dependency.Str}++; # already got that one
-            $RMD("id: $dependency.id(), src: $dependency.src(), spec: $dependency.spec()") if $RMD;
+            $RMD($dependency.Str()) if $RMD;
             @dependencies.push: $dependency;
         }
-        $RMD("Writing dependencies and byte code to $io.tmp") if $RMD;
+        $RMD("Writing dependencies and byte code to $io.tmp for source checksum: $source-checksum") if $RMD;
         self.store.store-unit(
             $compiler-id,
             $id,
-            self.store.new-unit(:$id, :@dependencies, :bytecode($bc.slurp(:bin))),
+            self.store.new-unit(:$id, :@dependencies, :$source-checksum, :bytecode($bc.slurp(:bin))),
         );
         $bc.unlink;
         self.store.store-repo-id($compiler-id, $id, :repo-id($*REPO.id));
