@@ -6,12 +6,23 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
         has CompUnit::PrecompilationDependency @!dependencies;
         has $!initialized = False;
         has $.checksum;
+        has $.source-checksum;
         has $!bytecode;
+        has $!store;
+        has Lock $!update-lock = Lock.new;
 
-        submethod BUILD(CompUnit::PrecompilationId :$!id, IO::Path :$!path, :@!dependencies, :$!bytecode --> Nil) {
+        submethod BUILD(
+            CompUnit::PrecompilationId :$!id,
+            IO::Path :$!path,
+            :$!source-checksum,
+            :@!dependencies,
+            :$!bytecode,
+            :$!store,
+            --> Nil
+        ) {
             if $!bytecode {
                 $!initialized = True;
-                $!checksum = nqp::sha1($!bytecode.decode("latin-1"));
+                $!checksum = nqp::sha1($!bytecode.decode('iso-8859-1'));
             }
         }
 
@@ -19,21 +30,24 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
             $!file = $!path.open(:r);
         }
 
-        method modified(--> Instant) {
+        method modified(--> Instant:D) {
             $!path.modified
         }
 
         method !read-dependencies() {
-            return if $!initialized;
-            self!open(:r) unless $!file;
+            $!update-lock.protect: {
+                return if $!initialized;
+                self!open(:r) unless $!file;
 
-            $!checksum     = $!file.get;
-            my $dependency = $!file.get;
-            while $dependency {
-                @!dependencies.push: CompUnit::PrecompilationDependency::File.deserialize($dependency);
-                $dependency = $!file.get;
+                $!checksum        = $!file.get;
+                $!source-checksum = $!file.get;
+                my $dependency    = $!file.get;
+                while $dependency {
+                    @!dependencies.push: CompUnit::PrecompilationDependency::File.deserialize($dependency);
+                    $dependency = $!file.get;
+                }
+                $!initialized = True;
             }
-            $!initialized = True;
         }
 
         method dependencies(--> Array[CompUnit::PrecompilationDependency]) {
@@ -41,39 +55,55 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
             @!dependencies
         }
 
-        method bytecode(--> Buf) {
-            self!read-dependencies;
-            $!bytecode //= $!file.slurp-rest(:bin,:close)
+        method bytecode(--> Buf:D) {
+            $!update-lock.protect: {
+                self!read-dependencies;
+                $!bytecode //= $!file.slurp-rest(:bin,:close)
+            }
         }
 
-        method bytecode-handle(--> IO::Handle) {
+        method bytecode-handle(--> IO::Handle:D) {
             self!read-dependencies;
             $!file
         }
 
+        method source-checksum() is rw {
+            self!read-dependencies;
+            $!source-checksum
+        }
+
         method checksum() is rw {
             self!read-dependencies;
             $!checksum
         }
 
-        method Str(--> Str) {
+        method Str(--> Str:D) {
             self.path.Str
         }
 
         method close(--> Nil) {
-            $!file.close if $!file;
-            $!file = Nil;
+            $!update-lock.protect: {
+                $!file.close if $!file;
+                $!file = Nil;
+            }
         }
 
         method save-to(IO::Path $precomp-file) {
             my $handle = $precomp-file.open(:w);
             $handle.print($!checksum ~ "\n");
+            $handle.print($!source-checksum ~ "\n");
             $handle.print($_.serialize ~ "\n") for @!dependencies;
             $handle.print("\n");
             $handle.write($!bytecode);
             $handle.close;
             $!path = $precomp-file;
         }
+
+        method is-up-to-date(CompUnit::PrecompilationDependency $dependency, Bool :$check-source --> Bool) {
+            my $result = self.CompUnit::PrecompilationUnit::is-up-to-date($dependency, :$check-source);
+            $!store.remove-from-cache($.id) unless $result;
+            $result
+        }
     }
 
     has IO::Path $.prefix is required;
@@ -82,49 +112,60 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
     has %!loaded;
     has %!compiler-cache;
     has %!dir-cache;
+    has Lock $!update-lock = Lock.new;
 
     submethod BUILD(IO::Path :$!prefix --> Nil) {
     }
 
     method new-unit(|c) {
-        CompUnit::PrecompilationUnit::File.new(|c)
+        CompUnit::PrecompilationUnit::File.new(|c, :store(self))
     }
 
     method !dir(CompUnit::PrecompilationId $compiler-id,
                 CompUnit::PrecompilationId $precomp-id)
     {
-        %!dir-cache{$compiler-id ~ $precomp-id} //=
-            (%!compiler-cache{$compiler-id} //= self.prefix.child($compiler-id.IO))
-                .child($precomp-id.substr(0, 2).IO)
+        $!update-lock.protect: {
+            %!dir-cache{$compiler-id ~ $precomp-id} //=
+                (%!compiler-cache{$compiler-id} //= self.prefix.add($compiler-id))
+                    .add($precomp-id.substr(0, 2))
+        }
     }
 
     method path(CompUnit::PrecompilationId $compiler-id,
                  CompUnit::PrecompilationId $precomp-id,
                  Str :$extension = '')
     {
-        self!dir($compiler-id, $precomp-id).child(($precomp-id ~ $extension).IO)
+        self!dir($compiler-id, $precomp-id).add($precomp-id ~ $extension)
     }
 
-    method !lock() {
+    method !lock(--> Nil) {
         return if $*W && $*W.is_precompilation_mode();
-        $!lock //= $.prefix.child('.lock').open(:create, :rw);
-        $!lock.lock(2) if $!lock-count == 0;
-        $!lock-count++;
+        my int $acquire-file-lock = $!update-lock.protect: {
+            $!lock //= $.prefix.add('.lock').open(:create, :rw);
+            $!lock-count++
+        }
+        $!lock.lock if $acquire-file-lock == 0;
     }
 
     method unlock() {
         return if $*W && $*W.is_precompilation_mode();
-        die "unlock when we're not locked!" if $!lock-count == 0;
-        $!lock-count-- if $!lock-count > 0;
-        $!lock && $!lock-count == 0 ?? $!lock.unlock !! True;
+        $!update-lock.protect: {
+            die "unlock when we're not locked!" if $!lock-count == 0;
+            $!lock-count-- if $!lock-count > 0;
+            $!lock && $!lock-count == 0 ?? $!lock.unlock !! True
+        }
     }
 
     method load-unit(CompUnit::PrecompilationId $compiler-id,
                 CompUnit::PrecompilationId $precomp-id)
     {
-        %!loaded{$precomp-id} //= do {
-            my $path = self.path($compiler-id, $precomp-id);
-            $path ~~ :e ?? CompUnit::PrecompilationUnit::File.new(:id($precomp-id), :$path) !! Nil
+        $!update-lock.protect: {
+            %!loaded{$precomp-id} //= do {
+                my $path = self.path($compiler-id, $precomp-id);
+                $path ~~ :e
+                    ?? CompUnit::PrecompilationUnit::File.new(:id($precomp-id), :$path, :store(self))
+                    !! Nil
+            }
         }
     }
 
@@ -140,10 +181,14 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
         }
     }
 
+    method remove-from-cache(CompUnit::PrecompilationId $precomp-id) {
+        $!update-lock.protect: { %!loaded{$precomp-id}:delete };
+    }
+
     method destination(CompUnit::PrecompilationId $compiler-id,
                        CompUnit::PrecompilationId $precomp-id,
-                       Str :$extension = '')
-        returns IO::Path
+                       Str :$extension = ''
+                       --> IO::Path:D)
     {
         unless $!prefix.e {
             $!prefix.mkdir or return;
@@ -155,14 +200,14 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
 
     method !file(CompUnit::PrecompilationId $compiler-id,
                  CompUnit::PrecompilationId $precomp-id,
-                 Str :$extension = '')
-        returns IO::Path
+                 Str :$extension = ''
+                 --> IO::Path:D)
     {
-        my $compiler-dir = self.prefix.child($compiler-id.IO);
+        my $compiler-dir = self.prefix.add($compiler-id);
         $compiler-dir.mkdir unless $compiler-dir.e;
         my $dest = self!dir($compiler-id, $precomp-id);
         $dest.mkdir unless $dest.e;
-        $dest.child(($precomp-id ~ $extension).IO)
+        $dest.add($precomp-id ~ $extension)
     }
 
     method store-file(CompUnit::PrecompilationId $compiler-id,
@@ -180,14 +225,14 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
         my $precomp-file = self!file($compiler-id, $precomp-id, :extension<.tmp>);
         $unit.save-to($precomp-file);
         $precomp-file.rename(self!file($compiler-id, $precomp-id));
-        %!loaded{$precomp-id}:delete;
+        self.remove-from-cache($precomp-id);
     }
 
     method store-repo-id(CompUnit::PrecompilationId $compiler-id,
                  CompUnit::PrecompilationId $precomp-id,
                  :$repo-id!)
     {
-        self!file($compiler-id, $precomp-id, :extension<.repo-id>).spurt($repo-id);
+        try self!file($compiler-id, $precomp-id, :extension<.repo-id>).spurt($repo-id);
     }
 
     method delete(
@@ -200,7 +245,7 @@ class CompUnit::PrecompilationStore::File does CompUnit::PrecompilationStore {
 
     method delete-by-compiler(CompUnit::PrecompilationId $compiler-id)
     {
-         my $compiler-dir = self.prefix.child($compiler-id.IO);
+         my $compiler-dir = self.prefix.add($compiler-id);
          for $compiler-dir.dir -> $subdir {
              $subdir.dir>>.unlink;
              $subdir.rmdir;
