@@ -17,13 +17,13 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
             my $name = $spec.short-name;
             return %!loaded{$name} if %!loaded{$name}:exists;
 
-            my $base := $!prefix.child($name.subst(:g, "::", $*SPEC.dir-sep) ~ '.').Str;
+            my $base := $!prefix.add($name.subst(:g, "::", $*SPEC.dir-sep) ~ '.').Str;
             return $base if %seen{$base}:exists;
             my $found;
 
             # find source file
             # pick a META6.json if it is there
-            if not %!meta and (my $meta = $!prefix.child('META6.json')) and $meta.f {
+            if not %!meta and (my $meta = $!prefix.add('META6.json')) and $meta.f {
                 try {
                     %!meta = Rakudo::Internals::JSON.from-json: $meta.slurp;
                     CATCH {
@@ -35,7 +35,7 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
             }
             if %!meta {
                 if %!meta<provides>{$name} -> $file {
-                    my $path = $file.IO.is-absolute ?? $file.IO !! $!prefix.child($file);
+                    my $path = $file.IO.is-absolute ?? $file.IO !! $!prefix.add($file);
                     $found = $path if $path.f;
                 }
             }
@@ -43,8 +43,8 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
             unless ?$found {
                 # deduce path to compilation unit from package name
                 for @extensions -> $extension {
-                    my $path = $base ~ $extension;
-                    $found = $path.IO if IO::Path.new-from-absolute-path($path).f;
+                    my $path = ($base ~ $extension).IO;
+                    $found = $path if $path.f;
                 }
             }
 
@@ -76,13 +76,8 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
               nqp::until(
                 nqp::eqaddr((my $pulled := $iter.pull-one),IterationEnd),
                 nqp::if(
-                  nqp::filereadable($pulled)
-                    && (my $pio := nqp::open($pulled,'r')),
-                  nqp::stmts(
-                    nqp::setencoding($pio,'iso-8859-1'),
-                    nqp::push_s($parts,nqp::sha1(nqp::readallfh($pio))),
-                    nqp::closefh($pio)
-                  )
+                  nqp::filereadable($pulled),
+                  nqp::push_s($parts,nqp::sha1(slurp($pulled, :enc<iso-8859-1>))),
                 )
               ),
               nqp::if(
@@ -96,7 +91,7 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
         )
     }
 
-    method resolve(CompUnit::DependencySpecification $spec) returns CompUnit {
+    method resolve(CompUnit::DependencySpecification $spec --> CompUnit:D) {
         my ($base, $file) = self!matching-file($spec);
 
         return CompUnit.new(
@@ -118,8 +113,8 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
         CompUnit::DependencySpecification $spec,
         CompUnit::PrecompilationRepository $precomp = self.precomp-repository(),
         CompUnit::PrecompilationStore :@precomp-stores = self!precomp-stores(),
-    )
-        returns CompUnit:D
+
+        --> CompUnit:D)
     {
         my ($base, $file) = self!matching-file($spec);
         if $base {
@@ -153,13 +148,13 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
         X::CompUnit::UnsatisfiedDependency.new(:specification($spec)).throw;
     }
 
-    method load(IO::Path:D $file) returns CompUnit:D {
+    method load(IO::Path:D $file --> CompUnit:D) {
         unless $file.is-absolute {
 
             # We have a $file when we hit: require "PATH" or use/require Foo:file<PATH>;
             my $precompiled =
               $file.Str.ends-with(Rakudo::Internals.PRECOMP-EXT);
-            my $path = $!prefix.child($file);
+            my $path = $!prefix.add($file);
 
             if $path.f {
                 return %!loaded{$file.Str} //= %seen{$path.Str} = CompUnit.new(
@@ -182,7 +177,7 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
 
     method short-id() { 'file' }
 
-    method loaded() returns Iterable {
+    method loaded(--> Iterable:D) {
         return %!loaded.values;
     }
 
@@ -197,16 +192,16 @@ class CompUnit::Repository::FileSystem does CompUnit::Repository::Locally does C
         # We now save the 'resources/' part of a resource's path in files, i.e:
         # "files" : [ "resources/libraries/xxx" => "resources/libraries/xxx.so" ]
         # but we also want to root any path request to the CUR's resources directory
-        $.prefix.parent.child('resources').child($key.subst(/^resources\//, ""));
+        $.prefix.parent.add('resources').add($key.subst(/^resources\//, ""));
     }
 
-    method precomp-store() returns CompUnit::PrecompilationStore {
+    method precomp-store(--> CompUnit::PrecompilationStore:D) {
         $!precomp-store //= CompUnit::PrecompilationStore::File.new(
-            :prefix(self.prefix.child('.precomp')),
+            :prefix(self.prefix.add('.precomp')),
         )
     }
 
-    method precomp-repository() returns CompUnit::PrecompilationRepository {
+    method precomp-repository(--> CompUnit::PrecompilationRepository:D) {
         $!precomp := CompUnit::PrecompilationRepository::Default.new(
             :store(self.precomp-store),
         ) unless $!precomp;
