@@ -5,7 +5,7 @@ role CompUnit::Repository::Locally {
 
     method new(CompUnit::Repository::Locally: Str:D :$prefix, CompUnit::Repository :$next-repo, *%args) {
         my $abspath := $*SPEC.rel2abs($prefix);
-        my $IO      := IO::Path.new-from-absolute-path($abspath);
+        my $IO      := $abspath.IO;
 
         state %instances;
         my $WHICH = self.^name ~ '|' ~ $abspath;
@@ -13,22 +13,22 @@ role CompUnit::Repository::Locally {
           self.bless(:prefix($IO), :lock(Lock.new), :$WHICH, :$next-repo, |%args);
     }
 
-    multi method Str(CompUnit::Repository::Locally:D:) { $!prefix.abspath }
+    multi method Str(CompUnit::Repository::Locally:D:) { $!prefix.absolute }
     multi method gist(CompUnit::Repository::Locally:D:) {
         self.path-spec
     }
     multi method perl(CompUnit::Repository::Locally:D:) {
-        $?CLASS.perl ~ '.new(' ~ $!prefix.abspath.perl ~ ')';
+        $?CLASS.perl ~ '.new(' ~ $!prefix.absolute.perl ~ ')';
     }
 
     multi method WHICH(CompUnit::Repository::Locally:D:) { $!WHICH }
 
     method path-spec(CompUnit::Repository::Locally:D:) {
-        self.short-id ~ '#' ~ $!prefix.abspath;
+        self.short-id ~ '#' ~ $!prefix.absolute;
     }
 
-    method source-file(Str $name --> IO::Path) {
-        self.prefix.child($name)
+    method source-file(Str $name --> IO::Path:D) {
+        self.prefix.add($name)
     }
 
     method prefix { "{$!prefix}".IO }
