@@ -72,7 +72,6 @@ class CompUnit::RepositoryRegistry {
              );
 
         # XXX Various issues with this stuff on JVM , TEMPORARY
-        my Mu $compiler := nqp::getcurhllsym('$COMPILER_CONFIG');
         try {
             if nqp::existskey($ENV,'HOME')
               ?? nqp::atkey($ENV,'HOME')
@@ -189,6 +188,10 @@ class CompUnit::RepositoryRegistry {
           !! Nil
     }
 
+    method register-name($name, CompUnit::Repository $repo) {
+        nqp::bindkey($custom-lib, $name, $repo);
+    }
+
     method name-for-repository(CompUnit::Repository $repo) {
         $*REPO; # initialize if not yet done
         my $iter := nqp::iterator($custom-lib);
@@ -213,6 +216,8 @@ class CompUnit::RepositoryRegistry {
     }
 
     method resolve-unknown-repos(@repos) {
+        # Cannot just use GLOBAL.WHO here as that gives a BOOTHash
+        my $global := nqp::list("GLOBAL");
         for @repos.pairs {
             if nqp::istype($_.value, CompUnit::Repository::Unknown) {
                 my $i = $_.key;
@@ -225,9 +230,7 @@ class CompUnit::RepositoryRegistry {
                 );
                 PROCESS::<$REPO> := $head;
 
-                my $global := nqp::list(); # Cannot just use GLOBAL.WHO here as that gives a BOOTHash
-                nqp::push($global, "GLOBAL");
-                $*W.find_symbol($global).WHO.merge-symbols($comp_unit.handle.globalish-package.WHO);
+                $*W.find_symbol($global).WHO.merge-symbols($comp_unit.handle.globalish-package);
                 my $new-repo = self.repository-for-spec($_.value.path-spec, :$next-repo);
                 if $i > 0 {
                     @repos[$i - 1].next-repo = $new-repo if $i > 0;
