@@ -74,12 +74,16 @@ my class Date does Dateish {
 
     multi method WHICH(Date:D:) {
         nqp::box_s(
-            nqp::concat(
-                nqp::concat(nqp::unbox_s(self.^name), '|'),
-                nqp::unbox_i(self.daycount)
+          nqp::concat(
+            nqp::if(
+              nqp::eqaddr(self.WHAT,Date),
+              'Date|',
+              nqp::concat(nqp::unbox_s(self.^name), '|')
             ),
-            ObjAt
-        );
+            nqp::unbox_i(self.daycount)
+          ),
+          ObjAt
+        )
     }
 
     method truncated-to(Cool $unit) {
@@ -125,7 +129,8 @@ my class Date does Dateish {
           nqp::existskey($h,'year')  ?? nqp::atkey($h,'year')  !! $!year,
           nqp::existskey($h,'month') ?? nqp::atkey($h,'month') !! $!month,
           nqp::existskey($h,'day')   ?? nqp::atkey($h,'day')   !! $!day,
-          :&!formatter,
+          formatter => nqp::existskey($h,'formatter')
+            ?? nqp::atkey($h,'formatter') !! &!formatter,
         )
     }
     method !clone-without-validating(*%_) { # A premature optimization.
@@ -192,10 +197,6 @@ multi sub infix:«>»(Date:D $a, Date:D $b) {
     $a.daycount > $b.daycount
 }
 
-Rakudo::Internals.REGISTER-DYNAMIC: '$*TZ', {
-    PROCESS::<$TZ> = Rakudo::Internals.get-local-timezone-offset();
-}
-
 sub sleep($seconds = Inf --> Nil) {
     # 1e9 seconds is a large enough value that still makes VMs sleep
     # larger values cause nqp::sleep() to exit immediatelly (esp. on 32-bit)
@@ -213,7 +214,7 @@ sub sleep($seconds = Inf --> Nil) {
     }
 }
 
-sub sleep-timer(Real() $seconds = Inf --> Duration) {
+sub sleep-timer(Real() $seconds = Inf --> Duration:D) {
     if $seconds <= 0 {
         Duration.new(0);
     }
@@ -224,7 +225,7 @@ sub sleep-timer(Real() $seconds = Inf --> Duration) {
     }
 }
 
-sub sleep-until(Instant() $until --> Bool) {
+sub sleep-until(Instant() $until --> Bool:D) {
     my $seconds = $until - now;
     return False if $seconds < 0;
 
