@@ -149,9 +149,13 @@ my class DateTime does Dateish {
         my Int $year  = $b * 100 + $d - 4800 + $m div 10;
 
         my $dt = self === DateTime
-          ?? nqp::create(self)!SET-SELF(
-               $year,$month,$day,$hour,$minute,$second,0,&formatter)
-          !! self.bless(
+          ?? ( %_ ?? die "Unexpected named parameter{"s" if %_ > 1} "
+                    ~ %_.keys.map({"`$_`"}).join(", ") ~ " passed. Were you "
+                    ~ "trying to use the named parameter form of .new() but "
+                    ~ "accidentally passed one named parameter as a positional?"
+                  !! nqp::create(self)!SET-SELF(
+                    $year,$month,$day,$hour,$minute,$second,0,&formatter)
+          ) !! self.bless(
                :$year,:$month,:$day,
                :$hour,:$minute,:$second,:timezone(0),:&formatter,|%_);
         $timezone ?? $dt.in-timezone($timezone) !! $dt
@@ -198,7 +202,7 @@ my class DateTime does Dateish {
           $0,$1,$2,$3,$4,+(~$5.subst(",",".")),%_,:$timezone,:&formatter)
     }
 
-    method now(:$timezone=$*TZ, :&formatter) returns DateTime:D {
+    method now(:$timezone=$*TZ, :&formatter --> DateTime:D) {
         self.new(nqp::time_n(), :$timezone, :&formatter)
     }
 
@@ -257,7 +261,7 @@ my class DateTime does Dateish {
     method offset()            { $!timezone }
     method offset-in-minutes() { $!timezone / 60 }
     method offset-in-hours()   { $!timezone / 3600 }
-    
+
     method hh-mm-ss()          { sprintf "%02d:%02d:%02d", $!hour,$!minute,$!second }
 
     method later(:$earlier, *%unit) {
@@ -375,6 +379,10 @@ my class DateTime does Dateish {
     }
 }
 
+Rakudo::Internals.REGISTER-DYNAMIC: '$*TZ', {
+    PROCESS::<$TZ> = Rakudo::Internals.get-local-timezone-offset
+}
+
 multi sub infix:«<»(DateTime:D \a, DateTime:D \b) {
     a.Instant < b.Instant
 }
@@ -399,5 +407,17 @@ multi sub infix:«<=>»(DateTime:D \a, DateTime:D \b) {
 multi sub infix:«cmp»(DateTime:D \a, DateTime:D \b) {
     a.Instant cmp b.Instant
 }
+multi sub infix:<->(DateTime:D \a, DateTime:D \b) {
+    a.Instant - b.Instant
+}
+multi sub infix:<->(DateTime:D \a, Duration:D \b) {
+    a.new(a.Instant - b).in-timezone(a.timezone)
+}
+multi sub infix:<+>(DateTime:D \a, Duration:D \b) {
+    a.new(a.Instant + b).in-timezone(a.timezone)
+}
+multi sub infix:<+>(Duration:D \a, DateTime:D \b) {
+    b.new(b.Instant + a).in-timezone(b.timezone)
+}
 
 # vim: ft=perl6 expandtab sw=4
