@@ -6,7 +6,7 @@ role Distribution {
     #   -   A Distribution may be represented internally by some other
     #       spec (such as using the file system itself for prereqs), as
     #       long as it can also be represented as the META6 hash format
-    method meta(--> Hash) {
+    method meta(--> Hash:D) {
         # Cannot just use ... here as that would break legacy code
         my $class-name = ::?CLASS.^name;
 
@@ -22,7 +22,7 @@ role Distribution {
     #       a socket wants to handle this role currently it would have to wrap `open` or `.slurp-rest`
     #       to handle any protocol negotiation as well as probably saving the data to a tmpfile and
     #       return an IO::Handle to that
-    method content($content-id --> IO::Handle) {
+    method content($content-id --> IO::Handle:D) {
         # Cannot just use ... here as that would break legacy code
         my $class-name = ::?CLASS.^name;
 
@@ -49,7 +49,7 @@ role Distribution {
                 has $.source-url;
                 method auth { $!auth // $!author // $!authority }
                 method ver  { $!ver // $!version }
-                method meta(--> Hash) {
+                method meta(--> Hash:D) {
                     {
                         :$!name,
                         :$.auth,
@@ -68,7 +68,7 @@ role Distribution {
                     ~ ":api<{$.meta<api>   // ''}>";
 
                 }
-                method content($content-id --> IO::Handle) { }
+                method content($content-id --> IO::Handle:D) { }
             }.new(|%_)
             !! self.bless(|%_)
     }
@@ -119,25 +119,31 @@ class Distribution::Hash does Distribution::Locally {
 class Distribution::Path does Distribution::Locally {
     has $!meta;
     submethod BUILD(:$!meta, :$!prefix --> Nil) { }
-    method new(IO::Path $prefix, IO::Path :$file is copy) {
-        my $path := ?$file ?? $file !! $prefix.child('META6.json');
-        die "No meta file located at {$path.abspath}" unless $path.e;
-        my $meta = Rakudo::Internals::JSON.from-json(slurp($path));
+    method new(IO::Path $prefix, IO::Path :$meta-file is copy) {
+        $meta-file //= $prefix.add('META6.json');
+        die "No meta file located at {$meta-file.path}" unless $meta-file.e;
+        my $meta = Rakudo::Internals::JSON.from-json($meta-file.slurp);
 
         # generate `files` (special directories) directly from the file system
-        my @bins = Rakudo::Internals.DIR-RECURSE($prefix.child('bin').absolute);
-        my $resources-dir = $prefix.child('resources');
-        my @resources = ($meta<resources> // []).map: {
-            $_ ~~ m/^libraries\/(.*)/
-                ?? (
-                    "resources/$_" => ~$resources-dir.child('libraries').child(
-                        $*VM.platform-library-name($0.Str.IO)
-                    ).absolute
-                ).hash
-                !! ~$resources-dir.child($_).absolute
-        };
-        my @files = grep *.defined, unique(|$meta<files>, |@bins, |@resources);
-        $meta<files>  = @files;
+        my %bins = Rakudo::Internals.DIR-RECURSE($prefix.add('bin').absolute).map(*.IO).map: -> $real-path {
+            my $name-path = $real-path.is-relative
+                ?? $real-path
+                !! $real-path.relative($prefix);
+            $name-path => $real-path.absolute
+        }
+
+        my $resources-dir = $prefix.add('resources');
+        my %resources = $meta<resources>.grep(*.?chars).map(*.IO).map: -> $path {
+            my $real-path = $path ~~ m/^libraries\/(.*)/
+                ?? $resources-dir.add('libraries').add( $*VM.platform-library-name($0.Str.IO) )
+                !! $resources-dir.add($path);
+            my $name-path = $path.is-relative
+                ?? "resources/{$path}"
+                !! "resources/{$path.relative($prefix)}";
+            $name-path => $real-path.absolute;
+        }
+
+        $meta<files> = |%bins, |%resources;
 
         self.bless(:$meta, :$prefix);
     }
@@ -145,22 +151,111 @@ class Distribution::Path does Distribution::Locally {
 }
 
 role CompUnit::Repository { ... }
+
+class Distribution::Resource {
+    has $.repo;
+    has $.repo-name;
+    has $.dist-id;
+    has $.key;
+
+    method IO() {
+        my $repo := self.repo-name
+            ?? CompUnit::RepositoryRegistry.repository-for-name(self.repo-name)
+            !! CompUnit::RepositoryRegistry.repository-for-spec(self.repo);
+        $repo.resource(self.dist-id, "resources/$.key")
+    }
+
+    method platform-library-name() {
+        my $library = self.IO;
+        ($library ~~ /\.<.alpha>+$/ or $library ~~ /\.so(\.<.digit>+)+$/) #Already a full name?
+            ??  $library
+            !!  $*VM.platform-library-name($library)
+    }
+
+    # delegate appropriate IO::Path methods to the resource IO::Path object
+    method Str(|c) {
+        self.IO.Str(|c)
+    }
+    method gist(|c) {
+        self.IO.gist(|c)
+    }
+    method perl(|c) {
+        self.IO.perl(|c)
+    }
+    method absolute(|c) {
+        self.IO.absolute(|c)
+    }
+    method is-absolute(|c) {
+        self.IO.is-absolute(|c)
+    }
+    method relative(|c) {
+        self.IO.relative(|c)
+    }
+    method is-relative(|c) {
+        self.IO.is-relative(|c)
+    }
+    method parts(|c) {
+        self.IO.parts(|c)
+    }
+    method volume(|c) {
+        self.IO.volume(|c)
+    }
+    method dirname(|c) {
+        self.IO.dirname(|c)
+    }
+    method basename(|c) {
+        self.IO.basename(|c)
+    }
+    method extension(|c) {
+        self.IO.extension(|c)
+    }
+    method open(|c) {
+        self.IO.open(|c)
+    }
+    method resolve(|c) {
+        self.IO.resolve(|c)
+    }
+    method slurp(|c) {
+        self.IO.slurp(|c)
+    }
+    method lines(|c) {
+        self.IO.lines(|c)
+    }
+    method comb(|c) {
+        self.IO.comb(|c)
+    }
+    method split(|c) {
+        self.IO.split(|c)
+    }
+    method words(|c) {
+        self.IO.words(|c)
+    }
+    method copy(|c) {
+        self.IO.copy(|c)
+    }
+}
+
 class Distribution::Resources does Associative {
     has Str $.dist-id;
     has Str $.repo;
+    has Str $.repo-name;
 
     proto method BUILD(|) { * }
 
     multi method BUILD(:$!dist-id, CompUnit::Repository :$repo --> Nil) {
-        $!repo = $repo.path-spec;
+        unless $repo.can('name') and $!repo-name = $repo.name and $!repo-name ne '' {
+            $!repo = $repo.path-spec;
+            $!repo-name = Str;
+        }
     }
 
-    multi method BUILD(:$!dist-id, Str :$!repo --> Nil) { }
+    multi method BUILD(:$!dist-id, :$repo, Str :$!repo-name --> Nil) { }
+    multi method BUILD(:$!dist-id, Str :$!repo, :$repo-name --> Nil) { }
 
     method from-precomp() {
         if %*ENV<RAKUDO_PRECOMP_DIST> -> \dist {
             my %data := Rakudo::Internals::JSON.from-json: dist;
-            self.new(:repo(%data<repo>), :dist-id(%data<dist-id>))
+            self.new(:repo(%data<repo>), :repo-name(%data<repo-name>), :dist-id(%data<dist-id>))
         }
         else {
             Nil
@@ -168,11 +263,11 @@ class Distribution::Resources does Associative {
     }
 
     method AT-KEY($key) {
-        CompUnit::RepositoryRegistry.repository-for-spec($.repo).resource($.dist-id, "resources/$key")
+        Distribution::Resource.new(:$.repo, :$.repo-name, :$.dist-id, :$key)
     }
 
     method Str() {
-        Rakudo::Internals::JSON.to-json: {repo => $.repo.Str, dist-id => $.dist-id};
+        Rakudo::Internals::JSON.to-json: {:$.repo, :$.repo-name, :$.dist-id}
     }
 }
 
