@@ -8,7 +8,7 @@ my class Exception {
     method backtrace(Exception:D:) {
         if $!bt { $!bt }
         elsif nqp::isconcrete($!ex) {
-            nqp::bindattr(self, Exception, '$!bt', Backtrace.new($!ex));
+            $!bt := Backtrace.new($!ex);
         }
         else { '' }
     }
@@ -18,7 +18,7 @@ my class Exception {
         nqp::isconcrete($!ex) && $!bt ?? Backtrace.new($!ex) !! ''
     }
     method reset-backtrace(Exception:D:) {
-        nqp::bindattr(self, Exception, '$!ex', Nil)
+        $!ex := Nil
     }
 
     multi method Str(Exception:D:) {
@@ -51,9 +51,8 @@ my class Exception {
     }
 
     method throw(Exception:D: $bt?) {
-        nqp::bindattr(self, Exception, '$!ex', nqp::newexception())
-            unless nqp::isconcrete($!ex) and $bt;
-        nqp::bindattr(self, Exception, '$!bt', $bt); # Even if !$bt
+        $!ex := nqp::newexception() unless nqp::isconcrete($!ex) and $bt;
+        $!bt := $bt; # Even if !$bt
         nqp::setpayload($!ex, nqp::decont(self));
         my $msg := try self.?message;
         if defined($msg) {
@@ -64,16 +63,14 @@ my class Exception {
         nqp::throw($!ex)
     }
     method rethrow(Exception:D:) {
-        nqp::bindattr(self, Exception, '$!ex', nqp::newexception())
-            unless nqp::isconcrete($!ex);
+        unless nqp::isconcrete($!ex) {
+            $!ex := nqp::newexception();
+            try nqp::setmessage($!ex, self.message);
+        }
         nqp::setpayload($!ex, nqp::decont(self));
         nqp::rethrow($!ex)
     }
 
-    method resumable(Exception:D:) {
-        nqp::p6bool(nqp::istrue(nqp::atkey($!ex, 'resume')));
-    }
-
     method resume(Exception:D: --> True) {
         nqp::resume($!ex);
     }
@@ -86,7 +83,7 @@ my class Exception {
         CATCH { $fail.exception.throw }
     }
 
-    method is-compile-time { False }
+    method is-compile-time(--> False) { }
 }
 
 my class X::SecurityPolicy is Exception {}
@@ -157,14 +154,53 @@ my class X::Method::NotFound is Exception {
     has Bool $.private = False;
     method message() {
         my $message = $.private
-          ?? "No such private method '$.method' for invocant of type '$.typename'"
+          ?? "No such private method '!$.method' for invocant of type '$.typename'"
           !! "No such method '$.method' for invocant of type '$.typename'";
+
+        my %suggestions;
+        my int $max_length = do given $.method.chars {
+            when 0..3 { 1 }
+            when 4..8 { 2 }
+            when 9..* { 3 }
+        }
+
         if $.method eq 'length' {
-            $message ~= "\nDid you mean 'elems', 'chars', 'graphs' or 'codes'?";
+            given $!invocant {
+                when List { %suggestions{$_} = 0 for <elems> }
+                when Cool { %suggestions{$_} = 0 for <chars codes>; }
+                default   { %suggestions{$_} = 0 for <elems chars codes>; }
+            }
+            
         }
         elsif $.method eq 'bytes' {
-            $message ~= "\nDid you mean '.encode(\$encoding).bytes'?";
+            %suggestions<encode($encoding).bytes> = 0;
         }
+
+        if nqp::can($!invocant.HOW, 'methods') {
+            for $!invocant.^methods(:all)>>.name -> $method_name {
+                my $dist = StrDistance.new(:before($.method), :after($method_name));
+                if $dist <= $max_length {
+                    %suggestions{$method_name} = $dist;
+                }
+            }
+        }
+
+        if nqp::can($!invocant.HOW, 'private_method_table') {
+            for $!invocant.^private_method_table.keys -> $method_name {
+                my $dist = StrDistance.new(:before($.method), :after($method_name));
+                if $dist <= $max_length {
+                    %suggestions{"!$method_name"} = $dist;
+                }
+            }
+        }
+
+        if +%suggestions == 1 {
+            $message ~= ". Did you mean '%suggestions.keys()'?";
+        }
+        elsif +%suggestions > 1 {
+            $message ~= ". Did you mean any of these?\n    { %suggestions.sort(*.value)>>.key.head(4).join("\n    ") }\n";
+        }
+
         $message;
     }
 }
@@ -341,25 +377,22 @@ do {
 
         try {
             my $v := $e.vault-backtrace;
-            my Mu $err := nqp::getstderr();
+            my Mu $err := $*ERR;
 
             $e.backtrace;  # This is where most backtraces actually happen
             if $e.is-compile-time || $e.backtrace && $e.backtrace.is-runtime {
-                nqp::printfh($err, $e.gist);
-                nqp::printfh($err, "\n");
-                if $v {
-                   nqp::printfh($err, "Actually thrown at:\n");
-                   nqp::printfh($err, $v.Str);
-                   nqp::printfh($err, "\n");
+                $err.say($e.gist);
+                if $v and !$e.gist.ends-with($v.Str) {
+                   $err.say("Actually thrown at:");
+                   $err.say($v.Str);
                 }
             }
             elsif Rakudo::Internals.VERBATIM-EXCEPTION(0) {
-                nqp::printfh($err, $e.Str);
+                $err.print($e.Str);
             }
             else {
-                nqp::printfh($err, "===SORRY!===\n");
-                nqp::printfh($err, $e.Str);
-                nqp::printfh($err, "\n");
+                $err.say("===SORRY!===");
+                $err.say($e.Str);
             }
             Rakudo::Internals.THE_END();
             CONTROL { when CX::Warn { .resume } }
@@ -378,11 +411,10 @@ do {
           nqp::if(
             nqp::iseq_i($type,nqp::const::CONTROL_WARN),
             nqp::stmts(
-              (my Mu $err := nqp::getstderr),
+              (my Mu $err := $*ERR),
               (my str $msg = nqp::getmessage($ex)),
-              nqp::printfh($err,nqp::if(nqp::chars($msg),$msg,"Warning")),
-              nqp::printfh($err, "\n"),
-              nqp::printfh($err, $backtrace.first-none-setting-line),
+              $err.say(nqp::if(nqp::chars($msg),$msg,"Warning")),
+              $err.print($backtrace.first-none-setting-line),
               nqp::resume($ex)
             )
           )
@@ -465,6 +497,11 @@ my class X::IO::Copy does X::IO {
     }
 }
 
+my class X::IO::Lock does X::IO {
+    has $.lock-type;
+    method message() { "Could not obtain $.lock-type lock: $.os-error" }
+}
+
 my class X::IO::Move does X::IO {
     has $.from;
     has $.to;
@@ -489,6 +526,12 @@ my class X::IO::NotAFile does X::IO {
     }
 }
 
+my class X::IO::Null does X::IO {
+    method message() {
+        "Cannot use null character (U+0000) as part of the path"
+    }
+}
+
 my class X::IO::Directory does X::IO {
     has $.path;
     has $.trying;
@@ -544,6 +587,25 @@ my class X::IO::Cwd does X::IO {
     }
 }
 
+my class X::IO::Flush does X::IO {
+    method message() {
+        "Cannot flush handle: $.os-error"
+    }
+}
+
+my class X::IO::NotAChild does X::IO {
+    has $.path;
+    has $.child;
+    method message() {
+      "Path {$.child.perl} is not a child of path {$.path.perl}"
+    }
+}
+
+my class X::IO::Resolve does X::IO {
+    has $.path;
+    method message() { "Failed to completely resolve {$.path.perl}" }
+}
+
 my class X::IO::Rmdir does X::IO {
     has $.path;
     method message() {
@@ -566,6 +628,15 @@ my class X::IO::Chmod does X::IO {
     }
 }
 
+my class X::IO::BinaryAndEncoding does X::IO {
+    method message { "Cannot open a handle in binary mode (:bin) and also specify an encoding" }
+}
+
+my class X::IO::BinaryMode does X::IO {
+    has $.trying;
+    method message { "Cannot do '$.trying' on a handle in binary mode" }
+}
+
 my role X::Comp is Exception {
     has $.filename;
     has $.pos;
@@ -625,7 +696,7 @@ my class X::Comp::Group is Exception {
     has @.sorrows;
     has @.worries;
 
-    method is-compile-time() { True }
+    method is-compile-time(--> True) { }
 
     multi method gist(::?CLASS:D:) {
         my $r = "";
@@ -695,14 +766,14 @@ my class X::Comp::BeginTime does X::Comp {
 
 # XXX a hack for getting line numbers from exceptions from the metamodel
 my class X::Comp::AdHoc is X::AdHoc does X::Comp {
-    method is-compile-time() { True }
+    method is-compile-time(--> True) { }
 }
 
 my class X::Comp::FailGoal does X::Comp {
     has $.dba;
     has $.goal;
 
-    method is-compile-time() { True }
+    method is-compile-time(--> True) { }
 
     method message { "Unable to parse expression in $.dba; couldn't find final $.goal" }
 }
@@ -712,7 +783,12 @@ my role X::Pod                 { }
 
 my class X::NYI is Exception {
     has $.feature;
-    method message() { "$.feature not yet implemented. Sorry. " }
+    has $.did-you-mean;
+    method message() {
+        my $msg = "$.feature not yet implemented. Sorry.";
+        $msg ~= "\nDid you mean: {$.did-you-mean.gist}?" if $.did-you-mean;
+        $msg
+    }
 }
 my class X::Comp::NYI is X::NYI does X::Comp { };
 my class X::NYI::Available is X::NYI {
@@ -758,14 +834,27 @@ Parenthesize as \\(...) if you intended a capture of a single numeric value./
 my class X::Worry::P5::LeadingZero is X::Worry::P5 {
     has $.value;
     method message {
-        if $!value ~~ /<[89]>/ {
-            "Leading 0 is not allowed. For octals, use '0o' prefix,"
-            ~ " but note that $!value is not a valid octal number";
-        }
-        else {
-            'Leading 0 does not indicate octal in Perl 6.'
-                ~ " Please use 0o$!value if you mean that.";
-        }
+        'Leading 0 has no meaning. If you meant to create an octal number'
+        ~ ", use '0o' prefix" ~ (
+#?if jvm
+            $!value ~~ /<[89]>/
+#?endif
+#?if !jvm
+            $!value.comb.grep(*.unival > 7)
+#?endif
+                ?? ", but note that $!value is not a valid octal number"
+                !! "; like, '0o$!value'"
+        ) ~ '. If you meant to create a string, please add quotation marks.'
+    }
+}
+
+my class X::Trait::Invalid is Exception {
+    has $.type;       # is, will, of etc.
+    has $.subtype;    # wrong subtype being tried
+    has $.declaring;  # variable, sub, parameter, etc.
+    has $.name;       # '$foo', '@bar', etc.
+    method message () {
+        "Cannot use '$.type $.subtype' on $.declaring '$.name'."
     }
 }
 
@@ -1108,6 +1197,26 @@ my class X::Parameter::WrongOrder does X::Comp {
     }
 }
 
+my class X::Parameter::InvalidConcreteness is Exception {
+    has $.expected;
+    has $.got;
+    has $.routine;
+    has $.param;
+    has Bool $.should-be-concrete;
+    has Bool $.param-is-invocant;
+
+    method message() {
+        $!routine = '<anon>' if not $!routine.defined or $!routine eq '';
+        $!param   = '<anon>' if not $!param.defined   or $!param   eq '';
+        my $beginning  = $!param-is-invocant  ?? 'Invocant of method' !! "Parameter '$!param' of routine";
+        my $must-be    = $!should-be-concrete ?? 'an object instance' !! 'a type object';
+        my $not-a      = $!should-be-concrete ?? 'a type object'      !! 'an object instance';
+        my $suggestion = $!should-be-concrete ?? '.new'               !! 'multi';
+
+        "$beginning '$!routine' must be $must-be of type '$!expected', not $not-a of type '$!got'.  Did you forget a '$suggestion'?"
+    }
+}
+
 my class X::Parameter::InvalidType does X::Comp {
     has $.typename;
     has @.suggestions;
@@ -1229,7 +1338,8 @@ my class X::Invalid::ComputedValue is Exception {
     has $.value;
     has $.reason;
     method message {
-        "$.name on $.method computed to $.value, which cannot be used"
+        "$.name {"on $.method " if $.method}computed to $.value,"
+            ~ " which cannot be used"
             ~ (" because $.reason" if $.reason);
     }
 }
@@ -1447,7 +1557,7 @@ my class X::Syntax::Perl5Var does X::Syntax {
       '$^I' => '$*INPLACE',
       '$^M' => 'a global form such as $*M',
       '$^N' => '$/[*-1]',
-      '$^O' => '$?DISTRO.name or $*DISTRO.name',
+      '$^O' => 'VM.osname',
       '$^R' => 'an explicit result variable',
       '$^S' => 'context function',
       '$^T' => '$*INITTIME',
@@ -1812,6 +1922,14 @@ my class X::Phaser::PrePost is Exception {
     }
 }
 
+my class X::Str::InvalidCharName is Exception {
+    has $.name;
+    method message() {
+        $!name.chars ?? "Unrecognized character name [{$!name}]"
+                     !! "Cannot use empty name as character name"
+    }
+}
+
 my class X::Str::Numeric is Exception {
     has $.source;
     has $.pos;
@@ -1986,16 +2104,13 @@ my class X::TypeCheck is Exception {
         ) // "?"
     }
     method priors() {
-        my $prior = do if nqp::isconcrete($!got) && $!got ~~ Failure {
-            "Earlier failure:\n " ~ $!got.mess ~ "\nFinal error:\n ";
-        }
-        else { '' }
-        $prior;
+        (try nqp::isconcrete($!got) && $!got ~~ Failure)
+          ?? "Earlier failure:\n " ~ $!got.mess ~ "\nFinal error:\n "
+          !! ''
     }
     method message() {
         self.priors() ~
         "Type check failed in $.operation; expected $.expectedn but got $.gotn";
-
     }
 }
 
@@ -2004,13 +2119,33 @@ my class X::TypeCheck::Binding is X::TypeCheck {
     method operation { 'binding' }
     method message() {
         my $to = $.symbol.defined && $.symbol ne '$'
-            ?? " to $.symbol" !! "";
-        my $expected = $.expected =:= $.got
+            ?? " to '$.symbol'"
+            !! "";
+        my $expected = (try nqp::eqaddr($.expected,$.got))
             ?? "expected type $.expectedn cannot be itself"
             !! "expected $.expectedn but got $.gotn";
         self.priors() ~ "Type check failed in $.operation$to; $expected";
     }
 }
+my class X::TypeCheck::Binding::Parameter is X::TypeCheck::Binding {
+    has Parameter $.parameter;
+    has Bool $.constraint;
+    method expectedn() {
+        $.constraint && $.expected ~~ Code
+            ?? 'anonymous constraint to be met'
+            !! callsame()
+    }
+    method message() {
+        my $to = $.symbol.defined && $.symbol ne '$'
+            ?? " to parameter '$.symbol'"
+            !! " to anonymous parameter";
+        my $expected = (try nqp::eqaddr($.expected,$.got))
+            ?? "expected type $.expectedn cannot be itself"
+            !! "expected $.expectedn but got $.gotn";
+        my $what-check = $.constraint ?? 'Constraint type' !! 'Type';
+        self.priors() ~ "$what-check check failed in $.operation$to; $expected";
+    }
+}
 my class X::TypeCheck::Return is X::TypeCheck {
     method operation { 'returning' }
     method message() {
@@ -2060,10 +2195,11 @@ my class X::TypeCheck::Splice is X::TypeCheck does X::Comp {
 }
 
 my class X::Assignment::RO is Exception {
-    has $.typename = "value";
+    has $.value = "value";
     method message {
-        "Cannot modify an immutable {$.typename}";
+        "Cannot modify an immutable {$.value.^name} ({$.value.gist})"
     }
+    method typename { $.value.^name } 
 }
 
 my class X::Assignment::RO::Comp does X::Comp {
@@ -2241,10 +2377,12 @@ my class X::Numeric::Real is Exception {
 
 my class X::Numeric::DivideByZero is Exception {
     has $.using;
+    has $.details;
     has $.numerator;
     method message() {
         "Attempt to divide{$.numerator ?? " $.numerator" !! ''} by zero"
-          ~ ( $.using ?? " using $.using" !! '' );
+          ~ ( $.using ?? " using $.using" !! '' )
+          ~ ( $_ with $.details );
     }
 }
 
@@ -2343,7 +2481,7 @@ my class X::Multi::NoMatch is Exception {
         else {
             @bits.push('...');
         }
-        if @cand[0] ~~ /': '/ {
+        if @cand && @cand[0] ~~ /': '/ {
             my $invocant = @bits.shift;
             my $first = @bits ?? @bits.shift !! '';
             @bits.unshift($invocant ~ ': ' ~ $first);
@@ -2358,9 +2496,10 @@ my class X::Multi::NoMatch is Exception {
                         ~  join("\n    ", '', sort keys @cand ∖ @un-rw-cand)
                     unless @cand == @un-rw-cand
                 )
-            !! join "\n    ",
-                'none of these signatures match:',
-                @cand
+            !! ( @cand
+                 ??  join "\n    ", 'none of these signatures match:', @cand
+                 !! "Routine does not have any candidates. Is only the proto defined?"
+               )
         );
     }
 }
@@ -2397,64 +2536,87 @@ my class X::PhaserExceptions is Exception {
     }
 }
 
+
+#?if jvm
 nqp::bindcurhllsym('P6EX', nqp::hash(
+#?endif
+#?if !jvm
+nqp::bindcurhllsym('P6EX', BEGIN nqp::hash(
+#?endif
   'X::TypeCheck::Binding',
-  sub (Mu $got, Mu $expected, $symbol?) {
+  -> Mu $got, Mu $expected, $symbol? {
       X::TypeCheck::Binding.new(:$got, :$expected, :$symbol).throw;
   },
+  'X::TypeCheck::Binding::Parameter',
+  -> Mu $got, Mu $expected, $symbol, $parameter, $is-constraint? {
+      my $constraint = $is-constraint ?? True !! False;
+      X::TypeCheck::Binding::Parameter.new(:$got, :$expected, :$symbol, :$parameter, :$constraint).throw;
+  },
   'X::TypeCheck::Assignment',
-  sub (Mu $symbol, Mu $got, Mu $expected) {
+  -> Mu $symbol, Mu $got, Mu $expected {
       X::TypeCheck::Assignment.new(:$symbol, :$got, :$expected).throw;
   },
   'X::TypeCheck::Return',
-  sub (Mu $got, Mu $expected) {
+  -> Mu $got, Mu $expected {
       X::TypeCheck::Return.new(:$got, :$expected).throw;
   },
   'X::Assignment::RO',
-  sub ($typename = "value") {
-      X::Assignment::RO.new(:$typename).throw;
+  -> $value = "value" {
+      X::Assignment::RO.new(:$value).throw;
   },
   'X::ControlFlow::Return',
-  sub () {
+  {
       X::ControlFlow::Return.new().throw;
   },
   'X::NoDispatcher',
-  sub ($redispatcher) {
+  -> $redispatcher {
       X::NoDispatcher.new(:$redispatcher).throw;
   },
   'X::Method::NotFound',
-  sub (Mu $invocant, $method, $typename, $private = False) {
+  -> Mu $invocant, $method, $typename, $private = False {
       X::Method::NotFound.new(:$invocant, :$method, :$typename, :$private).throw
   },
   'X::Multi::Ambiguous',
-  sub ($dispatcher, @ambiguous, $capture) {
+  -> $dispatcher, @ambiguous, $capture {
       X::Multi::Ambiguous.new(:$dispatcher, :@ambiguous, :$capture).throw
   },
   'X::Multi::NoMatch',
-  sub ($dispatcher, $capture) {
+  -> $dispatcher, $capture {
       X::Multi::NoMatch.new(:$dispatcher, :$capture).throw
   },
   'X::Role::Initialization',
-  sub ($role) {
+  -> $role {
       X::Role::Initialization.new(:$role).throw
   },
   'X::Role::Parametric::NoSuchCandidate',
-  sub (Mu $role) {
+  -> Mu $role {
       X::Role::Parametric::NoSuchCandidate.new(:$role).throw;
   },
   'X::Inheritance::NotComposed',
-  sub ($child-name, $parent-name) {
+  -> $child-name, $parent-name {
       X::Inheritance::NotComposed.new(:$child-name, :$parent-name).throw;
   },
   'X::Parameter::RW',
-  sub (Mu $got, $symbol) {
+  -> Mu $got, $symbol {
       X::Parameter::RW.new(:$got, :$symbol).throw;
   },
   'X::PhaserExceptions',
-  sub (@exceptions) {
+  -> @exceptions {
       X::PhaserExceptions.new(exceptions =>
         @exceptions.map(-> Mu \e { EXCEPTION(e) })).throw;
   },
+  'X::Trait::Invalid',
+  -> $type, $subtype, $declaring, $name {
+      X::Trait::Invalid.new(:$type, :$subtype, :$declaring, :$name).throw;
+  },
+  'X::Parameter::InvalidConcreteness',
+  -> $expected, $got, $routine, $param, Bool() $should-be-concrete, Bool() $param-is-invocant {
+      X::Parameter::InvalidConcreteness.new(:$expected, :$got, :$routine, :$param, :$should-be-concrete, :$param-is-invocant).throw;
+  },
+  'X::NYI',
+  -> $feature {
+      X::NYI.new(:$feature).throw;
+  },
 ));
 
 my class X::HyperWhatever::Multiple is Exception {
@@ -2657,8 +2819,7 @@ my class X::CompUnit::UnsatisfiedDependency is Exception {
 
 my class Exceptions::JSON {
     method process($ex) {
-        nqp::printfh(
-          nqp::getstderr,
+        $*ERR.print:
           Rakudo::Internals::JSON.to-json( $ex.^name => Hash.new(
             (message => $ex.?message),
             $ex.^attributes.grep(*.has_accessor).map: {
@@ -2669,8 +2830,7 @@ my class Exceptions::JSON {
                     ) given $ex."$attr"()
                 }
             }
-          ))
-        );
+          ));
         False  # done processing
     }
 }
