@@ -171,7 +171,7 @@ my class Hash { # declared in BOOTSTRAP
           nqp::unbox_s(nqp::istype(key,Str) ?? key !! key.Str), bindval)
     }
 
-    multi method DELETE-KEY(Hash:U:) { Nil }
+    multi method DELETE-KEY(Hash:U: --> Nil) { }
     multi method DELETE-KEY(Hash:D: Str:D \key) {
         nqp::if(
           (nqp::getattr(self,Map,'$!storage').DEFINITE
@@ -224,15 +224,21 @@ my class Hash { # declared in BOOTSTRAP
         })
     }
 
-    multi method DUMP(Hash:D: :$indent-step = 4, :%ctx?) {
-        return DUMP(self, :$indent-step) unless %ctx;
-
-        my Mu $attrs := nqp::list();
-        nqp::push($attrs, '$!descriptor');
-        nqp::push($attrs,  $!descriptor );
-        nqp::push($attrs, '$!storage'   );
-        nqp::push($attrs,  nqp::getattr(nqp::decont(self), Map, '$!storage'));
-        self.DUMP-OBJECT-ATTRS($attrs, :$indent-step, :%ctx);
+    multi method DUMP(Hash:D: :$indent-step = 4, :%ctx) {
+        nqp::if(
+          %ctx,
+          self.DUMP-OBJECT-ATTRS(
+            nqp::list(
+              '$!descriptor',
+              $!descriptor,
+              '$!storage',
+              nqp::getattr(nqp::decont(self),Map,'$!storage')
+            ),
+            :$indent-step,
+            :%ctx
+          ),
+          DUMP(self, :$indent-step)
+        )
     }
 
     # introspection
@@ -249,7 +255,7 @@ my class Hash { # declared in BOOTSTRAP
         nqp::isnull($!descriptor) ?? Any !! $!descriptor.default
     }
     method dynamic() {
-        nqp::isnull($!descriptor) ?? Nil !! nqp::p6bool($!descriptor.dynamic)
+        nqp::isnull($!descriptor) ?? False !! nqp::p6bool($!descriptor.dynamic)
     }
 
     method push(+values) {
@@ -613,10 +619,9 @@ my class Hash { # declared in BOOTSTRAP
         }
 
         method EXISTS-KEY(TKey \key) {
-            nqp::if(
-              nqp::getattr(self,Map,'$!storage').DEFINITE,
-              nqp::p6bool(nqp::existskey(
-                nqp::getattr(self,Map,'$!storage'),nqp::unbox_s(key.WHICH)))
+            nqp::p6bool(
+              nqp::defined(nqp::getattr(self,Map,'$!storage'))
+                && nqp::existskey(nqp::getattr(self,Map,'$!storage'),key.WHICH)
             )
         }
 
@@ -636,9 +641,36 @@ my class Hash { # declared in BOOTSTRAP
               TValue
             )
         }
+        method IterationBuffer() {
+            nqp::stmts(
+              (my $buffer := nqp::create(IterationBuffer)),
+              nqp::if(
+                nqp::defined(
+                  nqp::getattr(self,Map,'$!storage')
+                ) && nqp::elems(
+                  nqp::getattr(self,Map,'$!storage')
+                ),
+                nqp::stmts(
+                  (my $iterator := nqp::iterator(
+                    nqp::getattr(self,Map,'$!storage')
+                  )),
+                  nqp::setelems($buffer,nqp::elems(
+                    nqp::getattr(self,Map,'$!storage')
+                  )),
+                  (my int $i = -1),
+                  nqp::while(
+                    $iterator,
+                    nqp::bindpos($buffer,($i = nqp::add_i($i,1)),
+                      nqp::iterval(nqp::shift($iterator)))
+                  )
+                )
+              ),
+              $buffer
+            )
+        }
 
         method keys() {
-            Seq.new(class :: does Rakudo::Internals::MappyIterator {
+            Seq.new(class :: does Rakudo::Iterator::Mappy {
                 method pull-one() {
                     nqp::if(
                       $!iter,
@@ -650,7 +682,7 @@ my class Hash { # declared in BOOTSTRAP
             }.new(self))
         }
         method values() {
-            Seq.new(class :: does Rakudo::Internals::MappyIterator {
+            Seq.new(class :: does Rakudo::Iterator::Mappy {
                 method pull-one() {
                     nqp::if(
                       $!iter,
@@ -662,33 +694,11 @@ my class Hash { # declared in BOOTSTRAP
             }.new(self))
         }
         method kv() {
-            Seq.new(class :: does Rakudo::Internals::MappyIterator {
-                has $!pair;
-
-                method pull-one() {
-                    nqp::if(
-                      $!pair,
-                      nqp::stmts(
-                        (my $value := nqp::getattr($!pair,Pair,'$!value')),
-                        ($!pair := nqp::null),
-                        $value
-                      ),
-                      nqp::if(
-                        $!iter,
-                        nqp::getattr(
-                          ($!pair := nqp::iterval(nqp::shift($!iter))),
-                          Pair,'$!key'),
-                        IterationEnd
-                      )
-                    )
-                }
-            }.new(self))
-        }
-        method pairs() {
-            Seq.new(Rakudo::Internals::MappyIterator-values.new(self))
+            Seq.new(Rakudo::Iterator.Mappy-kv-from-pairs(self))
         }
+        method iterator() { Rakudo::Iterator.Mappy-values(self) }
         method antipairs() {
-            Seq.new(class :: does Rakudo::Internals::MappyIterator {
+            Seq.new(class :: does Rakudo::Iterator::Mappy {
                 method pull-one() {
                     nqp::if(
                       $!iter,
@@ -698,9 +708,6 @@ my class Hash { # declared in BOOTSTRAP
                  }
             }.new(self))
         }
-        method invert() {
-            self.map: { .value »=>» .key }
-        }
         multi method perl(::?CLASS:D \SELF:) {
             SELF.perlseen('Hash', {
                 my $TKey-perl   := TKey.perl;
@@ -732,6 +739,16 @@ my class Hash { # declared in BOOTSTRAP
               !! nqp::create(Capture)
         }
         method Map() { self.pairs.Map }
+
+        sub SETIFY(\objecthash, \type) {
+            nqp::create(type).SET-SELF(
+              Rakudo::QuantHash.ADD-OBJECTHASH-TO-SET(
+                nqp::create(Rakudo::Internals::IterationSet), objecthash
+              )
+            )
+        }
+        multi method Set(::?CLASS:D:)     { SETIFY(self,Set)     }
+        multi method SetHash(::?CLASS:D:) { SETIFY(self,SetHash) }
     }
     method ^parameterize(Mu:U \hash, Mu:U \t, |c) {
         if c.elems == 0 {
