@@ -11,12 +11,16 @@ my class Int does Real { # declared in BOOTSTRAP
 
     multi method WHICH(Int:D:) {
         nqp::box_s(
-            nqp::concat(
-                nqp::concat(nqp::unbox_s(self.^name), '|'),
-                nqp::tostr_I(self)
+          nqp::concat(
+            nqp::if(
+              nqp::eqaddr(self.WHAT,Int),
+              'Int|',
+              nqp::concat(nqp::unbox_s(self.^name), '|')
             ),
-            ObjAt
-        );
+            nqp::tostr_I(self)
+          ),
+          ObjAt
+        )
     }
     multi method new($value) {
         # clone to ensure we return a new object for any cached
@@ -56,7 +60,11 @@ my class Int does Real { # declared in BOOTSTRAP
     }
 
     method chr(Int:D:) {
-        nqp::p6box_s(nqp::chr(nqp::unbox_i(self)));
+        nqp::if(
+          nqp::isbig_I(self),
+          (die "chr codepoint too large: {self}"),
+          nqp::p6box_s(nqp::chr(nqp::unbox_i(self)))
+        )
     }
 
     method sqrt(Int:D:) { nqp::p6box_n(nqp::sqrt_n(nqp::tonum_I(self))) }
@@ -73,7 +81,7 @@ my class Int does Real { # declared in BOOTSTRAP
           ?? $digits && ! nqp::istype($digits, Whatever)
             ?? $digits < 0
               ?? Failure.new(X::OutOfRange.new(
-                   :what('digits argument to base'),:got($digits),:range<0..*>))
+                   :what('digits argument to base'),:got($digits),:range<0..1073741824>))
               !!  nqp::p6box_s(nqp::base_I(self,nqp::unbox_i($base)))
                     ~ '.'
                     ~ '0' x $digits
@@ -86,7 +94,7 @@ my class Int does Real { # declared in BOOTSTRAP
     # If do-not-want, user should cast invocant to proper domain.
     method polymod(Int:D: +@mods) {
         fail X::OutOfRange.new(
-          what => 'invocant to polymod', got => self, range => "0..*"
+          :what('invocant to polymod'), :got(self), :range<0..^Inf>
         ) if self < 0;
 
         gather {
@@ -119,7 +127,7 @@ my class Int does Real { # declared in BOOTSTRAP
     method expmod(Int:D: Int:D \base, Int:D \mod) {
         nqp::expmod_I(self, nqp::decont(base), nqp::decont(mod), Int);
     }
-    method is-prime(Int:D:) returns Bool:D {
+    method is-prime(Int:D: --> Bool:D) {
         nqp::p6bool(nqp::isprime_I(self, nqp::unbox_i(100)));
     }
 
@@ -154,11 +162,6 @@ my class Int does Real { # declared in BOOTSTRAP
 
     method narrow(Int:D:) { self }
 
-    my constant $?BITS = do {
-        my int $a = 0x1ffffffff;
-        nqp::iseq_i($a,8589934591) ?? 64 !! 32;
-    }
-
     method Range(Int:U:) {
         given self {
             when int  { $?BITS == 64 ??  int64.Range !!  int32.Range }
@@ -177,19 +180,16 @@ my class Int does Real { # declared in BOOTSTRAP
             when uint32 { Range.new( 0, 4294967295           ) }
             when uint16 { Range.new( 0, 65535                ) }
             when uint8  { Range.new( 0, 255                  ) }
+            when byte   { Range.new( 0, 255                  ) }
             # Bring back in a future Perl 6 version, or just put on the type object
             #when uint4  { Range.new( 0, 15                   ) }
             #when uint2  { Range.new( 0, 3                    ) }
             #when uint1  { Range.new( 0, 1                    ) }
 
-            when Int    {  # smartmatch matches both UInt and Int
+            default {  # some other kind of Int
                 .^name eq 'UInt'
                   ?? Range.new(    0, Inf, :excludes-max )
                   !! Range.new( -Inf, Inf, :excludes-min, :excludes-max )
-                }
-
-            default {
-                fail "Unknown integer type: {self.^name}";
             }
         }
     }
@@ -208,7 +208,7 @@ multi sub prefix:<-->(int $a is rw) {
     $a = nqp::sub_i($a, 1);
 }
 multi sub postfix:<++>(Int:D $a is rw) {
-    my \b = nqp::decont($a);
+    my \b := nqp::decont($a);
     $a = nqp::add_I(b, 1, Int);
     b
 }
@@ -218,7 +218,7 @@ multi sub postfix:<++>(int $a is rw) {
     $b
 }
 multi sub postfix:<-->(Int:D $a is rw) {
-    my \b = nqp::decont($a);
+    my \b := nqp::decont($a);
     $a = nqp::sub_I(b, 1, Int);
     b
 }
@@ -228,38 +228,38 @@ multi sub postfix:<-->(int $a is rw) {
     $b
 }
 
-multi sub prefix:<->(Int:D \a) returns Int {
+multi sub prefix:<->(Int:D \a --> Int:D) {
     nqp::neg_I(nqp::decont(a), Int);
 }
-multi sub prefix:<->(int $a) returns int {
+multi sub prefix:<->(int $a --> int) {
     nqp::neg_i($a)
 }
 
-multi sub abs(Int:D \a) returns Int:D {
+multi sub abs(Int:D \a --> Int:D) {
     nqp::abs_I(nqp::decont(a), Int);
 }
-multi sub abs(int $a) returns int {
+multi sub abs(int $a --> int) {
     nqp::abs_i($a)
 }
 
-multi sub infix:<+>(Int:D \a, Int:D \b) returns Int:D {
+multi sub infix:<+>(Int:D \a, Int:D \b --> Int:D) {
     nqp::add_I(nqp::decont(a), nqp::decont(b), Int);
 }
-multi sub infix:<+>(int $a, int $b) returns int {
+multi sub infix:<+>(int $a, int $b --> int) {
     nqp::add_i($a, $b)
 }
 
-multi sub infix:<->(Int:D \a, Int:D \b) returns Int:D {
+multi sub infix:<->(Int:D \a, Int:D \b --> Int:D) {
     nqp::sub_I(nqp::decont(a), nqp::decont(b), Int);
 }
-multi sub infix:<->(int $a, int $b) returns int {
+multi sub infix:<->(int $a, int $b --> int) {
     nqp::sub_i($a, $b)
 }
 
-multi sub infix:<*>(Int:D \a, Int:D \b) returns Int {
+multi sub infix:<*>(Int:D \a, Int:D \b --> Int:D) {
     nqp::mul_I(nqp::decont(a), nqp::decont(b), Int);
 }
-multi sub infix:<*>(int $a, int $b) returns int {
+multi sub infix:<*>(int $a, int $b --> int) {
     nqp::mul_i($a, $b);
 }
 
@@ -268,43 +268,61 @@ multi sub infix:<div>(Int:D \a, Int:D \b) {
       ?? nqp::div_I(nqp::decont(a), nqp::decont(b), Int)
       !! Failure.new(X::Numeric::DivideByZero.new(:using<div>, :numerator(a)))
 }
-multi sub infix:<div>(int $a, int $b) returns int {
+multi sub infix:<div>(int $a, int $b --> int) {
     # relies on opcode or hardware to detect division by 0
     nqp::div_i($a, $b)
 }
 
-multi sub infix:<%>(Int:D \a, Int:D \b) returns Int {
-    b
-      ?? nqp::mod_I(nqp::decont(a), nqp::decont(b), Int)
-      !! Failure.new(X::Numeric::DivideByZero.new(:using<%>, :numerator(a)))
+multi sub infix:<%>(Int:D \a, Int:D \b --> Int:D) {
+    nqp::if(
+      nqp::isbig_I(nqp::decont(a)) || nqp::isbig_I(nqp::decont(b)),
+      nqp::if(
+        b,
+        nqp::mod_I(nqp::decont(a),nqp::decont(b),Int),
+        Failure.new(X::Numeric::DivideByZero.new(:using<%>, :numerator(a)))
+      ),
+      nqp::if(
+        nqp::isne_i(b,0),
+        nqp::mod_i(    # quick fix RT #128318
+          nqp::add_i(nqp::mod_i(nqp::decont(a),nqp::decont(b)),b),
+          nqp::decont(b)
+        ),
+        Failure.new(X::Numeric::DivideByZero.new(:using<%>, :numerator(a)))
+      )
+    )
 }
-multi sub infix:<%>(int $a, int $b) returns int {
+multi sub infix:<%>(int $a, int $b --> int) {
     # relies on opcode or hardware to detect division by 0
     nqp::mod_i(nqp::add_i(nqp::mod_i($a,$b),$b),$b) # quick fix RT #128318
 }
 
 multi sub infix:<**>(Int:D \a, Int:D \b) {
-    my \power = nqp::pow_I(nqp::decont(a), nqp::decont(b >= 0 ?? b !! -b), Num, Int);
+    my $power := nqp::pow_I(nqp::decont(a), nqp::decont(b >= 0 ?? b !! -b), Num, Int);
     # when a**b is too big nqp::pow_I returns Inf
-    nqp::istype(power, Num) ?? Failure.new(X::Numeric::Overflow.new) !!
-        (b >= 0 ?? power !! 1 / power);
+    nqp::istype($power, Num)
+        ?? Failure.new(
+            b >= 0 ?? X::Numeric::Overflow.new !! X::Numeric::Underflow.new
+        ) !! b >= 0 ?? $power
+            !! ($power := 1 / $power) == 0 && a != 0
+                ?? Failure.new(X::Numeric::Underflow.new)
+                    !! $power;
 }
 
-multi sub infix:<**>(int $a, int $b) returns int {
+multi sub infix:<**>(int $a, int $b --> int) {
     nqp::pow_i($a, $b);
 }
 
-multi sub infix:<lcm>(Int:D \a, Int:D \b) returns Int {
+multi sub infix:<lcm>(Int:D \a, Int:D \b --> Int:D) {
     nqp::lcm_I(nqp::decont(a), nqp::decont(b), Int);
 }
-multi sub infix:<lcm>(int $a, int $b) returns int {
+multi sub infix:<lcm>(int $a, int $b --> int) {
     nqp::lcm_i($a, $b)
 }
 
-multi sub infix:<gcd>(Int:D \a, Int:D \b) returns Int {
+multi sub infix:<gcd>(Int:D \a, Int:D \b --> Int:D) {
     nqp::gcd_I(nqp::decont(a), nqp::decont(b), Int);
 }
-multi sub infix:<gcd>(int $a, int $b) returns int {
+multi sub infix:<gcd>(int $a, int $b --> int) {
     nqp::gcd_i($a, $b)
 }
 
@@ -379,21 +397,21 @@ multi sub infix:<+^>(Int:D \a, Int:D \b) {
 #    nqp::bitxor_i($a, $b);
 #}
 
-multi sub infix:«+<»(Int:D \a, Int:D \b) returns Int:D {
-    nqp::isbig_I(nqp::decont(b))
-      ?? Failure.new(X::NYI::BigInt.new(:op('+<'),:big(b)))
-      !! nqp::bitshiftl_I(nqp::decont(a), nqp::unbox_i(b), Int)
+multi sub infix:«+<»(Int:D \a, Int:D \b --> Int:D) {
+#?if moar
+    my $ignore = nqp::ctx(); # <-- kludge for RT#131306
+#?endif
+    nqp::bitshiftl_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
 #multi sub infix:«+<»(int $a, int $b) { RT#128655
 #    nqp::bitshiftl_i($a, $b);
 #}
 
-multi sub infix:«+>»(Int:D \a, Int:D \b) returns Int:D {
-    nqp::isbig_I(nqp::decont(b))
-      ?? Failure.new(X::NYI::BigInt.new(:op('+>'),:big(b)))
-      !! a < 0 && b > 31
-        ?? -1 # temp fix for RT#126942, remove if fixed otherwise
-        !! nqp::bitshiftr_I(nqp::decont(a), nqp::unbox_i(b), Int)
+multi sub infix:«+>»(Int:D \a, Int:D \b --> Int:D) {
+#?if moar
+    my $ignore = nqp::ctx(); # <-- kludge for RT#131306
+#?endif
+    nqp::bitshiftr_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
 #multi sub infix:«+>»(int $a, int $b) { RT#128655
 #    nqp::bitshiftr_i($a, $b)
@@ -407,9 +425,9 @@ multi sub prefix:<+^>(Int:D \a) {
 #}
 
 proto sub chr($) is pure  {*}
-multi sub chr(Int:D  \x) returns Str:D { x.chr     }
-multi sub chr(Cool \x) returns Str:D { x.Int.chr }
-multi sub chr(int $x) returns str {
+multi sub chr(Int:D  \x --> Str:D) { x.chr     }
+multi sub chr(Cool \x --> Str:D) { x.Int.chr }
+multi sub chr(int $x --> str) {
     nqp::chr($x);
 }
 
