@@ -13,7 +13,7 @@ my sub MAIN_HELPER($retval = 0) {
     my $m = callframe(1).my<&MAIN>;
     return $retval unless $m;
 
-    my $no-named-after = !$*MAIN-ALLOW-NAMED-ANYWHERE; 
+    my $no-named-after = !$*MAIN-ALLOW-NAMED-ANYWHERE;
 
     sub thevalue(\a) {
         ((my $type := ::(a)) andthen Metamodel::EnumHOW.ACCEPTS($type.HOW))
@@ -23,7 +23,7 @@ my sub MAIN_HELPER($retval = 0) {
 
     # Convert raw command line args into positional and named args for MAIN
     my sub process-cmd-args(@args is copy) {
-        my $positional := nqp::list;
+        my $positional := nqp::create(IterationBuffer);
         my %named;
 
         while ?@args {
@@ -53,7 +53,7 @@ my sub MAIN_HELPER($retval = 0) {
                       ?? thevalue(nqp::join("=",$split)) but False
                       !! thevalue(nqp::join("=",$split));
                 }
-                
+
                 # implicit value
                 else {
                     %named.push: $arg => !($0.chars);
@@ -109,8 +109,10 @@ my sub MAIN_HELPER($retval = 0) {
                 my $argument;
                 if $param.named {
                     if $param.slurpy {
-                        $argument  = "--<$param.usage-name()>=...";
-                        @optional-named.push("[$argument]");
+                        if $param.name { # ignore anon *%
+                            $argument  = "--<$param.usage-name()>=...";
+                            @optional-named.push("[$argument]");
+                        }
                     }
                     else {
                         my @names  = $param.named_names.reverse;
@@ -139,8 +141,10 @@ my sub MAIN_HELPER($retval = 0) {
                                 $simple-const ??       $constraints                !!
                                                  '<' ~ $param.type.^name     ~ '>' ;
 
-                    $argument = "[$argument ...]" if $param.slurpy;
-                    $argument = "[$argument]"     if $param.optional;
+                    $argument  = "[$argument ...]"          if $param.slurpy;
+                    $argument  = "[$argument]"              if $param.optional;
+                    $argument .= trans(["'"] => [q|'"'"'|]) if $argument.contains("'");
+                    $argument  = "'$argument'"              if $argument.contains(' ' | '"');
                     @positional.push($argument);
                 }
                 @arg-help.push($argument => $param.WHY.contents) if $param.WHY and (@arg-help.grep:{ .key eq $argument}) == Empty;  # Use first defined
