@@ -3,6 +3,17 @@ my class Mix does Mixy {
     has Real $!total;
 
 #--- interface methods
+    method SET-SELF(Mix:D: \elems) {
+        nqp::if(
+          nqp::elems(elems),
+          nqp::stmts(                 # need to have allocated %!elems
+            nqp::bindattr(%!elems,Map,'$!storage',elems),
+            self
+          ),
+          mix()
+        )
+    }
+
     multi method DELETE-KEY(Mix:D: \k) {
         X::Immutable.new(method => 'DELETE-KEY', typename => self.^name).throw;
     }
@@ -15,27 +26,35 @@ my class Mix does Mixy {
           $!WHICH := self!WHICH
         )
     }
-    method total(Mix:D: --> Real) {
+    method total(Mix:D: --> Real:D) {
         nqp::if(
           nqp::attrinited(self,Mix,'$!total'),
           $!total,
-          $!total := self!TOTAL
+          $!total := Rakudo::QuantHash.MIX-TOTAL(self.raw_hash)
+        )
+    }
+
+#--- object creation methods
+    multi method new(Mix:_:) {
+        nqp::if(
+          nqp::eqaddr(self.WHAT,Mix),
+          mix(),
+          nqp::create(self)
         )
     }
 
 #--- selection methods
-    multi method grab($count? --> Real) {
+    multi method grab($count? --> Real:D) {
         X::Immutable.new( method => 'grab', typename => self.^name ).throw;
     }
-    multi method grabpairs($count? --> Real) {
+    multi method grabpairs($count? --> Real:D) {
         X::Immutable.new( method => 'grabpairs', typename => self.^name ).throw;
     }
 
 #--- coercion methods
-    method Mix { self }
-    method MixHash { MixHash.new-from-pairs(%!elems.values) }
-    method Bag     {     Bag.new-from-pairs(%!elems.values.grep(*.value > 0).map({.key => .value.Int})) }
-    method BagHash { BagHash.new-from-pairs(%!elems.values.grep(*.value > 0).map({.key => .value.Int})) }
+    multi method Mix(Mix:D:) { self }
+
+    method clone() { nqp::clone(self) }
 
     proto method classify-list(|) {
         X::Immutable.new(:method<classify-list>, :typename(self.^name)).throw;
