@@ -44,7 +44,18 @@ my class Mu { # declared in BOOTSTRAP
         )
     }
 
+    proto method iterator(|) { * }
+    multi method iterator(Mu:) {
+        my $buf := nqp::create(IterationBuffer);
+        $buf.push(Mu);
+        # note: cannot use R:I.OneValue, as that doesn't (and shouldn't)
+        # take Mu for the value to produce, as Mu is used to indicate
+        # exhaustion.
+        Rakudo::Iterator.ReifiedList($buf)
+    }
+
     proto method split(|) { * }
+    proto method splice(|) is nodal { * }
 
     method emit {
         emit self;
@@ -67,6 +78,13 @@ my class Mu { # declared in BOOTSTRAP
     multi method WHY(Mu:) {
         my Mu $why;
 
+        my role Suggestion[$name] {
+            method gist {
+                "No documentation available for type '$name'.
+Perhaps it can be found at https://docs.perl6.org/type/$name"
+            }
+        }
+
         if nqp::can(self.HOW, 'WHY') {
             $why := self.HOW.WHY;
         }
@@ -74,7 +92,7 @@ my class Mu { # declared in BOOTSTRAP
         if $why.defined && !$.defined #`(ie. we're a type object) {
             $why.set_docee(self);
         }
-        $why // Any
+        $why // Nil but Suggestion[self.^name]
     }
 
     method set_why($why) {
@@ -101,7 +119,7 @@ my class Mu { # declared in BOOTSTRAP
     }
 
     proto method is-lazy (|) { * }
-    multi method is-lazy(Mu:) { False }
+    multi method is-lazy(Mu: --> False) { }
 
     method CREATE() {
         nqp::create(self)
@@ -311,6 +329,9 @@ my class Mu { # declared in BOOTSTRAP
         my $task;
         my $build;
         my int $code;
+        my int $int;
+        my num $num;
+        my str $str;
 
         nqp::while(
           nqp::islt_i($i = nqp::add_i($i,1),$count),
@@ -417,15 +438,60 @@ my class Mu { # declared in BOOTSTRAP
                             )
                           ),
 
-                          nqp::if( # 13
-                            nqp::iseq_i($code,13),
-                            nqp::getattr(self,  # not sure why this is needed
-                              nqp::atpos($task,1),
-                              nqp::atpos($task,2)
+                          nqp::if( # 8
+                            nqp::iseq_i($code,8),
+                            nqp::if(
+                              nqp::iseq_i($int = nqp::getattr_i(self,
+                                nqp::atpos($task,1),
+                                nqp::atpos($task,2)
+                              ), 0),
+                              nqp::bindattr_i(self,
+                                nqp::atpos($task,1),
+                                nqp::atpos($task,2),
+                                (nqp::atpos($task,3)(self,$int))
+                              )
                             ),
-                            die("Invalid BUILD_LEAST_DERIVED plan")
-                          )
-        )))))))));
+
+                            nqp::if( # 9
+                              nqp::iseq_i($code,9),
+                              nqp::if(
+                                nqp::iseq_n($num = nqp::getattr_n(self,
+                                  nqp::atpos($task,1),
+                                  nqp::atpos($task,2)
+                                ), 0e0),
+                                nqp::bindattr_n(self,
+                                  nqp::atpos($task,1),
+                                  nqp::atpos($task,2),
+                                  (nqp::atpos($task,3)(self,$num))
+                                )
+                              ),
+
+                              nqp::if( # 10
+                                nqp::iseq_i($code,10),
+                                nqp::if(
+                                  nqp::isnull_s($str = nqp::getattr_s(self,
+                                    nqp::atpos($task,1),
+                                    nqp::atpos($task,2)
+                                  )),
+                                  nqp::bindattr_s(self,
+                                    nqp::atpos($task,1),
+                                    nqp::atpos($task,2),
+                                    (nqp::atpos($task,3)(self,$str))
+                                  )
+                                ),
+
+                                  nqp::if( # 13
+                                    nqp::iseq_i($code,13),
+                                    # Force vivification, for the sake of meta-object
+                                    # mix-ins at compile time ending up with correctly
+                                    # shared containers.
+                                    nqp::getattr(self,
+                                      nqp::atpos($task,1),
+                                      nqp::atpos($task,2)
+                                    ),
+                                    die("Invalid BUILD_LEAST_DERIVED plan")
+                                  )
+        ))))))))))));
         self
     }
 
@@ -501,12 +567,14 @@ my class Mu { # declared in BOOTSTRAP
     multi method gist(Mu:D:) { self.perl }
 
     method perlseen(Mu:D \SELF: $id, $perl, *%named) {
+        my $sigil = nqp::iseq_s($id, 'Array') ?? '@'
+            !! nqp::iseq_s($id, 'Hash') ?? '%' !! '\\';
         if nqp::not_i(nqp::isnull(nqp::getlexdyn('$*perlseen'))) {
             my \sems := $*perlseen;
             my str $WHICH = nqp::unbox_s(self.WHICH);
             if nqp::existskey(sems,$WHICH) && nqp::atkey(sems,$WHICH) {
                 nqp::bindkey(sems,$WHICH,2);
-                "{$id}_{nqp::objectid(SELF)}";
+                $sigil x nqp::isne_s($sigil, '\\') ~ "{$id}_{nqp::objectid(SELF)}";
             }
             else {
                 nqp::bindkey(sems,$WHICH,1);
@@ -514,7 +582,9 @@ my class Mu { # declared in BOOTSTRAP
                 my int $value = nqp::atkey(sems,$WHICH);
                 nqp::deletekey(sems,$WHICH);
                 $value == 2
-                  ?? "(my \\{$id}_{nqp::objectid(SELF)} = $result)"
+                  ?? nqp::iseq_s($sigil, '\\')
+                    ??  "(my {$sigil}{$id}_{nqp::objectid(SELF)} = $result)"
+                    !! "((my {$sigil}{$id}_{nqp::objectid(SELF)}) = $result)"
                   !! $result
             }
         }
@@ -622,7 +692,12 @@ my class Mu { # declared in BOOTSTRAP
         SELF.^can($name)
     }
 
-    method clone(*%twiddles) {
+    proto method clone (|) { * }
+    multi method clone(Mu:U: *%twiddles) {
+        %twiddles and die 'Cannot set attribute values when cloning a type object';
+        self
+    }
+    multi method clone(Mu:D: *%twiddles) {
         my $cloned := nqp::clone(self);
         if %twiddles.elems {
             for self.^attributes.flat -> $attr {
@@ -700,14 +775,14 @@ my class Mu { # declared in BOOTSTRAP
             ).throw;
     }
 
-    method dispatch:<.=>(\mutate: \name, |c) is raw {
+    method dispatch:<.=>(\mutate: Str() $name, |c) is raw {
         $/ := nqp::getlexcaller('$/');
-        mutate = mutate."{name}"(|c)
+        mutate = mutate."$name"(|c)
     }
 
-    method dispatch:<.?>(Mu \SELF: \name, |c) is raw {
-        nqp::can(SELF, name) ??
-            SELF."{name}"(|c) !!
+    method dispatch:<.?>(Mu \SELF: Str() $name, |c) is raw {
+        nqp::can(SELF,$name) ??
+            SELF."$name"(|c) !!
             Nil
     }
 
@@ -726,7 +801,7 @@ my class Mu { # declared in BOOTSTRAP
     method dispatch:<.*>(Mu \SELF: \name, |c) {
         my @mro = SELF.^mro;
         my int $mro_count = @mro.elems;
-        my $results := nqp::list;
+        my $results := nqp::create(IterationBuffer);
         my int $i = -1;
         while nqp::islt_i(++$i,$mro_count) {
             my $obj = @mro[$i];
@@ -737,15 +812,20 @@ my class Mu { # declared in BOOTSTRAP
         nqp::p6bindattrinvres(nqp::create(List),List,'$!reified',$results)
     }
 
-    method dispatch:<hyper>(Mu \SELF: \name, |c) {
-        my $listcan = List.can(name);
-        $listcan && $listcan[0].?nodal
-          ?? c
-            ?? HYPER( sub (\obj) is nodal { obj."{name}"(|c) }, SELF )
-            !! HYPER( sub (\obj) is nodal { obj."{name}"() }, SELF )
-          !! c
-            ?? HYPER( -> \obj { obj."{name}"(|c) }, SELF )
-            !! HYPER( -> \obj { obj."{name}"() }, SELF )
+    method dispatch:<hyper>(Mu \SELF: Str() $name, |c) {
+        nqp::if(
+          nqp::can(List,$name) && nqp::can(List.can($name).AT-POS(0),"nodal"),
+          nqp::if(
+            c,
+            HYPER( sub (\obj) is nodal { obj."$name"(|c) }, SELF ),
+            HYPER( sub (\obj) is nodal { obj."$name"() }, SELF )
+          ),
+          nqp::if(
+            c,
+            HYPER( -> \obj { obj."$name"(|c) }, SELF ),
+            HYPER( -> \obj { obj."$name"() }, SELF )
+          )
+        )
     }
 
     method WALK(:$name!, :$canonical, :$ascendant, :$descendant, :$preorder, :$breadth,
@@ -854,19 +934,19 @@ multi sub infix:<eqv>(Any:D \a, Any:D \b) {
     )
 }
 
-multi sub infix:<eqv>(@a, @b) {
+multi sub infix:<eqv>(Iterable:D \a, Iterable:D \b) {
     nqp::p6bool(
       nqp::unless(
-        nqp::eqaddr(@a,@b),                                    # identity
+        nqp::eqaddr(a,b),                                    # identity
         nqp::if(
-          nqp::eqaddr(@a.WHAT,@b.WHAT),                        # same type
+          nqp::eqaddr(a.WHAT,b.WHAT),                        # same type
           nqp::if(
-            nqp::iseq_i((my int $elems = @a.elems),@b.elems),  # same # elems
+            nqp::iseq_i((my int $elems = a.elems),b.elems),  # same # elems
             nqp::stmts(
               (my int $i = -1),
               nqp::while(
                 nqp::islt_i(($i = nqp::add_i($i,1)),$elems)    # not exhausted
-                  && @a.AT-POS($i) eqv @b.AT-POS($i),          # still same
+                  && a.AT-POS($i) eqv b.AT-POS($i),          # still same
                 nqp::null
               ),
               nqp::iseq_i($i,$elems)                      # exhausted = success!
@@ -937,6 +1017,12 @@ sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
     }
 }
 
+# U+2212 minus (forward call to regular minus)
+proto sub  infix:<−>(|)  is pure { * }
+multi sub  infix:<−>(|c)         {  infix:<->(|c) }
+proto sub prefix:<−>(|)  is pure { * }
+multi sub prefix:<−>(|c)         { prefix:<->(|c) }
+
 # These must collapse Junctions
 proto sub so(Mu $) {*}
 multi sub so(Mu $x)  { ?$x }
