@@ -5,7 +5,9 @@ my role Numeric {
     multi method Numeric(Numeric:D:) { self }
 
     multi method ACCEPTS(Numeric:D: Any:D \a) {
-        self.isNaN && a.isNaN or a == self;
+        (try my \numeric = a.Numeric).defined
+                ?? (self.isNaN && numeric.isNaN or numeric == self)
+                !! False
     }
 
     proto method log(|) {*}
@@ -20,6 +22,7 @@ my role Numeric {
     }
     method roots(Cool $n) { self.Complex.roots($n.Int) }
 
+    method FatRat(Numeric:D:) { self.Rat.FatRat }
     multi method Bool(Numeric:D:) { self != 0 }
 
     multi method gist(Numeric:D:) { self.Str }
@@ -30,9 +33,11 @@ my role Numeric {
 }
 
 multi sub infix:<eqv>(Numeric:D \a, Numeric:D \b) {
-    nqp::p6bool(
-      nqp::eqaddr(a,b) || (nqp::eqaddr(a.WHAT,b.WHAT) && a == b) # RT #127951
-    )
+    nqp::p6bool( # RT #127951
+        nqp::eqaddr(a,b) || (
+            nqp::eqaddr(a.WHAT,b.WHAT)
+            && nqp::if(nqp::istype(a, Num), (a === b), (a == b))
+    )) # for Nums use === to properly handle signed zeros and NaNs
 }
 
 ## arithmetic operators
@@ -43,8 +48,6 @@ multi sub prefix:<+>(\a) { a.Numeric }
 proto sub prefix:<->($?) is pure { * }
 multi sub prefix:<->(\a) { -a.Numeric }
 
-sub prefix:<−>($n) is pure { prefix:<->($n) }
-
 proto sub abs($) is pure { * }
 multi sub abs(\a) { abs a.Numeric }
 
@@ -191,10 +194,6 @@ proto sub infix:<->(Mu $?, Mu $?) is pure   { * }
 multi sub infix:<->($x = 0)      { -$x.Numeric }
 multi sub infix:<->(\a, \b)    { a.Numeric - b.Numeric }
 
-proto sub infix:<−>(Mu $?, Mu $?) is pure { * }
-multi sub infix:<−>($x = 0)    { -$x.Numeric }
-multi sub infix:<−>(\a, \b) is pure { a.Numeric - b.Numeric }
-
 proto sub infix:<*>(Mu $?, Mu $?) is pure   { * }
 multi sub infix:<*>($x = 1)      { $x.Numeric }
 multi sub infix:<*>(\a, \b)    { a.Numeric * b.Numeric }
@@ -219,11 +218,33 @@ multi sub infix:<%>(\a, \b)    { a.Real % b.Real }
 proto sub infix:<%%>(Mu $?, Mu $?) is pure  { * }
 multi sub infix:<%%>() { Failure.new("No zero-arg meaning for infix:<%%>") }
 multi sub infix:<%%>($)         { Bool::True }
-multi sub infix:<%%>(\a, \b)   {
-    b
-      ?? a.Real % b.Real == 0
-      !! Failure.new(X::Numeric::DivideByZero.new(
-           using => 'infix:<%%>', numerator => a))
+multi sub infix:<%%>(Int:D \a, Int:D \b) {
+    nqp::if(
+      nqp::isbig_I(nqp::decont(a)) || nqp::isbig_I(nqp::decont(b)),
+      nqp::if(
+        b,
+        nqp::p6bool(nqp::not_i(nqp::mod_I(nqp::decont(a),nqp::decont(b),Int))),
+        Failure.new(
+          X::Numeric::DivideByZero.new(using => 'infix:<%%>', numerator => a)
+        )
+      ),
+      nqp::if(
+        nqp::isne_i(b,0),
+        nqp::p6bool(nqp::not_i(nqp::mod_i(nqp::decont(a),nqp::decont(b)))),
+        Failure.new(
+          X::Numeric::DivideByZero.new(using => 'infix:<%%>', numerator => a)
+        )
+      )
+    )
+}
+multi sub infix:<%%>(\a, \b) {
+    nqp::if(
+      b,
+      (a.Real % b.Real == 0),
+      Failure.new(
+        X::Numeric::DivideByZero.new(using => 'infix:<%%>', numerator => a)
+      )
+    )
 }
 
 proto sub infix:<lcm>(Mu $?, Mu $?) is pure  { * }
@@ -268,6 +289,7 @@ sub infix:<=~=>(|c) { infix:<≅>(|c) }
 proto sub infix:<!=>(Mu $?, Mu $?) is pure  { * }
 multi sub infix:<!=>($?)        { Bool::True }
 multi sub infix:<!=>(Mu \a, Mu \b)   { not a == b }
+sub infix:<≠>(|c) is pure { infix:<!=>(|c) }
 
 proto sub infix:«<»(Mu $?, Mu $?) is pure   { * }
 multi sub infix:«<»($?)         { Bool::True }
@@ -276,6 +298,7 @@ multi sub infix:«<»(\a, \b)    { a.Real < b.Real }
 proto sub infix:«<=»(Mu $?, Mu $?) is pure  { * }
 multi sub infix:«<=»($?)        { Bool::True }
 multi sub infix:«<=»(\a, \b)   { a.Real <= b.Real }
+sub infix:«≤»(|c) is pure { infix:«<=»(|c) }
 
 proto sub infix:«>»(Mu $?, Mu $?) is pure   { * }
 multi sub infix:«>»($?)         { Bool::True }
@@ -284,6 +307,7 @@ multi sub infix:«>»(\a, \b)    { a.Real > b.Real }
 proto sub infix:«>=»(Mu $?, Mu $?) is pure  { * }
 multi sub infix:«>=»($?)        { Bool::True }
 multi sub infix:«>=»(\a, \b)   { a.Real >= b.Real }
+sub infix:«≥»(|c) is pure { infix:«>=»(|c) }
 
 ## bitwise operators
 
