@@ -16,7 +16,7 @@ Rakudo::Internals.REGISTER-DYNAMIC: '$*PID', {
 }
 
 Rakudo::Internals.REGISTER-DYNAMIC: '$*EXECUTABLE', {
-    PROCESS::<$EXECUTABLE> := IO::Path.new-from-absolute-path(
+    PROCESS::<$EXECUTABLE> := (
 #?if jvm
       $*VM.properties<perl6.execname>
       // $*VM.properties<perl6.prefix> ~ '/bin/perl6-j'
@@ -26,7 +26,7 @@ Rakudo::Internals.REGISTER-DYNAMIC: '$*EXECUTABLE', {
       // ($*VM.config<prefix> ~ '/bin/'
         ~ ($*VM.config<osname> eq 'MSWin32' ?? 'perl6-m.bat' !! 'perl6-m'))
 #?endif
-    );
+    ).IO;
 }
 
 Rakudo::Internals.REGISTER-DYNAMIC: '$*EXECUTABLE-NAME', {
@@ -42,7 +42,7 @@ Rakudo::Internals.REGISTER-DYNAMIC: '$*PROGRAM', {
 }
 
 Rakudo::Internals.REGISTER-DYNAMIC: '$*TMPDIR', {
-    PROCESS::<$TMPDIR> := $*SPEC.tmpdir;
+    PROCESS::<$TMPDIR> = $*SPEC.tmpdir;
 }
 
 Rakudo::Internals.REGISTER-DYNAMIC: '$*TOLERANCE', {
@@ -59,15 +59,19 @@ Rakudo::Internals.REGISTER-DYNAMIC: '$*REPO', {
 }
 
 Rakudo::Internals.REGISTER-DYNAMIC: '$*HOME', {
-    my $HOME;
+    my $HOME is default(Nil);
 
     if %*ENV<HOME> -> $home {
         $HOME = $home;
     }
     elsif Rakudo::Internals.IS-WIN {
-        $HOME = %*ENV<HOMEDRIVE> ~ %*ENV<HOMEPATH>;
+        my $env := %*ENV;
+        $env<HOMEDRIVE> && $env<HOMEPATH> && ($HOME
+          = nqp::concat($env<HOMEDRIVE>, $env<HOMEPATH>));
     }
-    PROCESS::<$HOME> = $HOME ?? IO::Path.new($HOME) !! Nil;
+
+    $HOME = IO::Path.new($HOME) if $HOME;
+    PROCESS::<$HOME> := $HOME # bind container so Nil default is kept
 }
 
 {
