@@ -24,7 +24,7 @@ my role X::Promise::Broken {
             callsame().indent(4)
     }
 }
-my class Promise {
+my class Promise does Awaitable {
     has $.scheduler;
     has $.status;
     has $!result is default(Nil);
@@ -75,14 +75,13 @@ my class Promise {
         self.vow.keep(result)
     }
 
-    method !keep(Mu \result) {
+    method !keep(Mu \result --> Nil) {
         $!lock.protect({
             $!result := result;
             $!status = Kept;
             self!schedule_thens();
             $!cond.signal_all;
         });
-        Nil
     }
 
     proto method break(|) { * }
@@ -93,7 +92,7 @@ my class Promise {
         self.vow.break(result)
     }
 
-    method !break(\result) {
+    method !break(\result --> Nil) {
         $!lock.protect({
             $!result = nqp::istype(result, Exception)
                 ?? result
@@ -102,10 +101,9 @@ my class Promise {
             self!schedule_thens();
             $!cond.signal_all;
         });
-        Nil
     }
 
-    method !schedule_thens() {
+    method !schedule_thens(--> Nil) {
         while @!thens {
             $!scheduler.cue(@!thens.shift, :catch(@!thens.shift))
         }
@@ -151,24 +149,68 @@ my class Promise {
         if $!status == Broken || $!status == Kept {
             # Already have the result, start immediately.
             nqp::unlock($!lock);
-            Promise.start( { code(self) }, :$!scheduler);
+            self.WHAT.start( { code(self) }, :$!scheduler);
         }
         else {
             # Create a Promise, and push 2 entries to @!thens: something that
             # starts the then code, and something that handles its exceptions.
             # They will be sent to the scheduler when this promise is kept or
             # broken.
-            my $then_promise = Promise.new(:$!scheduler);
-            my $vow = $then_promise.vow;
-            @!thens.push({ $vow.keep(code(self)) });
+            my $then-p := self.new(:$!scheduler);
+            nqp::bindattr($then-p, Promise, '$!dynamic_context', nqp::ctx());
+            my $vow = $then-p.vow;
+            @!thens.push({ my $*PROMISE := $then-p; $vow.keep(code(self)) });
             @!thens.push(-> $ex { $vow.break($ex) });
             nqp::unlock($!lock);
-            $then_promise
+            $then-p
+        }
+    }
+
+    my class PromiseAwaitableHandle does Awaitable::Handle {
+        has &!add-subscriber;
+
+        method not-ready(&add-subscriber) {
+            nqp::create(self)!not-ready(&add-subscriber)
+        }
+        method !not-ready(&add-subscriber) {
+            $!already = False;
+            &!add-subscriber := &add-subscriber;
+            self
+        }
+
+        method subscribe-awaiter(&subscriber --> Nil) {
+            &!add-subscriber(&subscriber);
+        }
+    }
+
+    method get-await-handle(--> Awaitable::Handle:D) {
+        if $!status == Broken {
+            PromiseAwaitableHandle.already-failure($!result)
+        }
+        elsif $!status == Kept {
+            PromiseAwaitableHandle.already-success($!result)
+        }
+        else {
+            PromiseAwaitableHandle.not-ready: -> &on-ready {
+                nqp::lock($!lock);
+                if $!status == Broken || $!status == Kept {
+                    # Already have the result, call on-ready immediately.
+                    nqp::unlock($!lock);
+                    on-ready($!status == Kept, $!result)
+                }
+                else {
+                    # Push 2 entries to @!thens (only need the first one in
+                    # this case; second we push 'cus .then uses it).
+                    @!thens.push({ on-ready($!status == Kept, $!result) });
+                    @!thens.push(Callable);
+                    nqp::unlock($!lock);
+                }
+            }
         }
     }
 
     method start(Promise:U: &code, :&catch, :$scheduler = $*SCHEDULER, |c) {
-        my $p := Promise.new(:$scheduler);
+        my $p := self.new(:$scheduler);
         nqp::bindattr($p, Promise, '$!dynamic_context', nqp::ctx());
         my $vow = $p.vow;
         $scheduler.cue(
@@ -178,7 +220,7 @@ my class Promise {
     }
 
     method in(Promise:U: $seconds, :$scheduler = $*SCHEDULER) {
-        my $p   = Promise.new(:$scheduler);
+        my $p   = self.new(:$scheduler);
         my $vow = $p.vow;
         $scheduler.cue({ $vow.keep(True) }, :in($seconds));
         $p
@@ -191,7 +233,7 @@ my class Promise {
     method allof(Promise:U: *@p) { self!until_n_kept(@p, +@p, 'allof') }
 
     method !until_n_kept(@promises, Int $N, Str $combinator) {
-        my $p = Promise.new;
+        my $p = self.new;
         unless @promises {
             $p.keep;
             return $p
@@ -214,7 +256,7 @@ my class Promise {
         $p
     }
 
-    method Supply(Promise:D:) {
+    multi method Supply(Promise:D:) {
         Supply.on-demand: -> $s {
             self.then({
                 if self.status == Kept {
