@@ -39,18 +39,6 @@ do {
         splice(@values, $insert_pos, 0, $value);
     }
 
-    my sub mkpath(IO::Path $full-path) {
-        my ( :$directory, *% ) := $full-path.parts;
-        my @parts = $*SPEC.splitdir($directory);
-
-        for [\~] @parts.map(* ~ '/') -> $path {
-            mkdir $path;
-            unless $path.IO ~~ :d {
-                fail "Unable to mkpath '$full-path': $path is not a directory";
-            }
-        }
-    }
-
     my role ReadlineBehavior[$WHO] {
         my &readline    = $WHO<&readline>;
         my &add_history = $WHO<&add_history>;
@@ -85,7 +73,7 @@ do {
 
         method completions-for-line(Str $line, int $cursor-index) { ... }
 
-        method history-file() returns Str { ... }
+        method history-file(--> Str:D) { ... }
 
         method init-line-editor {
             linenoiseSetCompletionCallback(sub ($line, $c) {
@@ -96,10 +84,8 @@ do {
 
         method teardown-line-editor {
             my $err = linenoiseHistorySave($.history-file);
-
-            if $err {
-                note "Couldn't save your history to $.history-file";
-            }
+            return if !$err;
+            note "Couldn't save your history to $.history-file";
         }
 
         method repl-read(Mu \prompt) {
@@ -129,7 +115,7 @@ do {
                     /^ "&"? $<word>=[\w* <.lower> \w*] $/ ?? ~$<word> !! []
                 }).sort;
 
-        method update-completions {
+        method update-completions(--> Nil) {
             my $context := self.compiler.context;
 
             return unless $context;
@@ -138,13 +124,11 @@ do {
             my $it := nqp::iterator($pad);
 
             while $it {
-                my $e := nqp::shift($it);
-                my $k := nqp::iterkey_s($e);
+                my $k := nqp::iterkey_s(nqp::shift($it));
                 my $m = $k ~~ /^ "&"? $<word>=[\w* <.lower> \w*] $/;
-                if $m {
-                    my $word = ~$m<word>;
-                    sorted-set-insert(@!completions, $word);
-                }
+                next if !$m;
+                my $word = ~$m<word>;
+                sorted-set-insert(@!completions, $word);
             }
 
             my $PACKAGE = self.compiler.eval('$?PACKAGE', :outer_ctx($context));
@@ -224,9 +208,6 @@ do {
         }
 
         sub mixin-line-editor($self) {
-            my $new-self;
-            my Bool $problem;
-
             my %editor-to-mixin = (
                 :Linenoise(&mixin-linenoise),
                 :Readline(&mixin-readline),
@@ -234,39 +215,34 @@ do {
             );
 
             if %*ENV<RAKUDO_LINE_EDITOR> -> $line-editor {
-                if %editor-to-mixin{$line-editor} -> $mixin {
-                    ( $new-self, $problem ) = $mixin($self);
-
-                    if $new-self {
-                        return $new-self;
-                    } else {
-                        unless $problem {
-                            say "Could not find $line-editor module";
-                        }
-                        return $self but FallbackBehavior;
-                    }
-                } else {
+                if !%editor-to-mixin{$line-editor} {
                     say "Unrecognized line editor '$line-editor'";
                     return $self but FallbackBehavior;
                 }
-            } else {
-                ( $new-self, $problem ) = mixin-readline($self, :fallback<Linenoise>);
 
+                my $mixin = %editor-to-mixin{$line-editor};
+                my ( $new-self, $problem ) = $mixin($self);
                 return $new-self if $new-self;
 
-                ( $new-self, $problem ) = mixin-linenoise($self);
-
-                return $new-self if $new-self;
-
-                if $problem {
-                    say 'Continuing without tab completions or line editor';
-                    say 'You may want to consider using rlwrap for simple line editor functionality';
-                } else {
-                    say 'You may want to `zef install Readline` or `zef install Linenoise` or use rlwrap for a line editor';
-                }
-                say '';
+                say "Could not find $line-editor module" unless $problem;
+                return $self but FallbackBehavior;
             }
 
+            my ( $new-self, $problem ) = mixin-readline($self, :fallback<Linenoise>);
+            return $new-self if $new-self;
+
+            ( $new-self, $problem ) = mixin-linenoise($self);
+            return $new-self if $new-self;
+
+            if $problem {
+                say 'Continuing without tab completions or line editor';
+                say 'You may want to consider using rlwrap for simple line editor functionality';
+            }
+            elsif !$*DISTRO.is-win and !( %*ENV<_>:exists and %*ENV<_>.ends-with: 'rlwrap' ) {
+                say 'You may want to `zef install Readline` or `zef install Linenoise` or use rlwrap for a line editor';
+            }
+            say '';
+
             $self but FallbackBehavior
         }
 
@@ -278,7 +254,7 @@ do {
             $self
         }
 
-        method init(Mu \compiler, $multi-line-enabled) {
+        method init(Mu \compiler, $multi-line-enabled --> Nil) {
             $!compiler := compiler;
             $!multi-line-enabled = $multi-line-enabled;
         }
@@ -287,23 +263,19 @@ do {
             self.?teardown-line-editor;
         }
 
-        method repl-eval($code, *%adverbs) {
+        method repl-eval($code, \exception, *%adverbs) {
 
             CATCH {
                 when X::Syntax::Missing {
-                    if $!multi-line-enabled && .pos == $code.chars {
-                        return $!need-more-input;
-                    } else {
-                        .throw;
-                    }
+                    return $!need-more-input
+                      if $!multi-line-enabled && .pos == $code.chars;
+                    .throw;
                 }
 
                 when X::Comp::FailGoal {
-                    if $!multi-line-enabled && .pos == $code.chars {
-                        return $!need-more-input;
-                    } else {
-                        .throw;
-                    }
+                    return $!need-more-input
+                      if $!multi-line-enabled && .pos == $code.chars;
+                    .throw;
                 }
 
                 when X::ControlFlow::Return {
@@ -311,8 +283,8 @@ do {
                 }
 
                 default {
-                    # Use the exception as the result of the eval, to be printed
-                    return $_;
+                    exception = $_;
+                    return;
                 }
             }
 
@@ -358,8 +330,9 @@ do {
                 my $*CTXSAVE := self;
                 my $*MAIN_CTX;
 
-                my $output = self.repl-eval(
+                my $output is default(Nil) = self.repl-eval(
                     $code,
+                    my $exception,
                     :outer_ctx($!save_ctx),
                     |%adverbs);
 
@@ -380,8 +353,15 @@ do {
 
                 reset;
 
-                # Only print the result if there wasn't some other output
-                if $initial_out_position == $*OUT.tell {
+                # Print the result if:
+                # - there wasn't some other output
+                # - the result is an *unhandled* Failure
+                # - print an exception if one had occured
+                if $exception.DEFINITE {
+                    self.repl-print($exception);
+                }
+                elsif $initial_out_position == $*OUT.tell
+                    or $output ~~ Failure and not $output.handled {
                     self.repl-print($output);
                 }
 
@@ -396,7 +376,7 @@ do {
         }
 
         # Inside of the EVAL it does like caller.ctxsave
-        method ctxsave() {
+        method ctxsave(--> Nil) {
             $*MAIN_CTX := nqp::ctxcaller(nqp::ctx());
             $*CTXSAVE := 0;
         }
@@ -405,37 +385,30 @@ do {
             $value.WHERE == $!need-more-input.WHERE
         }
 
-        method input-toplevel-control($value) {
+        method input-toplevel-control(Mu $value) {
             $value.WHERE == $!control-not-allowed.WHERE
         }
 
-        method repl-print(Mu $value) {
+        method repl-print(Mu $value --> Nil) {
             say $value;
             CATCH {
                 default { say $_ }
             }
         }
 
-        method history-file returns Str {
-            return ~$!history-file if $!history-file.defined;
+        method history-file(--> Str:D) {
+            return $!history-file.absolute if $!history-file.defined;
 
-            $!history-file = do
-                if $*ENV<RAKUDO_HIST> {
-                    IO::Path.new($*ENV<RAKUDO_HIST>)
-                } else {
-                    IO::Path.new($*HOME).child('.perl6').child('rakudo-history')
-                }
-            try {
-                mkpath($!history-file);
+            $!history-file = $*ENV<RAKUDO_HIST>
+                ?? $*ENV<RAKUDO_HIST>.IO
+                !! ($*HOME || $*TMPDIR).add('.perl6/rakudo-history');
 
-                CATCH {
-                    when X::AdHoc & ({ .Str ~~ m:s/Unable to mkpath/ }) {
-                        note "I ran into a problem trying to set up history: $_";
-                        note 'Sorry, but history will not be saved at the end of your session';
-                    }
-                }
+            without mkdir $!history-file.parent {
+                note "I ran into a problem trying to set up history: {.exception.message}";
+                note 'Sorry, but history will not be saved at the end of your session';
             }
-            ~$!history-file
+
+            $!history-file.absolute
         }
     }
 }
