@@ -16,7 +16,7 @@ my class Range is Cool does Iterable does Positional {
         $!is-int   = nqp::istype($!min,Int) && nqp::istype($!max,Int);
         self
     }
-    method is-lazy { self.infinite }
+    multi method is-lazy(Range:D:) { self.infinite }
 
     # The order of "method new" declarations matters here, to ensure
     # appropriate candidate tiebreaking when mixed type arguments
@@ -72,14 +72,14 @@ my class Range is Cool does Iterable does Positional {
     }
     multi method new(\min, \max) { nqp::create(self)!SET-SELF(min,max,0,0,0) }
 
-    method excludes-min() { ?$!excludes-min }
-    method excludes-max() { ?$!excludes-max }
-    method infinite()     { ?$!infinite     }
-    method is-int()       { ?$!is-int       }
+    method excludes-min() { nqp::p6bool($!excludes-min) }
+    method excludes-max() { nqp::p6bool($!excludes-max) }
+    method infinite()     { nqp::p6bool($!infinite)     }
+    method is-int()       { nqp::p6bool($!is-int)       }
 
     multi method WHICH (Range:D:) {
-        self.^name
-          ~ "|$!min"
+        (nqp::istype(self.WHAT,Range) ?? 'Range|' !! (self.^name ~ '|'))
+          ~ $!min
           ~ ("^" if $!excludes-min)
           ~ '..'
           ~ ("^" if $!excludes-max)
@@ -96,7 +96,9 @@ my class Range is Cool does Iterable does Positional {
     method elems {
         $!is-int
           ?? 0 max $!max - $!excludes-max - $!min - $!excludes-min + 1
-          !! $!infinite ?? Inf !! nextsame;
+          !! $!infinite
+            ?? Failure.new(X::Cannot::Lazy.new(:action<.elems>))
+            !! nextsame
     }
 
     method iterator() {
@@ -104,7 +106,7 @@ my class Range is Cool does Iterable does Positional {
         if $!is-int
           && !nqp::isbig_I(nqp::decont($!min))
           && !nqp::isbig_I(nqp::decont($!max)) {
-            Rakudo::Internals.IntRangeIterator(
+            Rakudo::Iterator.IntRange(
               $!min + $!excludes-min, $!max - $!excludes-max)
         }
 
@@ -122,30 +124,32 @@ my class Range is Cool does Iterable does Positional {
             class :: does Iterator {
                 has $!i;
 
-                method !SET-SELF(\i)  { $!i = i; self }
+                method !SET-SELF(\i)  { $!i = i - 1; self }
                 method new(\i)    { nqp::create(self)!SET-SELF(i) }
-                method pull-one() { $!i++ }
+                method pull-one() { ++$!i }
                 method is-lazy()  { True  }
             }.new($!min + $!excludes-min)
         }
 
         # if we have (simple) char range
         elsif nqp::istype($!min,Str) {
-            my $min = $!excludes-min ?? $!min.succ !! $!min;
-            $min after $!max
+            $!min after $!max
               ?? ().iterator
-              !! $min.chars == 1 && nqp::istype($!max,Str) && $!max.chars == 1
+              !! $!min.chars == 1 && nqp::istype($!max,Str) && $!max.chars == 1
                 ?? class :: does Iterator {
                        has int $!i;
                        has int $!n;
 
-                       method !SET-SELF(\from,\end) {
-                           $!i = nqp::ord(nqp::unbox_s(from)) - 1;
-                           $!n = nqp::ord(nqp::unbox_s(end));
+                       method !SET-SELF(\from,\end,\excludes-min,\excludes-max) {
+                           $!i = nqp::ord(nqp::unbox_s(from))
+                               - (excludes-min ?? 0 !! 1);
+                           $!n = nqp::ord(nqp::unbox_s(end))
+                               - (excludes-max ?? 1 !! 0);
                            self
                        }
-                       method new(\from,\end) {
-                           nqp::create(self)!SET-SELF(from,end)
+                       method new(\from,\end,\excludes-min,\excludes-max) {
+                           nqp::create(self)!SET-SELF(
+                              from,end,excludes-min,excludes-max)
                        }
                        method pull-one() {
                            ( $!i = $!i + 1 ) <= $!n
@@ -161,8 +165,11 @@ my class Range is Cool does Iterable does Positional {
                        method count-only() { nqp::p6box_i($!n - $!i) }
                        method bool-only() { nqp::p6bool(nqp::isgt_i($!n,$!i)) }
                        method sink-all(--> IterationEnd) { $!i = $!n }
-                   }.new($min, $!excludes-max ?? $!max.pred !! $!max)
-                !! SEQUENCE($min,$!max,:exclude_end($!excludes-max)).iterator
+                   }.new($!min, $!max, $!excludes-min, $!excludes-max)
+                !! SEQUENCE(
+                       ($!excludes-min ?? $!min.succ !! $!min),
+                       $!max, :exclude_end($!excludes-max)
+                   ).iterator
         }
 
         # General case according to spec
@@ -207,8 +214,9 @@ my class Range is Cool does Iterable does Positional {
                             $i = $i.succ;
                         }
                     }
+                    $!i = $e.succ;
                 }
-                method sink-all(--> IterationEnd) { $!i = $!e }
+                method sink-all(--> IterationEnd) { $!i = $!e.succ }
             }.new($!excludes-min ?? $!min.succ !! $!min,$!excludes-max,$!max)
         }
     }
@@ -246,8 +254,8 @@ my class Range is Cool does Iterable does Positional {
         elsif $!max === -Inf {
             class :: does Iterator {
                 method new()      { nqp::create(self) }
-                method pull-one() { Inf }
-                method is-lazy()  { True  }
+                method pull-one(--> Inf) { }
+                method is-lazy(--> True) { }
             }.new
         }
 
@@ -364,10 +372,34 @@ my class Range is Cool does Iterable does Positional {
     }
 
     method bounds() { (nqp::decont($!min), nqp::decont($!max)) }
-    method int-bounds() {
+    proto method int-bounds(|) { * }
+    multi method int-bounds($from is rw, $to is rw) {
+        nqp::if(
+          $!is-int,
+          nqp::stmts(
+            ($from = $!min + $!excludes-min),
+            ($to   = $!max - $!excludes-max)
+          ),
+          nqp::if(
+            nqp::istype($!min,Real)
+              && $!min.floor == $!min
+              && nqp::istype($!max,Real)
+              && nqp::istype($!min.Int, Int)  # exclude NaN and Infs, who will fail() here
+              && nqp::istype($!max.Int, Int),
+            nqp::stmts(
+              ($from = $!min.floor + $!excludes-min),
+              ($to   = $!max.floor - ($!excludes-max && $!max.Int == $!max))
+            ),
+            (die "Cannot determine integer bounds")
+          )
+        )
+    }
+    multi method int-bounds() {
         $!is-int
           ?? ($!min + $!excludes-min, $!max - $!excludes-max)
           !! nqp::istype($!min,Real) && $!min.floor == $!min && nqp::istype($!max,Real)
+                && nqp::istype($!min.Int, Int) # exclude NaN and Infs, who will fail() here
+                && nqp::istype($!max.Int, Int)
             ?? ($!min.floor + $!excludes-min, $!max.floor - ($!excludes-max && $!max.Int == $!max))
             !! Failure.new("Cannot determine integer bounds")
     }
@@ -396,30 +428,49 @@ my class Range is Cool does Iterable does Positional {
           !! ($!excludes-min ?? got after $!min !! not got before $!min)
                && ($!excludes-max ?? got before $!max !! not got after $!max)
     }
+    multi method ACCEPTS(Range:D: Complex:D \got) {
+        nqp::istype(($_ := got.Real), Failure) ?? False !! nextwith $_
+    }
     multi method ACCEPTS(Range:D: Range \topic) {
-        (topic.min > $!min
-         || topic.min == $!min
-            && !(!topic.excludes-min && $!excludes-min))
-        &&
-        (topic.max < $!max
-         || topic.max == $!max
-            && !(!topic.excludes-max && $!excludes-max))
+        nqp::istype($!min, Numeric)
+            ?? # RHS is a numeric range, use numeric comparators
+                try {
+                    (topic.min > $!min
+                     || topic.min == $!min
+                        && !(!topic.excludes-min && $!excludes-min))
+                    &&
+                    (topic.max < $!max
+                     || topic.max == $!max
+                        && !(!topic.excludes-max && $!excludes-max))
+                } // False # don't explode on failures to coerce to numerics
+            !! # RHS is a stringy range, use stringy comparators
+                (topic.min gt $!min
+                 || topic.min eq $!min
+                    && !(!topic.excludes-min && $!excludes-min))
+                &&
+                (topic.max lt $!max
+                 || topic.max eq $!max
+                    && !(!topic.excludes-max && $!excludes-max))
     }
 
     multi method AT-POS(Range:D: int \pos) {
         $!is-int
             ?? self.EXISTS-POS(pos)
                 ?? $!min + $!excludes-min + pos
-                !! Failure.new(X::OutOfRange.new(
-                    :what($*INDEX // 'Index'), :got(pos), :range(0..$.elems-1)))
+                !! pos < 0
+                    ?? Failure.new(X::OutOfRange.new(
+                        :what($*INDEX // 'Index'), :got(pos), :range<0..^Inf>
+                    )) !! Nil
             !! self.list.AT-POS(pos);
     }
     multi method AT-POS(Range:D: Int:D \pos) {
         $!is-int
             ?? self.EXISTS-POS(pos)
                 ?? $!min + $!excludes-min + pos
-                !! Failure.new(X::OutOfRange.new(
-                    :what($*INDEX // 'Index'), :got(pos), :range(0..$.elems-1)))
+                !! pos < 0
+                    ?? Failure.new(X::OutOfRange.new(
+                        :what($*INDEX // 'Index'), :got(pos), :range<0..^Inf>
+                    )) !! Nil
             !! self.list.AT-POS(nqp::unbox_i(pos));
     }
 
@@ -443,7 +494,7 @@ my class Range is Cool does Iterable does Positional {
                     }
                     method new(\b,\e) { nqp::create(self)!SET-SELF(b,e) }
                     method pull-one() { $!min + nqp::rand_I($!elems, Int) }
-                    method is-lazy()  { True }
+                    method is-lazy(--> True) { }
                 }.new($!min + $!excludes-min, $elems))
               !! self.list.roll(*)
         }
@@ -632,7 +683,7 @@ my class Range is Cool does Iterable does Positional {
 
     method in-range($got, $what?) {
         self.ACCEPTS($got)
-          || X::OutOfRange.new(:what($what // 'Value'),:$got,:range(self)).throw
+          || X::OutOfRange.new(:what($what // 'Value'),:got($got.perl),:range(self)).throw
     }
 
     multi method minmax(Range:D:) {
@@ -684,13 +735,13 @@ multi sub infix:<*>(Range:D \a, Real:D \b) { a.clone-with-op(&[*], b) }
 multi sub infix:<*>(Real:D \a, Range:D \b) { b.clone-with-op(&[*], a) }
 multi sub infix:</>(Range:D \a, Real:D \b) { a.clone-with-op(&[/], b) }
 
-multi sub infix:<cmp>(Range:D \a, Range:D \b) returns Order:D {
+multi sub infix:<cmp>(Range:D \a, Range:D \b --> Order:D) {
     a.min cmp b.min || a.excludes-min cmp b.excludes-min || a.max cmp b.max || b.excludes-max cmp a.excludes-max
 }
-multi sub infix:<cmp>(Num(Real) \a, Range:D \b) returns Order:D { (a..a) cmp b }
-multi sub infix:<cmp>(Range:D \a, Num(Real) \b) returns Order:D { a cmp (b..b) }
+multi sub infix:<cmp>(Num(Real) \a, Range:D \b --> Order:D) { (a..a) cmp b }
+multi sub infix:<cmp>(Range:D \a, Num(Real) \b --> Order:D) { a cmp (b..b) }
 
-multi sub infix:<cmp>(Positional \a, Range:D \b) returns Order:D { a cmp b.list }
-multi sub infix:<cmp>(Range:D \a, Positional \b) returns Order:D { a.list cmp b }
+multi sub infix:<cmp>(Positional \a, Range:D \b --> Order:D) { a cmp b.list }
+multi sub infix:<cmp>(Range:D \a, Positional \b --> Order:D) { a.list cmp b }
 
 # vim: ft=perl6 expandtab sw=4
