@@ -2,7 +2,6 @@
 my class Rat is Cool does Rational[Int, Int] {
     method Rat   (Rat:D: Real $?) { self }
     method FatRat(Rat:D: Real $?) { FatRat.new($!numerator, $!denominator); }
-    method Range(Rat:U:) { Range.new(-Inf,Inf) }
     multi method perl(Rat:D:) {
         if $!denominator == 1 {
             $!numerator ~ '.0'
@@ -225,7 +224,9 @@ multi sub infix:<**>(Rational \a, Int \b) {
 }
 
 multi sub infix:<==>(Rational:D \a, Rational:D \b) {
-    a.numerator * b.denominator == b.numerator * a.denominator
+    nqp::isfalse(a.denominator) || nqp::isfalse(b.denominator)
+        ?? a.Num == b.Num
+        !! a.numerator * b.denominator == b.numerator * a.denominator
 }
 multi sub infix:<==>(Rational:D \a, Int:D \b) {
     a.REDUCE-ME;
@@ -235,8 +236,14 @@ multi sub infix:<==>(Int:D \a, Rational:D \b) {
     b.REDUCE-ME;
     a == b.numerator && b.denominator == 1;
 }
-multi sub infix:<===>(Rational:D \a, Rational:D \b) returns Bool:D {
-    a.WHAT =:= b.WHAT && a == b
+multi sub infix:<===>(Rational:D \a, Rational:D \b --> Bool:D) {
+    # Check whether we have 0-denominator rationals as well. Those can
+    # be `==` but have different numerator values and so should not `===` True.
+    # Since we're already checking equality first, we only need to check the
+    # zeroeness of the denominator of just one parameter
+    a.WHAT =:= b.WHAT
+        && (a == b || (a.isNaN && b.isNaN))
+        && (a.denominator.Bool || a.numerator == b.numerator)
 }
 
 multi sub infix:«<»(Rational:D \a, Rational:D \b) {
