@@ -4,15 +4,19 @@ my role Rational[::NuT, ::DeT] does Real {
 
     multi method WHICH(Rational:D:) {
         nqp::box_s(
-            nqp::concat(
-                nqp::concat(nqp::unbox_s(self.^name), '|'),
-                nqp::concat(
-                    nqp::tostr_I($!numerator),
-                    nqp::concat('/', nqp::tostr_I($!denominator))
-                )
+          nqp::concat(
+            nqp::if(
+              nqp::eqaddr(self.WHAT,Rational),
+              'Rational|',
+              nqp::concat(nqp::unbox_s(self.^name), '|')
             ),
-            ObjAt
-        );
+            nqp::concat(
+              nqp::tostr_I($!numerator),
+              nqp::concat('/', nqp::tostr_I($!denominator))
+            )
+          ),
+          ObjAt
+        )
     }
 
     method new(NuT \nu = 0, DeT \de = 1) {
@@ -20,8 +24,8 @@ my role Rational[::NuT, ::DeT] does Real {
 
         # 0 denominator take it verbatim to support Inf/-Inf/NaN
         if de == 0 {
-            nqp::bindattr($new,self.WHAT,'$!numerator',  nqp::decont(nu));
-            nqp::bindattr($new,self.WHAT,'$!denominator',nqp::decont(de));
+            nqp::bindattr($new,::?CLASS,'$!numerator',  nqp::decont(nu));
+            nqp::bindattr($new,::?CLASS,'$!denominator',nqp::decont(de));
         }
 
         # normalize
@@ -33,8 +37,8 @@ my role Rational[::NuT, ::DeT] does Real {
                 $numerator   = -$numerator;
                 $denominator = -$denominator;
             }
-            nqp::bindattr($new,self.WHAT,'$!numerator',  nqp::decont($numerator));
-            nqp::bindattr($new,self.WHAT,'$!denominator',nqp::decont($denominator));
+            nqp::bindattr($new,::?CLASS,'$!numerator',  nqp::decont($numerator));
+            nqp::bindattr($new,::?CLASS,'$!denominator',nqp::decont($denominator));
         }
 
         $new
@@ -51,22 +55,31 @@ my role Rational[::NuT, ::DeT] does Real {
     }
 
     method floor(Rational:D:) {
-        # correct formula
         $!denominator == 1
             ?? $!numerator
             !! $!numerator div $!denominator
     }
 
     method ceiling(Rational:D:) {
-        # correct formula
+        self.REDUCE-ME;
         $!denominator == 1
             ?? $!numerator
             !! ($!numerator div $!denominator + 1)
     }
 
-    method Int() { self.truncate }
-
+    method Int() {
+        $!denominator
+            ?? self.truncate
+            !! fail X::Numeric::DivideByZero.new:
+                   :details('when coercing Rational to Int')
+    }
     method Bridge() { self.Num }
+    method Range(::?CLASS:U:) { Range.new(-Inf, Inf) }
+    method isNaN {
+        nqp::p6bool(
+            nqp::isfalse(self.numerator) && nqp::isfalse(self.denominator)
+        )
+    }
 
     multi method Str(::?CLASS:D:) {
         if nqp::istype($!numerator,Int) {
@@ -104,6 +117,12 @@ my role Rational[::NuT, ::DeT] does Real {
     }
 
     method base($base, Any $digits? is copy) {
+        # XXX TODO: this $base check can be delegated to Int.base once Num/0 gives Inf/NaN,
+        # instead of throwing (which happens in the .log() call before we reach Int.base
+        2 <= $base <= 36 or Failure.new(X::OutOfRange.new(
+            what => "base argument to base", :got($base), :range<2..36>)
+        );
+
         my $prec;
         if $digits ~~ Whatever {
             $digits = Nil;
@@ -119,7 +138,8 @@ my role Rational[::NuT, ::DeT] does Real {
             }
             else {
                 fail X::OutOfRange.new(
-                    what => 'digits argument to base', got => $digits, range => "0..*"
+                    :what('digits argument to base'), :got($digits),
+                    :range<0..^Inf>,
                 )
             }
         }
@@ -190,7 +210,7 @@ my role Rational[::NuT, ::DeT] does Real {
         self.new($!numerator - $!denominator, $!denominator);
     }
 
-    method norm() { self }
+    method norm() { self.REDUCE-ME; self }
 
     method narrow(::?CLASS:D:) {
         self.REDUCE-ME;
