@@ -15,7 +15,7 @@
                   (my int $numdims = nqp::numdimensions($reified))
                 ),
                 X::NYI.new(
-                  feature => "Partially dimensioned views of arrays").throw,
+                  feature => "Partially dimensioned views of shaped arrays").throw,
                 nqp::stmts(
                   (my $indices := nqp::getattr(@indices,List,'$!reified')),
                   (my $idxs := nqp::list_i),
@@ -224,7 +224,7 @@
         }
 
         sub MEMCPY(Mu \to, Mu \from) {
-            class :: does Rakudo::Internals::ShapeLeafIterator {
+            class :: does Rakudo::Iterator::ShapeLeaf {
                 has $!from;
                 has $!desc;
                 method INIT(Mu \to, Mu \from) {
@@ -245,7 +245,7 @@
             }.new(to,from).sink-all
         }
         sub INTCPY(Mu \to, Mu \from) {
-            class :: does Rakudo::Internals::ShapeLeafIterator {
+            class :: does Rakudo::Iterator::ShapeLeaf {
                 has $!from;
                 method INIT(Mu \to, Mu \from) {
                     nqp::stmts(
@@ -268,7 +268,7 @@
             }.new(to,from).sink-all
         }
         sub NUMCPY(Mu \to, Mu \from) {
-            class :: does Rakudo::Internals::ShapeLeafIterator {
+            class :: does Rakudo::Iterator::ShapeLeaf {
                 has $!from;
                 method INIT(Mu \to, Mu \from) {
                     nqp::stmts(
@@ -294,7 +294,7 @@
         proto method STORE(|) { * }
         multi method STORE(::?CLASS:D: ::?CLASS:D \in) {
             nqp::if(
-              in.shape eqv self.shape && nqp::eqaddr(in.WHAT,self.WHAT),
+              in.shape eqv self.shape,
               nqp::stmts(
                 MEMCPY(self,in),     # VM-supported memcpy-like thing?
                 self
@@ -323,7 +323,7 @@
             )
         }
         multi method STORE(::?CLASS:D: Iterable:D \in) {
-            class :: does Rakudo::Internals::ShapeBranchIterator {
+            class :: does Rakudo::Iterator::ShapeBranch {
                 has $!iterators;
                 has $!desc;
                 method INIT(\to,\from) {
@@ -357,7 +357,7 @@
                             IterationEnd
                           ),
                           nqp::bindpos($!iterators,$i,  # add an empty one
-                            Rakudo::Internals.EmptyIterator),
+                            Rakudo::Iterator.Empty),
                           nqp::if(                      # is it an iterator?
                             nqp::istype($item,Iterable) && nqp::isconcrete($item),
                             nqp::bindpos($!iterators,$i,$item.iterator),
@@ -391,12 +391,38 @@
             }.new(self,in).sink-all;
             self
         }
+        multi method STORE(::?CLASS:D: Iterator:D \iterator) {
+            class :: does Rakudo::Iterator::ShapeLeaf {
+                has Mu $!iterator;
+                has Mu $!desc;
+                method INIT(\list,\iterator) {
+                    nqp::stmts(
+                      ($!iterator := iterator),
+                      ($!desc := nqp::getattr(list,Array,'$!descriptor')),
+                      self.SET-SELF(list)
+                    )
+                }
+                method new(\list,\iter) { nqp::create(self).INIT(list,iter) }
+                method result(--> Nil) {
+                    nqp::unless(
+                      nqp::eqaddr(
+                        (my $pulled := $!iterator.pull-one),IterationEnd),
+                      nqp::ifnull(
+                        nqp::atposnd($!list,$!indices),
+                        nqp::bindposnd($!list,$!indices,
+                          nqp::p6scalarfromdesc($!desc))
+                      ) = $pulled
+                    )
+                }
+            }.new(self,iterator).sink-all;
+            self
+        }
         multi method STORE(::?CLASS:D: Mu \item) {
             X::Assignment::ToShaped.new(shape => self.shape).throw
         }
 
         multi method kv(::?CLASS:D:) {
-            Seq.new(class :: does Rakudo::Internals::ShapeLeafIterator {
+            Seq.new(class :: does Rakudo::Iterator::ShapeLeaf {
                 has int $!on-key;
                 method result() is raw {
                     nqp::if(
@@ -405,7 +431,7 @@
                         (my $result := self.indices),
                         (nqp::bindpos_i($!indices,$!maxdim,  # back 1 for next
                           nqp::sub_i(nqp::atpos_i($!indices,$!maxdim),1))),
-                        $result  
+                        $result
                       ),
                       nqp::atposnd($!list,$!indices)
                     )
@@ -420,22 +446,56 @@
             }.new(self))
         }
         multi method pairs(::?CLASS:D:) {
-            Seq.new(class :: does Rakudo::Internals::ShapeLeafIterator {
+            Seq.new(class :: does Rakudo::Iterator::ShapeLeaf {
+                has Mu $!desc;
+                method !INIT(\list) {
+                    nqp::stmts(
+                      ($!desc := nqp::getattr(list,Array,'$!descriptor')),
+                      self.SET-SELF(list)
+                    )
+                }
+                method new(Mu \list) { nqp::create(self)!INIT(list) }
                 method result() {
-                    Pair.new(self.indices,nqp::atposnd($!list,$!indices))
+                    Pair.new(
+                      self.indices,
+                      nqp::ifnull(
+                        nqp::atposnd($!list,$!indices),
+                        nqp::stmts(
+                          # By the time the block gets executed, the $!indices
+                          # may be at the next iteration already or even reset
+                          # because we reached the end.  So we need to make
+                          # a copy of the indices now.
+                          (my $indices := nqp::clone($!indices)),
+                          nqp::p6bindattrinvres(
+                            (my $scalar := nqp::p6scalarfromdesc($!desc)),
+                            Scalar,
+                           '$!whence',
+                            -> { nqp::bindposnd($!list,$indices,$scalar) }
+                          )
+                        )
+                      )
+                    )
                 }
             }.new(self))
         }
         multi method antipairs(::?CLASS:D:) {
-            Seq.new(class :: does Rakudo::Internals::ShapeLeafIterator {
+            Seq.new(class :: does Rakudo::Iterator::ShapeLeaf {
                 method result() {
                     Pair.new(nqp::atposnd($!list,$!indices),self.indices)
                 }
             }.new(self))
         }
 
+        multi method List(::?CLASS:D:) {
+            nqp::stmts(
+              self.iterator.push-all(
+                (my $list := nqp::create(IterationBuffer))),
+              nqp::p6bindattrinvres(nqp::create(List),List,'$!reified',$list)
+            )
+        }
+
         method iterator(::?CLASS:D:) {
-            class :: does Rakudo::Internals::ShapeLeafIterator {
+            class :: does Rakudo::Iterator::ShapeLeaf {
                 has Mu $!desc;
                 method !INIT(\list) {
                     nqp::stmts(
@@ -468,7 +528,6 @@
         # A shaped array isn't lazy, these methods don't need to go looking
         # into the "todo".
         method eager() { self }
-        method is-lazy() { False }
 
         method sum() is nodal { self.Any::sum }
         multi method elems(::?CLASS:D:) {
