@@ -56,7 +56,7 @@ my class X::Supply::New is Exception {
 # a Supply go in here.
 my class Supplier { ... }
 my class Supplier::Preserving { ... }
-my class Supply {
+my class Supply does Awaitable {
     has Tappable $!tappable;
 
     proto method new(|) { * }
@@ -101,10 +101,10 @@ my class Supply {
                     catch => -> \ex { $p.quit(ex) });
                 Tap.new(&!closing)
             }
-            
-            method live() { False }
-            method sane() { True }
-            method serial() { True }
+
+            method live(--> False) { }
+            method sane(--> True) { }
+            method serial(--> True) { }
         }.new(:&producer, :&closing, :$scheduler))
     }
 
@@ -136,9 +136,9 @@ my class Supply {
                 Tap.new({ $cancellation.cancel })
             }
 
-            method live { False }
-            method sane { True }
-            method serial { False }
+            method live(--> False) { }
+            method sane(--> True) { }
+            method serial(--> False) { }
         }.new(:$interval, :$delay, :$scheduler));
     }
 
@@ -151,8 +151,8 @@ my class Supply {
     my role SimpleOpTappable does Tappable {
         has $!source;
         method live() { $!source.live }
-        method sane() { True }
-        method serial() { True }
+        method sane(--> True) { }
+        method serial(--> True) { }
         method !cleanup(int $cleaned-up is rw, $source-tap) {
             if $source-tap && !$cleaned-up  {
                 $cleaned-up = 1;
@@ -188,7 +188,7 @@ my class Supply {
                 Tap.new({ self!cleanup($cleaned-up, $source-tap) })
             }
         }.new(source => self))
-    }    
+    }
 
     method sanitize() {
         $!tappable.sane ?? self !! Supply.new(class :: does SimpleOpTappable {
@@ -573,7 +573,7 @@ my class Supply {
     ## Coercions
     ##
 
-    method Supply(Supply:) { self }
+    multi method Supply(Supply:D:) { self }
 
     method Channel(Supply:D:) {
         my $c = Channel.new();
@@ -585,9 +585,9 @@ my class Supply {
     }
 
     my class ConcQueue is repr('ConcBlockingQueue') { }
-    method list(Supply:D:) {
+    multi method list(Supply:D:) {
         gather {
-            my Mu \queue = ConcQueue.CREATE;
+            my Mu \queue = nqp::create(ConcQueue);
             my $exception;
             self.tap(
                 -> \val { nqp::push(queue, val) },
@@ -620,6 +620,31 @@ my class Supply {
 
     method wait(Supply:D:) { await self.Promise }
 
+    my class SupplyAwaitableHandle does Awaitable::Handle {
+        has $!supply;
+
+        method not-ready(Supply:D \supply) {
+            nqp::create(self)!not-ready(supply)
+        }
+        method !not-ready(\supply) {
+            $!already = False;
+            $!supply := supply;
+            self
+        }
+
+        method subscribe-awaiter(&subscriber --> Nil) {
+            my $final := Nil;
+            $!supply.tap:
+                -> \val { $final := val },
+                done => { subscriber(True, $final) },
+                quit => -> \ex { subscriber(False, ex) };
+        }
+    }
+
+    method get-await-handle(--> Awaitable::Handle) {
+        SupplyAwaitableHandle.not-ready(self)
+    }
+
     method unique(Supply:D $self: :&as, :&with, :$expires) {
         supply {
             if $expires {
@@ -766,7 +791,9 @@ my class Supply {
         }
     }
 
-    proto method rotor(|) {*}
+    multi method rotor(Supply:D $self: Int:D $batch, :$partial) {
+        self.rotor(($batch,), :$partial)
+    }
     multi method rotor(Supply:D $self: *@cycle, :$partial) {
         my @c := @cycle.is-lazy ?? @cycle !! (@cycle xx *).flat.cache;
         supply {
@@ -872,22 +899,22 @@ my class Supply {
             my int $pos;
             my int $nextpos;
             my int $found;
-            
+
             whenever self -> \val {
                 $str   = $str ~ nqp::unbox_s(val);
                 $chars = nqp::chars($str);
                 $pos   = 0;
-            
+
                 while ($left = $chars - $pos) > 0 {
                     $nextpos = nqp::findcclass(
                       nqp::const::CCLASS_NEWLINE, $str, $pos, $left
                     );
-            
+
                     last
                       if $nextpos >= $chars     # no line delimiter
                       or $nextpos == $chars - 1 # broken CRLF ?
                         && nqp::eqat($str, "\r", $nextpos); # yes!
-            
+
                     if $chomp {
                         emit( ($found = $nextpos - $pos)
                           ?? nqp::p6box_s(nqp::substr($str,$pos,$found))
@@ -936,18 +963,18 @@ my class Supply {
                 $chars = nqp::chars($str);
                 $pos   = nqp::findnotcclass(
                   nqp::const::CCLASS_WHITESPACE, $str, 0, $chars);
- 
+
                 while ($left = $chars - $pos) > 0 {
                     $nextpos = nqp::findcclass(
                       nqp::const::CCLASS_WHITESPACE, $str, $pos, $left
                     );
- 
+
                     last unless $left = $chars - $nextpos; # broken word
- 
+
                     emit( nqp::box_s(
                       nqp::substr( $str, $pos, $nextpos - $pos ), Str)
                     );
- 
+
                     $pos = nqp::findnotcclass(
                       nqp::const::CCLASS_WHITESPACE,$str,$nextpos,$left);
                 }
@@ -1035,6 +1062,16 @@ my class Supply {
         }
     }
 
+    method skip(Supply:D: Int(Cool) $number = 1) {
+        supply {
+            my int $size = $number + 1;
+            my int $skipping = $size > 1;
+            whenever self {
+                .emit unless $skipping && ($skipping = --$size)
+            }
+        }
+    }
+
     method min(Supply:D $self: &by = &infix:<cmp>) {
         my &cmp = &by.arity == 2 ?? &by !! { by($^a) cmp by($^b) }
         supply {
@@ -1095,8 +1132,9 @@ my class Supply {
         }
     }
 
-    method reverse(Supply:D:)                 { self.grab( {.reverse} ) }
-    method sort(Supply:D: &by = &infix:<cmp>) { self.grab( {.sort(&by)} ) }
+    method reverse(Supply:D:)        { self.grab( {.reverse} ) }
+    multi method sort(Supply:D:)     { self.grab( {.sort} ) }
+    multi method sort(Supply:D: &by) { self.grab( {.sort(&by)} ) }
 
     method zip(**@s, :&with) {
         @s.unshift(self) if self.DEFINITE;  # add if instance method
@@ -1139,10 +1177,10 @@ my class Supply {
 
         supply {
             my @values;
-    
+
             my $uninitialised = +@s; # how many supplies have yet to emit until we
                                      # can start emitting, too?
-    
+
             if $initial {
                 @values = @$initial;
                 $uninitialised = 0 max $uninitialised - @$initial;
@@ -1325,7 +1363,7 @@ my class Supply {
                     my str $type;
                     my str $value;
                     Rakudo::Internals.KEY_COLON_VALUE(val,$type,$value);
- 
+
                     if $type eq 'limit' {
                         $allowed = $allowed + $value - $limit;
                         $limit   = $value;
@@ -1440,9 +1478,9 @@ my class Supplier {
             }
         }
 
-        method live     { True  }
-        method serial() { False }
-        method sane()   { False }
+        method live(--> True) { }
+        method serial(--> False) { }
+        method sane(--> False)  { }
     }
 
     has $!taplist;
@@ -1496,8 +1534,10 @@ my class Supplier::Preserving is Supplier {
         # need lock on modification).
         has Mu $!tappers;
 
-        # Events to reply, and a lock to protect it.
+        # Events to reply, whether the replay was done, and a lock to protect
+        # updates to these.
         has @!replay;
+        has int $!replay-done;
         has $!replay-lock = Lock.new;
 
         method tap(&emit, &done, &quit) {
@@ -1508,79 +1548,91 @@ my class Supplier::Preserving is Supplier {
                     ?? nqp::clone($!tappers)
                     !! nqp::list();
                 nqp::push($update, $tle);
-                $!tappers := $update;
                 $replay = 1 if nqp::elems($update) == 1;
+                self!replay($tle) if $replay;
+                $!tappers := $update;
             });
-            self!replay($tle) if $replay;
             Tap.new({
                 $!lock.protect({
                     my Mu $update := nqp::list();
                     for nqp::hllize($!tappers) -> \entry {
                         nqp::push($update, entry) unless entry =:= $tle;
                     }
+                    $!replay-done = 0 if nqp::elems($update) == 0;
                     $!tappers := $update;
                 });
             })
         }
 
         method emit(\value --> Nil) {
-            my int $sent = 0;
-            my $snapshot := $!tappers;
-            if nqp::isconcrete($snapshot) {
-                my int $n = nqp::elems($snapshot);
-                loop (my int $i = 0; $i < $n; $i = $i + 1) {
-                    nqp::atpos($snapshot, $i).emit()(value);
-                    $sent = 1;
+            loop {
+                my int $sent = 0;
+                my $snapshot := $!tappers;
+                if nqp::isconcrete($snapshot) {
+                    $sent = nqp::elems($snapshot);
+                    loop (my int $i = 0; $i < $sent; $i = $i + 1) {
+                        nqp::atpos($snapshot, $i).emit()(value);
+                    }
                 }
-            }
-            unless $sent {
-                self!add-replay({ $_.emit()(value) })
+                return if $sent;
+                return if self!add-replay({ $_.emit()(value) });
             }
         }
 
         method done(--> Nil) {
-            my int $sent = 0;
-            my $snapshot := $!tappers;
-            if nqp::isconcrete($snapshot) {
-                my int $n = nqp::elems($snapshot);
-                loop (my int $i = 0; $i < $n; $i = $i + 1) {
-                    nqp::atpos($snapshot, $i).done()();
-                    $sent = 1;
+            loop {
+                my int $sent = 0;
+                my $snapshot := $!tappers;
+                if nqp::isconcrete($snapshot) {
+                    $sent = nqp::elems($snapshot);
+                    loop (my int $i = 0; $i < $sent; $i = $i + 1) {
+                        nqp::atpos($snapshot, $i).done()();
+                    }
                 }
-            }
-            unless $sent {
-                self!add-replay({ $_.done()() })
+                return if $sent;
+                return if self!add-replay({ $_.done()() });
             }
         }
 
         method quit($ex --> Nil) {
-            my int $sent = 0;
-            my $snapshot := $!tappers;
-            if nqp::isconcrete($snapshot) {
-                my int $n = nqp::elems($snapshot);
-                loop (my int $i = 0; $i < $n; $i = $i + 1) {
-                    nqp::atpos($snapshot, $i).quit()($ex);
-                    $sent = 1;
+            loop {
+                my int $sent = 0;
+                my $snapshot := $!tappers;
+                if nqp::isconcrete($snapshot) {
+                    $sent = nqp::elems($snapshot);
+                    loop (my int $i = 0; $i < $sent; $i = $i + 1) {
+                        nqp::atpos($snapshot, $i).quit()($ex);
+                    }
                 }
-            }
-            unless $sent {
-                self!add-replay({ $_.quit()($ex) })
+                return if $sent;
+                return if self!add-replay({ $_.quit()($ex) });
             }
         }
 
-        method !add-replay(&replay) {
-            $!replay-lock.protect: { @!replay.push(&replay) }
+        method !add-replay(&replay --> Bool) {
+            $!replay-lock.protect: {
+                if $!replay-done {
+                    False
+                }
+                else {
+                    @!replay.push(&replay);
+                    True
+                }
+            }
         }
 
         method !replay($tle) {
-            while $!replay-lock.protect({ @!replay.shift }) -> $rep {
-                $rep($tle)
+            $!replay-lock.protect: {
+                while @!replay.shift -> $rep {
+                    $rep($tle);
+                }
+                $!replay-done = 1;
             }
         }
 
-        method live     { True  }
-        method serial() { False }
-        method sane()   { False }
+        method live(--> True) { }
+        method serial(--> False) { }
+        method sane(--> False) { }
     }
 
     method new() {
@@ -1735,7 +1787,8 @@ sub SUPPLY(&block) {
                 nqp::handle(code(),
                     'EMIT', $emitter(),
                     'DONE', $done(),
-                    'CATCH', $catch());
+                    'CATCH', $catch(),
+                    'NEXT', 0);
             });
         }
 
@@ -1755,9 +1808,9 @@ sub SUPPLY(&block) {
             }
         }
 
-        method live { False }
-        method sane { True }
-        method serial { True }
+        method live(--> False) { }
+        method sane(--> True) { }
+        method serial(--> True) { }
     }.new(:&block))
 }
 
