@@ -102,7 +102,7 @@ my class NFD is Uni {
 
 my class NFC is Uni {
     method new(|) {
-        die "Cannot create an NFD directly"; # XXX typed, better message
+        die "Cannot create an NFC directly"; # XXX typed, better message
     }
 
     method NFC() { self }
@@ -110,7 +110,7 @@ my class NFC is Uni {
 
 my class NFKD is Uni {
     method new(|) {
-        die "Cannot create an NFD directly"; # XXX typed, better message
+        die "Cannot create an NFKD directly"; # XXX typed, better message
     }
 
     method NFKD() { self }
@@ -120,6 +120,6 @@ my class NFKC is Uni {
     method NFKC() { self }
 
     method new(|) {
-        die "Cannot create an NFD directly"; # XXX typed, better message
+        die "Cannot create an NFKC directly"; # XXX typed, better message
     }
 }
