@@ -1,14 +1,14 @@
 class X::Cannot::New { ... }
 
 my class Whatever {
-    multi method ACCEPTS(Whatever:D: $topic) { True }
-    multi method perl(Whatever:D:) { '*' }
-    multi method Str(Whatever:D:)  { '*' }
+    multi method ACCEPTS(Whatever:D: $ --> True) { }
+    multi method perl(Whatever:D: --> '*') { }
+    multi method Str(Whatever:D: --> '*') { }
 }
 
 my class HyperWhatever {
     multi method new(HyperWhatever:) { X::Cannot::New.new(class => self).throw }
-    multi method ACCEPTS(HyperWhatever:D: $topic) { True }
+    multi method ACCEPTS(HyperWhatever:D: $ --> True) { }
     multi method perl(HyperWhatever:D:) { '**' }
 }
 
