@@ -5,7 +5,15 @@ my class IntStr is Int is Str {
         nqp::bindattr_s(SELF, Str, '$!value', $s);
         SELF;
     }
-
+    multi method ACCEPTS(IntStr:D: Any:D \a) {
+        nqp::if(
+          nqp::istype(a, Numeric),
+            self.Int.ACCEPTS(a),
+          nqp::if(
+            nqp::istype(a, Str),
+            self.Str.ACCEPTS(a),
+            self.Str.ACCEPTS(a) && self.Int.ACCEPTS(a)))
+    }
     multi method Numeric(IntStr:D:) { self.Int }
     method Int(IntStr:D:) { nqp::add_I(self, 0, Int) }
     multi method Str(IntStr:D:) { nqp::getattr_s(self, Str, '$!value') }
@@ -20,7 +28,15 @@ my class NumStr is Num is Str {
         nqp::bindattr_s(SELF, Str, '$!value', $s);
         SELF;
     }
-
+    multi method ACCEPTS(NumStr:D: Any:D \a) {
+        nqp::if(
+          nqp::istype(a, Numeric),
+          self.Num.ACCEPTS(a),
+          nqp::if(
+            nqp::istype(a, Str),
+            self.Str.ACCEPTS(a),
+            self.Str.ACCEPTS(a) && self.Num.ACCEPTS(a)))
+    }
     multi method Numeric(NumStr:D:) { self.Num }
     method Num(NumStr:D:) { nqp::getattr_n(self, Num, '$!value') }
     multi method Str(NumStr:D:) { nqp::getattr_s(self, Str, '$!value') }
@@ -36,7 +52,15 @@ my class RatStr is Rat is Str {
         nqp::bindattr_s(SELF, Str, '$!value', $s);
         SELF;
     }
-
+    multi method ACCEPTS(RatStr:D: Any:D \a) {
+        nqp::if(
+          nqp::istype(a, Numeric),
+          self.Rat.ACCEPTS(a),
+          nqp::if(
+            nqp::istype(a, Str),
+            self.Str.ACCEPTS(a),
+            self.Str.ACCEPTS(a) && self.Rat.ACCEPTS(a)))
+    }
     multi method Numeric(RatStr:D:) { self.Rat }
     method Rat(RatStr:D:) { Rat.new(nqp::getattr(self, Rat, '$!numerator'), nqp::getattr(self, Rat, '$!denominator')) }
     multi method Str(RatStr:D:) { nqp::getattr_s(self, Str, '$!value') }
@@ -52,7 +76,15 @@ my class ComplexStr is Complex is Str {
         nqp::bindattr_s(SELF, Str, '$!value', $s);
         SELF;
     }
-
+    multi method ACCEPTS(ComplexStr:D: Any:D \a) {
+        nqp::if(
+          nqp::istype(a, Numeric),
+          self.Complex.ACCEPTS(a),
+          nqp::if(
+            nqp::istype(a, Str),
+            self.Str.ACCEPTS(a),
+            self.Str.ACCEPTS(a) && self.Complex.ACCEPTS(a)))
+    }
     multi method Numeric(ComplexStr:D:) { self.Complex }
     method Complex(ComplexStr:D:) { Complex.new(nqp::getattr_n(self, Complex, '$!re'), nqp::getattr_n(self, Complex, '$!im')) }
     multi method Str(ComplexStr:D:) { nqp::getattr_s(self, Str, '$!value') }
@@ -62,31 +94,46 @@ my class ComplexStr is Complex is Str {
 
 # we define cmp ops for these allomorphic types as numeric first, then Str. If
 # you want just one half of the cmp, you'll need to coerce the args
-multi sub infix:<cmp>(IntStr:D $a, IntStr:D $b) {
-    $a.Int cmp $b.Int || $a.Str cmp $b.Str
-}
-multi sub infix:<cmp>(RatStr:D $a, RatStr:D $b) {
-    $a.Rat cmp $b.Rat || $a.Str cmp $b.Str
-}
-multi sub infix:<cmp>(NumStr:D $a, NumStr:D $b) {
-    $a.Num cmp $b.Num || $a.Str cmp $b.Str
-}
-multi sub infix:<cmp>(ComplexStr:D $a, ComplexStr:D $b) {
-    $a.Complex cmp $b.Complex || $a.Str cmp $b.Str
-}
+multi sub infix:<cmp>(IntStr:D     $a, IntStr:D     $b) { $a.Int     cmp $b.Int     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(IntStr:D     $a, RatStr:D     $b) { $a.Int     cmp $b.Rat     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(IntStr:D     $a, NumStr:D     $b) { $a.Int     cmp $b.Num     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(IntStr:D     $a, ComplexStr:D $b) { $a.Int     cmp $b.Complex || $a.Str cmp $b.Str }
 
-multi sub infix:<eqv>(IntStr:D $a, IntStr:D $b) {
-    $a.Int eqv $b.Int || $a.Str eqv $b.Str
-}
-multi sub infix:<eqv>(RatStr:D $a, RatStr:D $b) {
-    $a.Rat eqv $b.Rat || $a.Str eqv $b.Str
-}
-multi sub infix:<eqv>(NumStr:D $a, NumStr:D $b) {
-    $a.Num eqv $b.Num || $a.Str eqv $b.Str
-}
-multi sub infix:<eqv>(ComplexStr:D $a, ComplexStr:D $b) {
-    $a.Complex eqv $b.Complex || $a.Str eqv $b.Str
-}
+multi sub infix:<cmp>(RatStr:D     $a, IntStr:D     $b) { $a.Rat     cmp $b.Int     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(RatStr:D     $a, RatStr:D     $b) { $a.Rat     cmp $b.Rat     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(RatStr:D     $a, NumStr:D     $b) { $a.Rat     cmp $b.Num     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(RatStr:D     $a, ComplexStr:D $b) { $a.Rat     cmp $b.Complex || $a.Str cmp $b.Str }
+
+multi sub infix:<cmp>(NumStr:D     $a, IntStr:D     $b) { $a.Num     cmp $b.Int     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(NumStr:D     $a, RatStr:D     $b) { $a.Num     cmp $b.Rat     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(NumStr:D     $a, NumStr:D     $b) { $a.Num     cmp $b.Num     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(NumStr:D     $a, ComplexStr:D $b) { $a.Num     cmp $b.Complex || $a.Str cmp $b.Str }
+
+multi sub infix:<cmp>(ComplexStr:D $a, IntStr:D     $b) { $a.Complex cmp $b.Int     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(ComplexStr:D $a, RatStr:D     $b) { $a.Complex cmp $b.Rat     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(ComplexStr:D $a, NumStr:D     $b) { $a.Complex cmp $b.Num     || $a.Str cmp $b.Str }
+multi sub infix:<cmp>(ComplexStr:D $a, ComplexStr:D $b) { $a.Complex cmp $b.Complex || $a.Str cmp $b.Str }
+
+
+multi sub infix:<eqv>(IntStr:D     $a, IntStr:D     $b) { $a.Int     eqv $b.Int     && $a.Str eqv $b.Str }
+multi sub infix:<eqv>(IntStr:D     $a, RatStr:D     $b --> False) {}
+multi sub infix:<eqv>(IntStr:D     $a, NumStr:D     $b --> False) {}
+multi sub infix:<eqv>(IntStr:D     $a, ComplexStr:D $b --> False) {}
+
+multi sub infix:<eqv>(RatStr:D     $a, IntStr:D     $b --> False) {}
+multi sub infix:<eqv>(RatStr:D     $a, RatStr:D     $b) { $a.Rat     eqv $b.Rat     && $a.Str eqv $b.Str }
+multi sub infix:<eqv>(RatStr:D     $a, NumStr:D     $b --> False) {}
+multi sub infix:<eqv>(RatStr:D     $a, ComplexStr:D $b --> False) {}
+
+multi sub infix:<eqv>(NumStr:D     $a, IntStr:D     $b --> False) {}
+multi sub infix:<eqv>(NumStr:D     $a, RatStr:D     $b --> False) {}
+multi sub infix:<eqv>(NumStr:D     $a, NumStr:D     $b) { $a.Num     eqv $b.Num     && $a.Str eqv $b.Str }
+multi sub infix:<eqv>(NumStr:D     $a, ComplexStr:D $b --> False) {}
+
+multi sub infix:<eqv>(ComplexStr:D $a, IntStr:D     $b --> False) {}
+multi sub infix:<eqv>(ComplexStr:D $a, RatStr:D     $b --> False) {}
+multi sub infix:<eqv>(ComplexStr:D $a, NumStr:D     $b --> False) {}
+multi sub infix:<eqv>(ComplexStr:D $a, ComplexStr:D $b) { $a.Complex eqv $b.Complex && $a.Str eqv $b.Str }
 
 multi sub infix:<===>(IntStr:D $a, IntStr:D $b) {
     $a.Int === $b.Int && $a.Str === $b.Str
