@@ -44,6 +44,13 @@ multi sub infix:<does>(Mu:U \obj, **@roles) is raw {
     X::Does::TypeObject.new(type => obj).throw
 }
 
+# we need this candidate tighter than infix:<cmp>(Real:D, Real:D)
+# but can't yet use `is default` at the place where that candidate
+# is defined because it uses `infix:<does>`
+multi sub infix:<cmp>(Rational:D \a, Rational:D \b) is default {
+    a.isNaN || b.isNaN ?? a.Num cmp b.Num !! a <=> b
+}
+
 proto sub infix:<but>(|) is pure { * }
 multi sub infix:<but>(Mu:D \obj, Mu:U \rolish) {
     my $role := rolish.HOW.archetypes.composable() ?? rolish !!
@@ -207,11 +214,11 @@ sub SEQUENCE(\left, Mu \right, :$exclude_end) {
             if $code.defined { }
             elsif @tail.grep(Real).elems != @tail.elems {
                 if @tail.elems > 1 {
-                    if @tail[*-1].WHAT === $endpoint.WHAT {
-                        $code = succpred(@tail[*-1], $endpoint);
+                    if @tail.tail.WHAT === $endpoint.WHAT {
+                        $code = succpred(@tail.tail, $endpoint);
                     }
                     else {
-                        $code = succpred(@tail[*-2], @tail[*-1]);
+                        $code = succpred(@tail[*-2], @tail.tail);
                     }
                 }
                 elsif nqp::istype($endpoint, Stringy) and nqp::istype($a, Stringy) and nqp::isconcrete($endpoint) {
@@ -545,7 +552,7 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
         nqp::split('::',my str $name = @chunks.join('::'))),
       nqp::stmts(
         (my str $first = nqp::shift($parts)),
-        nqp::if(
+        nqp::if( # move the sigil to the last part of the name if available
           nqp::elems($parts),
           nqp::stmts(
             (my str $sigil = nqp::substr($first,0,1)),
@@ -570,14 +577,19 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
           )
         ),
         (my Mu $thing := nqp::if(
-          $root.EXISTS-KEY($first),
-          $root.AT-KEY($first),
+          $root.EXISTS-KEY('%REQUIRE_SYMBOLS')
+            && (my $REQUIRE_SYMBOLS := $root.AT-KEY('%REQUIRE_SYMBOLS'))
+            && ($REQUIRE_SYMBOLS{$first}:exists),
+          $REQUIRE_SYMBOLS{$first},
           nqp::if(
-            GLOBAL::.EXISTS-KEY($first),
-            GLOBAL::.AT-KEY($first),
-            X::NoSuchSymbol.new(symbol => $name).fail
-          )
-        )),
+            $root.EXISTS-KEY($first),
+            $root.AT-KEY($first),
+            nqp::if(
+              GLOBAL::.EXISTS-KEY($first),
+              GLOBAL::.AT-KEY($first),
+              X::NoSuchSymbol.new(symbol => $name).fail
+            )
+          ))),
         nqp::while(
           nqp::elems($parts),
           nqp::if(
@@ -592,88 +604,147 @@ sub INDIRECT_NAME_LOOKUP($root, *@chunks) is raw {
     )
 }
 
-sub REQUIRE_IMPORT($compunit, *@syms) {
+sub REQUIRE_IMPORT($compunit, $existing-path,$top-existing-pkg,$stubname, *@syms --> Nil) {
     my $handle := $compunit.handle;
     my $DEFAULT := $handle.export-package()<DEFAULT>.WHO;
-    my $GLOBALish := $handle.globalish-package.WHO;
+    my $GLOBALish := $handle.globalish-package;
     my @missing;
+    my $block := CALLER::.EXISTS-KEY('%REQUIRE_SYMBOLS')
+        ?? CALLER::MY::
+        !! CALLER::OUTER::;
+
+    my $targetWHO;
+    my $sourceWHO;
+    if $existing-path {
+        my @existing-path = @$existing-path;
+        my $topname := @existing-path.shift;
+        $targetWHO := $top-existing-pkg.WHO;
+        $sourceWHO := $GLOBALish.AT-KEY($topname).WHO;
+        # Yes! the target CAN be the source if it's something like Cool::Utils
+        # because Cool is common to both compunits..so no need to do anything
+        unless $targetWHO === $sourceWHO {
+            # We want to skip over the parts of the Package::That::Already::Existed
+            for @existing-path {
+                $targetWHO := $targetWHO.AT-KEY($_).WHO;
+                $sourceWHO := $sourceWHO.AT-KEY($_).WHO;
+            }
+            # Now we are just above our target stub. If it exists
+            # delete it so it can be replaced by the real one we're importing.
+            if $stubname {
+                $targetWHO.DELETE-KEY($stubname);
+            }
+            $targetWHO.merge-symbols($sourceWHO);
+        }
+    } elsif $stubname {
+        $targetWHO := $block.AT-KEY($stubname).WHO;
+        $sourceWHO := $GLOBALish.AT-KEY($stubname).WHO;
+        $targetWHO.merge-symbols($sourceWHO);
+    }
     # Set the runtime values for compile time stub symbols
     for @syms {
         unless $DEFAULT.EXISTS-KEY($_) {
             @missing.push: $_;
             next;
         }
-        OUTER::CALLER::{$_} := $DEFAULT{$_};
+        $block{$_} := $DEFAULT{$_};
     }
     if @missing {
         X::Import::MissingSymbols.new(:from($compunit.short-name), :@missing).throw;
     }
     # Merge GLOBAL from compunit.
-    GLOBAL::.merge-symbols($GLOBALish);
-    Nil;
+    nqp::gethllsym('perl6','ModuleLoader').merge_globals(
+        $block<%REQUIRE_SYMBOLS>,
+        $GLOBALish,
+    );
 }
 
 sub infix:<andthen>(+a) {
-    my $ai := a.iterator;
-    my Mu $current := $ai.pull-one;
-    return Bool::True if $current =:= IterationEnd;
-    nqp::until(
-        (($_ := $ai.pull-one) =:= IterationEnd),
-        nqp::stmts(
-            (return Empty unless $current.defined),
-            ($current := $_ ~~ Callable
-                ?? (.count ?? $_($current) !! $_())
-                !! $_
-            ),
-        ),
-        :nohandler, # do not handle control stuff in thunks
-    );
-    $current;
+    # We need to be able to process `Empty` in our args, which we can get
+    # when we're chained with, say, `andthen`. Since Empty disappears in normal
+    # arg handling, we use nqp::p6argvmarray op to fetch the args, and then
+    # emulate the `+@foo` slurpy by inspecting the list the op gave us.
+    nqp::if(
+      (my int $els = nqp::elems(my $args := nqp::p6argvmarray)),
+      nqp::stmts(
+        (my $current := nqp::atpos($args, 0)),
+        nqp::if( # emulate the +@foo slurpy
+          nqp::iseq_i($els, 1) && nqp::istype($current, Iterable),
+          nqp::stmts(
+            ($args := $current.List),
+            ($current := $args[0]),
+            $els = $args.elems)),
+        (my int $i),
+        nqp::until(
+          nqp::iseq_i($els, $i = nqp::add_i($i, 1))
+          || ( # if $current not defined, set it to Empty and bail from the loop
+            nqp::isfalse($current.defined)
+            && nqp::stmts(($current := Empty), 1)
+          ),
+          ($current := nqp::if(
+            nqp::istype(($_ := $args[$i]), Callable),
+            nqp::if(.count, $_($current), $_()),
+            $_)),
+          :nohandler), # do not handle control stuff in thunks
+        $current), # either the last arg or Empty if any but last were undefined
+      True) # We were given no args, return True
 }
+
 sub infix:<notandthen>(+a) {
-    my $ai := a.iterator;
-    my Mu $current := $ai.pull-one;
-    return Bool::True if $current =:= IterationEnd;
-    nqp::until(
-        (($_ := $ai.pull-one) =:= IterationEnd),
-        nqp::stmts(
-            (return Empty if $current.defined),
-            ($current := $_ ~~ Callable
-                ?? (.count ?? $_($current) !! $_())
-                !! $_
-            ),
-        ),
-        :nohandler, # do not handle control stuff in thunks
-    );
-    $current;
+    # We need to be able to process `Empty` in our args, which we can get
+    # when we're chained with, say, `andthen`. Since Empty disappears in normal
+    # arg handling, we use nqp::p6argvmarray op to fetch the args, and then
+    # emulate the `+@foo` slurpy by inspecting the list the op gave us.
+    nqp::if(
+      (my int $els = nqp::elems(my $args := nqp::p6argvmarray)),
+      nqp::stmts(
+        (my $current := nqp::atpos($args, 0)),
+        nqp::if( # emulate the +@foo slurpy
+          nqp::iseq_i($els, 1) && nqp::istype($current, Iterable),
+          nqp::stmts(
+            ($args := $current.List),
+            ($current := $args[0]),
+            $els = $args.elems)),
+        (my int $i),
+        nqp::until(
+          nqp::iseq_i($els, $i = nqp::add_i($i, 1))
+          || ( # if $current is defined, set it to Empty and bail from the loop
+            $current.defined
+            && nqp::stmts(($current := Empty), 1)
+          ),
+          ($current := nqp::if(
+            nqp::istype(($_ := $args[$i]), Callable),
+            nqp::if(.count, $_($current), $_()),
+            $_)),
+          :nohandler), # do not handle control stuff in thunks
+        $current), # either the last arg or Empty if any but last were undefined
+      True) # We were given no args, return True
 }
 
-sub infix:<orelse>(+a) {
-    my $ai := a.iterator;
-    my Mu $current := $ai.pull-one;
-    return Nil if $current =:= IterationEnd;
-
-    # Flag for heuristic when we were passed an Empty as LHS
-    my int $handle-empty = 1;
-    nqp::until(
-        (($_ := $ai.pull-one) =:= IterationEnd),
-        nqp::stmts(
-            (return $current if $current.defined),
-            ($handle-empty = 0),
-            ($current := $_ ~~ Callable
-                ?? (.count ?? $_($current) !! $_())
-                !! $_
-            ),
-        ),
-        :nohandler, # do not handle control stuff in thunks
-    );
-
-    if $handle-empty and $current ~~ Callable {
-        $_ := Empty; # set $_ in the Callable
-        $current := $current.count ?? $current($_) !! $current();
-    }
-
-    $current;
+sub infix:<orelse>(+$) {
+    # We need to be able to process `Empty` in our args, which we can get
+    # when we're chained with, say, `andthen`. Since Empty disappears in normal
+    # arg handling, we use nqp::p6argvmarray op to fetch the args, and then
+    # emulate the `+@foo` slurpy by inspecting the list the op gave us.
+    nqp::if(
+      (my int $els = nqp::elems(my $args := nqp::p6argvmarray)),
+      nqp::stmts(
+        (my $current := nqp::atpos($args, 0)),
+        nqp::if( # emulate the +@foo slurpy
+          nqp::iseq_i($els, 1) && nqp::istype($current, Iterable),
+          nqp::stmts(
+            ($args := $current.List),
+            ($current := $args[0]),
+            $els = $args.elems)),
+        (my int $i),
+        nqp::until(
+          nqp::iseq_i($els, $i = nqp::add_i($i, 1)) || $current.defined,
+          ($current := nqp::if(
+            nqp::istype(($_ := $args[$i]), Callable),
+            nqp::if(.count, $_($current), $_()),
+            $_)),
+          :nohandler), # do not handle control stuff in thunks
+        $current),
+      Nil) # We were given no args, return Nil
 }
 
 # next three sub would belong to traits.pm if PseudoStash were available
@@ -708,7 +779,19 @@ multi sub trait_mod:<is>(Routine $r, Str :$looser!) {
 proto sub infix:<∘> (&?, &?) {*}
 multi sub infix:<∘> () { *.self }
 multi sub infix:<∘> (&f) { &f }
-multi sub infix:<∘> (&f, &g --> Block) { (&f).count > 1 ?? -> |args { f |g |args } !! -> |args { f g |args } }
+multi sub infix:<∘> (&f, &g --> Block:D) {
+    my \ret = &f.count > 1
+        ?? -> |args { f |g |args }
+        !! -> |args { f  g |args }
+
+    my role FakeSignature[$arity, $count, $of] {
+        method arity { $arity }
+        method count { $count }
+        method of    { $of    }
+    }
+    ret.^mixin(FakeSignature[&g.arity, &g.count, &f.of]);
+    ret
+}
 my &infix:<o> := &infix:<∘>;
 
 # vim: ft=perl6 expandtab sw=4
