@@ -16,7 +16,18 @@ multi sub trait_mod:<is>(Mu:U $child, Mu:U $parent) {
         $child.^add_parent($parent);
     }
     elsif $parent.HOW.archetypes.inheritalizable() {
-        $child.^add_parent($parent.^inheritalize)
+        if my @required-methods = $parent.^methods.grep({$_.yada}) {
+            my $type = $child.HOW.archetypes.inheritable()
+                ?? 'Class '
+                !! $child.HOW.archetypes.inheritalizable()
+                    ?? 'Role '
+                    !! '';
+            die $type ~ "{$child.^name} can't pun role {$parent.^name} because it has required methods: "
+                ~ @required-methods.map({$_.name}).join(', ') ~ '. Did you mean to use "does" instead?';
+        }
+        else {
+            $child.^add_parent($parent.^inheritalize)
+        }
     }
     else {
         X::Inheritance::Unsupported.new(
@@ -129,7 +140,7 @@ multi sub trait_mod:<is>(Routine:D $r, :$raw!) {
     $r.set_rw(); # for now, until we have real raw handling
 }
 multi sub trait_mod:<is>(Routine:D $r, :$default!) {
-    $r does role { method default() { True } }
+    $r does role { method default(--> True) { } }
 }
 multi sub trait_mod:<is>(Routine:D $r, :$DEPRECATED!) {
     my $new := nqp::istype($DEPRECATED,Bool)
@@ -145,7 +156,15 @@ multi sub trait_mod:<is>(Routine:D $r, :$onlystar!) {
 }
 multi sub trait_mod:<is>(Routine:D $r, :prec(%spec)!) {
     my role Precedence {
-        has %.prec;
+        has %!prec;
+        proto method prec(|) { * }
+        multi method prec() is raw { %!prec }
+        multi method prec(Str:D $key) {
+            nqp::ifnull(
+              nqp::atkey(nqp::getattr(%!prec,Map,'$!storage'),$key),
+              ''
+            )
+        }
     }
     if nqp::istype($r, Precedence) {
         for %spec {
@@ -163,6 +182,7 @@ multi sub trait_mod:<is>(Routine $r, :&equiv!) {
     nqp::can(&equiv, 'prec')
         ?? trait_mod:<is>($r, :prec(&equiv.prec))
         !! die "Routine given to equiv does not appear to be an operator";
+    $r.prec<assoc>:delete;
 }
 multi sub trait_mod:<is>(Routine $r, :&tighter!) {
     die "Routine given to tighter does not appear to be an operator"
@@ -317,24 +337,24 @@ multi sub trait_mod:<of>(Routine:D $target, Mu:U $type) {
 }
 
 multi sub trait_mod:<is>(Routine:D $r, :$hidden-from-backtrace!) {
-    $r.^mixin( role { method is-hidden-from-backtrace { True } } );
+    $r.^mixin( role { method is-hidden-from-backtrace(--> True) { } } );
 }
 
 multi sub trait_mod:<is>(Routine:D $r, :$hidden-from-USAGE!) {
     $r.^mixin( role {
-        method is-hidden-from-USAGE { True }
+        method is-hidden-from-USAGE(--> True) { }
     });
 }
 
 multi sub trait_mod:<is>(Routine:D $r, :$pure!) {
     $r.^mixin( role {
-        method IS_PURE { True }
+        method IS_PURE(--> True) { }
     });
 }
 
 multi sub trait_mod:<is>(Routine:D $r, :$nodal!) {
     $r.^mixin( role {
-        method nodal { True }
+        method nodal(--> True) { }
     });
 }
 
