@@ -136,7 +136,8 @@ public final class Binder {
         long elems = typeCaps.elems(tc);
         StaticCodeInfo sci = cf.codeRef.staticInfo;
         for (long i = 0; i < elems; i++) {
-            String name = typeCaps.at_pos_boxed(tc, i).get_str(tc);
+            typeCaps.at_pos_native(tc, i);
+            String name = tc.native_s;
             cf.oLex[sci.oTryGetLexicalIdx(name)] = type;
         }
     }
@@ -225,8 +226,10 @@ public final class Binder {
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_NUM }, null);
     private static final CallSiteDescriptor ACCEPTS_s = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_STR }, null);
-    private static final CallSiteDescriptor bindThrower = new CallSiteDescriptor(
-        new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_STR }, null);
+    private static final CallSiteDescriptor bindParamThrower = new CallSiteDescriptor(
+        new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ,
+            CallSiteDescriptor.ARG_STR, CallSiteDescriptor.ARG_OBJ
+        }, null);
     private static int bindOneParam(ThreadContext tc, RakOps.GlobalExt gcx, CallFrame cf, SixModelObject param,
             Object origArg, byte origFlag, boolean noNomTypeCheck, Object[] error) {
         /* Get parameter flags and variable name. */
@@ -381,8 +384,8 @@ public final class Binder {
                     "$!nominal_type", HINT_nominal_type);
                 if ((paramFlags & SIG_ELEM_NOMINAL_GENERIC) != 0) {
                     SixModelObject HOW = nomType.st.HOW;
-                    SixModelObject ig = Ops.findmethod(tc, HOW,
-                        "instantiate_generic");
+                    SixModelObject ig = Ops.findmethod(HOW,
+                        "instantiate_generic", tc);
                     SixModelObject ContextRef = tc.gc.ContextRef;
                     SixModelObject cc = ContextRef.st.REPR.allocate(tc, ContextRef.st);
                     ((ContextRefInstance)cc).context = cf;
@@ -395,13 +398,13 @@ public final class Binder {
                  * positional bind failover. */
                 if (nomType == gcx.Positional) {
                     if (Ops.istype_nodecont(arg_o, gcx.PositionalBindFailover, tc) != 0) {
-                        SixModelObject ig = Ops.findmethod(tc, arg_o, "cache");
+                        SixModelObject ig = Ops.findmethod(arg_o, "cache", tc);
                         Ops.invokeDirect(tc, ig, Ops.invocantCallSite, new Object[] { arg_o });
                         arg_o = Ops.result_o(tc.curFrame);
                         decontValue = Ops.decont(arg_o, tc);
                     }
                     else if (Ops.istype_nodecont(decontValue, gcx.PositionalBindFailover, tc) != 0) {
-                        SixModelObject ig = Ops.findmethod(tc, decontValue, "cache");
+                        SixModelObject ig = Ops.findmethod(decontValue, "cache", tc);
                         Ops.invokeDirect(tc, ig, Ops.invocantCallSite, new Object[] { decontValue });
                         decontValue = Ops.result_o(tc.curFrame);
                     }
@@ -413,11 +416,12 @@ public final class Binder {
                     /* Type check failed; produce error if needed. */
                     if (error != null) {
 
-                        SixModelObject thrower = RakOps.getThrower(tc, "X::TypeCheck::Binding");
+                        SixModelObject thrower = RakOps.getThrower(tc, "X::TypeCheck::Binding::Parameter");
                         if (thrower != null) {
                             error[0] = thrower;
-                            error[1] = bindThrower;
-                            error[2] = new Object[] { decontValue, nomType.st.WHAT, (varName != null ? varName : "<anon>") };
+                            error[1] = bindParamThrower;
+                            error[2] = new Object[] { decontValue, nomType.st.WHAT,
+                                (varName != null ? varName : "<anon>"), param };
                         }
                         else {
                             error[0] = String.format(
@@ -488,8 +492,7 @@ public final class Binder {
             if (Ops.istype(decontValue, coerceType, tc) == 0) {
                 param.get_attribute_native(tc, gcx.Parameter, "$!coerce_method", HINT_coerce_method);
                 String methName = tc.native_s;
-                SixModelObject coerceMeth = Ops.findmethod(tc,
-                    decontValue, methName);
+                SixModelObject coerceMeth = Ops.findmethodNonFatal(decontValue, methName, tc);
                 if (coerceMeth != null) {
                     Ops.invokeDirect(tc, coerceMeth,
                         Ops.invocantCallSite,
@@ -661,7 +664,7 @@ public final class Binder {
                 capture = decontValue;
             }
             else {
-                SixModelObject meth = Ops.findmethod(decontValue, "Capture", tc);
+                SixModelObject meth = Ops.findmethodNonFatal(decontValue, "Capture", tc);
                 if (meth == null) {
                     if (error != null)
                         error[0] = "Could not turn argument into capture";
@@ -893,9 +896,10 @@ public final class Binder {
                     }
 
                     SixModelObject slurpyType = (flags & SIG_ELEM_IS_RAW) != 0 ? gcx.List : gcx.Array;
-                    SixModelObject sm = Ops.findmethod(tc, slurpyType,
+                    SixModelObject sm = Ops.findmethod(slurpyType,
                         (flags & SIG_ELEM_SLURPY_ONEARG) != 0 ? "from-slurpy-onearg" :
-                        (flags & SIG_ELEM_SLURPY_POS) != 0 ? "from-slurpy-flat" : "from-slurpy");
+                        (flags & SIG_ELEM_SLURPY_POS) != 0 ? "from-slurpy-flat" : "from-slurpy",
+                        tc);
                     Ops.invokeDirect(tc, sm, slurpyFromArgs, new Object[] { slurpyType, slurpy });
                     SixModelObject bindee = Ops.result_o(tc.curFrame);
 
@@ -943,7 +947,8 @@ public final class Binder {
                 if (namedArgsCopy != null) {
                     long numNames = namedNames.elems(tc);
                     for (long j = 0; j < numNames; j++) {
-                        String name = namedNames.at_pos_boxed(tc, j).get_str(tc);
+                        namedNames.at_pos_native(tc, j);
+                        String name = tc.native_s;
                         lookup = namedArgsCopy.remove(name);
                         if (lookup != null)
                             break;
@@ -959,10 +964,12 @@ public final class Binder {
                             CallSiteDescriptor.ARG_OBJ, false, error);
                     }
                     else if (!suppressArityFail) {
-                        if (error != null)
+                        if (error != null) {
+                            namedNames.at_pos_native(tc, 0);
                             error[0] = "Required named argument '" +
-                                namedNames.at_pos_boxed(tc, 0).get_str(tc) +
+                                tc.native_s +
                                 "' not passed";
+                        }
                         return BIND_RESULT_FAIL;
                     }
                 }
