@@ -78,15 +78,16 @@ static void rakudo_scalar_store(MVMThreadContext *tc, MVMObject *cont, MVMObject
     /* Check it's an assignable container. */
     if (rcd && IS_CONCRETE(rcd))
         rw = rcd->rw;
-    if (!rw)
+    if (!rw) {
         if (rcd && IS_CONCRETE(rcd) && rcd->name) {
             char *c_name = MVM_string_utf8_encode_C_string(tc, rcd->name);
             char *waste[] = { c_name, NULL };
             MVM_exception_throw_adhoc_free(tc, waste, "Cannot assign to a readonly variable (%s) or a value", c_name);
         }
-        else
+        else {
             MVM_exception_throw_adhoc(tc, "Cannot assign to a readonly variable or a value");
-
+        }
+    }
     /* Handle Nil and type-checking. */
     if (!obj) {
         MVM_exception_throw_adhoc(tc, "Cannot assign a null value to a Perl 6 scalar");
