@@ -263,7 +263,7 @@ static MVMuint8 s_p6recont_ro[] = {
 };
 static void p6recont_ro(MVMThreadContext *tc, MVMuint8 *cur_op) {
     MVMObject *check = GET_REG(tc, 2).o;
-    if (STABLE(check)->container_spec == Rakudo_containers_get_scalar()) {
+    if (IS_CONCRETE(check) && STABLE(check)->container_spec == Rakudo_containers_get_scalar()) {
         MVMObject *desc = ((Rakudo_Scalar *)check)->descriptor;
         if (!MVM_is_null(tc, desc) && ((Rakudo_ContainerDescriptor *)desc)->rw) {
             /* We have an rw container; re-containerize it. */
@@ -366,8 +366,12 @@ static void p6decontrv(MVMThreadContext *tc, MVMuint8 *cur_op) {
 static void p6decontrv_spesh(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins) {
     /* If it's already deconted, can just become a set. */
     MVMSpeshFacts *obj_facts = MVM_spesh_get_and_use_facts(tc, g, ins->operands[1]);
-    if (obj_facts->flags & (MVM_SPESH_FACT_DECONTED | MVM_SPESH_FACT_TYPEOBJ))
+    if (obj_facts->flags & (MVM_SPESH_FACT_DECONTED | MVM_SPESH_FACT_TYPEOBJ)) {
+        MVMSpeshFacts *res_facts = MVM_spesh_get_facts(tc, g, ins->operands[0]);
         ins->info = MVM_op_get_op(MVM_OP_set);
+        res_facts->flags = obj_facts->flags;
+        res_facts->type = obj_facts->type;
+    }
 }
 
 static MVMuint8 s_p6capturelex[] = {
@@ -544,7 +548,10 @@ static void p6finddispatcher(MVMThreadContext *tc, MVMuint8 *cur_op) {
     ctx = tc->cur_frame->caller; /* Skip over routine using this op. */
     while (ctx) {
         /* Do we have a dispatcher here? */
-        MVMRegister *disp_lex = MVM_frame_try_get_lexical(tc, ctx, str_dispatcher, MVM_reg_obj);
+        MVMRegister *disp_lex;
+        MVMROOT(tc, ctx, {
+            disp_lex = MVM_frame_try_get_lexical(tc, ctx, str_dispatcher, MVM_reg_obj);
+        });
         if (disp_lex) {
             MVMObject *maybe_dispatcher = disp_lex->o;
             if (!MVM_is_null(tc, maybe_dispatcher)) {
@@ -561,14 +568,18 @@ static void p6finddispatcher(MVMThreadContext *tc, MVMuint8 *cur_op) {
                                 ((MVMContext *)ctx_ref)->body.context, ctx);
                         MVMROOT(tc, ctx_ref, {
                             capture = MVM_args_use_capture(tc, ctx);
+                            MVMROOT(tc, capture, {
+                                p6sub = MVM_frame_get_code_object(tc, (MVMCode *)ctx->code_ref);
+                                MVMROOT(tc, p6sub, {
+                                    meth = MVM_6model_find_method_cache_only(tc, dispatcher, str_vivify_for);
+                                });
+                            });
                         });
                     });
                     });
-                    p6sub = MVM_frame_get_code_object(tc, (MVMCode *)ctx->code_ref);
 
                     /* Lookup method, invoke it, and set up callback to ensure it
                      * is also stored in the lexical. */
-                    meth = MVM_6model_find_method_cache_only(tc, dispatcher, str_vivify_for);
                     meth = MVM_frame_find_invokee(tc, meth, NULL);
                     *(tc->interp_cur_op) += 4; /* Get right return address. */
                     MVM_args_setup_thunk(tc, res_reg, MVM_RETURN_OBJ, &disp_callsite);
